@@ -1,13 +1,17 @@
 <?php
 
-$block_name = _("weather.com");
+/* Disable block if not configured. */
+if (!empty($GLOBALS['conf']['weatherdotcom']['partner_id']) &&
+    !empty($GLOBALS['conf']['weatherdotcom']['license_key'])) {
+    $block_name = _("weather.com");
+}
 
 /**
  * The Horde_Block_weatherdotcom class provides an applet for the
  * portal screen to display weather and forecast data from weather.com
  * for a specified location.
  *
- * $Horde: horde/lib/Block/weatherdotcom.php,v 1.37.4.3 2005/03/16 13:26:31 jan Exp $
+ * $Horde: horde/lib/Block/weatherdotcom.php,v 1.37.4.8 2005/09/30 03:12:01 selsky Exp $
  *
  * @package Horde_Block
  */
@@ -140,19 +144,44 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
             $matches[0] :
             $weatherDotCom->searchLocation($this->_params['location']));
         if (is_a($search, 'PEAR_Error')) {
-            return $search->getmessage();
+            switch ($search->getCode()) {
+            case SERVICES_WEATHER_ERROR_SERVICE_NOT_FOUND:
+                return _("Requested service could not be found.");
+            case SERVICES_WEATHER_ERROR_UNKNOWN_LOCATION:
+                return _("Unknown location provided.");
+            case SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA:
+                return _("Server data wrong or not available.");
+            case SERVICES_WEATHER_ERROR_CACHE_INIT_FAILED:
+                return _("Cache init was not completed.");
+            case SERVICES_WEATHER_ERROR_DB_NOT_CONNECTED:
+                return _("MetarDB is not connected.");
+            case SERVICES_WEATHER_ERROR_UNKNOWN_ERROR:
+                return _("An unknown error has occured.");
+            case SERVICES_WEATHER_ERROR_NO_LOCATION:
+                return _("No location provided.");
+            case SERVICES_WEATHER_ERROR_INVALID_LOCATION:
+                return _("Invalid location provided.");
+            case SERVICES_WEATHER_ERROR_INVALID_PARTNER_ID:
+                return _("Invalid partner id.");
+            case SERVICES_WEATHER_ERROR_INVALID_PRODUCT_CODE:
+                return _("Invalid product code.");
+            case SERVICES_WEATHER_ERROR_INVALID_LICENSE_KEY:
+                return _("Invalid license key.");
+            default:
+                return $search->getmessage();
+            }
         }
 
         if (is_array($search)) {
             // Several locations returned due to imprecise location
             // parameter.
-            $html = _("Several locations possible with the parameter: ");
-            $html .= $this->_params['location'];
-            $html .= "<br/><ul>";
+            $html = _("Several locations possible with the parameter: ") .
+                $this->_params['location'] .
+                '<br /><ul>';
             foreach ($search as $id_weather=>$real_location) {
                 $html .= "<li>$real_location ($id_weather)</li>\n";
             }
-            $html .= "</ul>";
+            $html .= '</ul>';
             return $html;
         }
 
@@ -170,9 +199,9 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
         }
 
         // Location and local time.
-        $html .= '<div class="control">';
-        $html .= '<b>' . $location['name'] . '</b> ' . _("Local time: ") . $location['time'];
-        $html .= '</div>';
+        $html .= '<div class="control">' .
+            '<b>' . $location['name'] . '</b> ' . _("Local time: ") . $location['time'] .
+            '</div>';
 
         // Sunrise/sunset.
         $html .= '<b>' . _("Sunrise: ") . '</b>' .
@@ -183,16 +212,16 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
             $location['sunset'];
 
         // Temperature.
-        $html .= '<br /><b>' . _("Temperature: ") . '</b>';
-        $html .= round($weather['temperature']) . '&deg;' . String::upper($units['temp']);
+        $html .= '<br /><b>' . _("Temperature: ") . '</b>' .
+            round($weather['temperature']) . '&deg;' . String::upper($units['temp']);
 
         // Dew point.
-        $html .= ' <b>' . _("Dew point: ") . '</b>';
-        $html .= round($weather['dewPoint']) . '&deg;' . String::upper($units['temp']);
+        $html .= ' <b>' . _("Dew point: ") . '</b>' .
+            round($weather['dewPoint']) . '&deg;' . String::upper($units['temp']);
 
         // Feels like temperature.
-        $html .= ' <b>' . _("Feels like: ") . '</b>';
-        $html .= round($weather['feltTemperature']) . '&deg;' . String::upper($units['temp']);
+        $html .= ' <b>' . _("Feels like: ") . '</b>' .
+            round($weather['feltTemperature']) . '&deg;' . String::upper($units['temp']);
 
         // Pressure and trend.
         $html .= '<br /><b>' . _("Pressure: ") . '</b>';
@@ -208,20 +237,20 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
         } elseif ($weather['windDirection'] == 'CALM') {
             $html .= _("Calm");
         } else {
-            $html .= _("From the ") . $weather['windDirection'];
-            $html .= ' (' . $weather['windDegrees'] . ')';
+            $html .= _("From the ") . $weather['windDirection'] .
+                ' (' . $weather['windDegrees'] . ')';
         }
         $html .= _(" at ") . round($weather['wind']) . ' ' . $units['wind'];
 
         // Humidity.
-        $html .= '<br /><b>' . _("Humidity: ") . '</b>';
-        $html .= $weather['humidity'] . '%';
+        $html .= '<br /><b>' . _("Humidity: ") . '</b>' .
+            $weather['humidity'] . '%';
 
         // Visibility.
-        $html .= ' <b>' . _("Visibility: ") . '</b>';
-        $html .= is_numeric($weather['visibility']) ?
-            round($weather['visibility']) . ' ' . $units['vis'] :
-            $weather['visibility'];
+        $html .= ' <b>' . _("Visibility: ") . '</b>' .
+            is_numeric($weather['visibility']) ?
+                round($weather['visibility']) . ' ' . $units['vis'] :
+                $weather['visibility'];
 
         // UV index.
         $html .= ' <b>' . _("U.V. index: ") . '</b>';
@@ -234,8 +263,8 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
             Horde::img('block/weatherdotcom/32x32/' .
                        ($weather['conditionIcon'] == '-' ? 'na' : $weather['conditionIcon']) . '.png',
                        $condition,
-                       'align="middle"');
-        $html .= ' ' . $condition;
+                       'align="middle"') .
+            ' ' . $condition;
 
         // Do the forecast now.
         $html .= '<div class="control" style="text-align:center"><b>' . sprintf(_("%d-day forecast"), $this->_params['days'])
@@ -274,25 +303,25 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
             if ($which > 0 || ($which == 0 &&
                                (strtotime($location['time']) < strtotime('14:00')))) {
                 $condition = implode(' / ', array_map('_', explode(' / ', $day['day']['condition'])));
-                $html .= '<b><i>' . _("Day") . '</i></b><br/>';
-                $html .= Horde::img('block/weatherdotcom/23x23/' .
-                                    ($day['day']['conditionIcon'] == '-' ? 'na' : $day['day']['conditionIcon']) . '.png',
-                                    $condition);
-                $html .= '<br />' . $condition;
+                $html .= '<b><i>' . _("Day") . '</i></b><br />' .
+                    Horde::img('block/weatherdotcom/23x23/' .
+                               ($day['day']['conditionIcon'] == '-' ? 'na' : $day['day']['conditionIcon']) . '.png',
+                               $condition) .
+                    '<br />' . $condition;
             } else {
                 $html .= '&nbsp;';
             }
             $html .= '</span>';
 
             // Night forecast.
-            $html .= '<span style="text-align:center; float:right; width:50%;">';
-            $html .= '<b><i>' . _("Night") . '</i></b><br/>';
+            $html .= '<span style="text-align:center; float:right; width:50%;">' .
+                '<b><i>' . _("Night") . '</i></b><br />';
             $night = implode(' / ', array_map('_', explode(' / ', $day['night']['condition'])));
             $html .= Horde::img('block/weatherdotcom/23x23/' .
                                 ($day['night']['conditionIcon'] == '-' ? 'na' : $day['night']['conditionIcon']) . '.png',
-                                $night);
-            $html .= '<br />' . $night;
-            $html .= '</span>';
+                                $night) .
+                '<br />' . $night .
+                '</span>';
 
             $html .= '</div></td>';
         }
@@ -301,83 +330,17 @@ class Horde_Block_Horde_weatherdotcom extends Horde_Block {
         // Display a bar at the bottom of the block with the required
         // attribution to weather.com and the logo, both linked to
         // weather.com with the partner ID.
-        $html .= '<div style="text-align:right;" class="linedRow">';
-        $html .= _("Weather data provided by") . ' ';
-        $html .= Horde::link('http://www.weather.com/?prod=xoap&amp;par=' .
-                             $weatherDotCom->_partnerID,
-                             'weather.com', '', '_blank', '', 'weather.com');
-        $html .= '<i>weather.com</i>&reg; ';
-        $html .= Horde::img('block/weatherdotcom/32x32/TWClogo_32px.png',
-                            'weather.com logo', 'align="middle"');
-        $html .= '</a></div>';
+        $html .= '<div style="text-align:right;" class="linedRow">' .
+            _("Weather data provided by") . ' ' .
+            Horde::link('http://www.weather.com/?prod=xoap&amp;par=' .
+                        $weatherDotCom->_partnerID,
+                        'weather.com', '', '_blank', '', 'weather.com') .
+            '<i>weather.com</i>&reg; ' .
+            Horde::img('block/weatherdotcom/32x32/TWClogo_32px.png',
+                       'weather.com logo', 'align="middle"') .
+            '</a></div>';
 
         return $html;
     }
 
-    /**
-     * Possible strings from weather.com, just to have them in the
-     * source for localization reasons. This method should never be
-     * called (and doesn't need to be); it's purpose is to ensure that
-     * xgettext has a place to pull these strings from.
-     */
-    function defineGettextStrings()
-    {
-        _("AM Rain");
-        _("AM Showers");
-        _("AM Snow Showers");
-        _("Clear");
-        _("Cloudy");
-        _("Fair");
-        _("Few Showers");
-        _("Few Snow Showers");
-        _("Fog");
-        _("Haze");
-        _("Isolated T-Storms");
-        _("Light Drizzle");
-        _("Light Rain");
-        _("Light Rain Early");
-        _("Light Rain Late");
-        _("Light Rain Shower");
-        _("Light Rain with Thunder");
-        _("Light Snow");
-        _("Light Snow Shower");
-        _("Mostly Clear");
-        _("Mostly Cloudy");
-        _("Mostly Sunny");
-        _("Partly Cloudy");
-        _("PM Light Rain");
-        _("PM Showers");
-        _("PM Snow");
-        _("PM Snow Showers");
-        _("PM T-Storms");
-        _("Rain");
-        _("Rain Early");
-        _("Rain Late");
-        _("Rain Shower");
-        _("Rain and Snow");
-        _("Rain to Snow");
-        _("Showers");
-        _("Showers Early");
-        _("Showers Late");
-        _("Showers in the Vicinity");
-        _("Snow");
-        _("Snow Shower");
-        _("Snow Showers");
-        _("Snow Showers Early");
-        _("Snow Showers Late");
-        _("Sunny");
-        _("T-Storm");
-        _("T-Storms");
-        _("Thunder");
-        _("Wind");
-        _("Wind Early");
-        _("Low");
-        _("Moderate");
-        _("High");
-        _("Very High");
-        _("falling");
-        _("steady");
-        _("rising");
-    }
-
 }
