@@ -7,7 +7,7 @@ include_once 'Horde/Util.php';
  * The Horde:: class provides the functionality shared by all Horde
  * applications.
  *
- * $Horde: framework/Horde/Horde.php,v 1.489.2.9 2005/02/01 19:00:45 chuck Exp $
+ * $Horde: framework/Horde/Horde.php,v 1.489.2.26 2005/08/24 14:04:36 selsky Exp $
  *
  * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
@@ -25,8 +25,6 @@ class Horde {
     /**
      * Logs a message to the global Horde log backend.
      *
-     * @access public
-     *
      * @param mixed $message     Either a string or a PEAR_Error object.
      * @param string $file       What file was the log function called from
      *                           (e.g. __FILE__)?
@@ -66,7 +64,7 @@ class Horde {
             $message = $message->getMessage();
             if (!empty($userinfo)) {
                 if (is_array($userinfo)) {
-                    $userinfo = implode(', ', $userinfo);
+                    $userinfo = @implode(', ', $userinfo);
                 }
                 $message .= ': ' . $userinfo;
             }
@@ -118,8 +116,6 @@ class Horde {
      * Destroys any existing session on login and make sure to use a new
      * session ID, to avoid session fixation issues. Should be called before
      * checking a login.
-     *
-     * @access public
      */
     function getCleanSession()
     {
@@ -147,8 +143,6 @@ class Horde {
     /**
      * Aborts with a fatal error, displaying debug information to the user.
      *
-     * @access public
-     *
      * @param mixed $error   A PEAR_Error object with debug information or an
      *                       error message.
      * @param integer $file  The file in which the error occured.
@@ -177,7 +171,7 @@ class Horde {
                 break;
 
             case HORDE_ERROR_DRIVER_CONFIG:
-                $message = sprintf(_("Required '%s' not specified in %s configuration."), $info['field'], $info['name']) . '<br />' .
+                $message = sprintf(_("Required \"%s\" not specified in %s configuration."), $info['field'], $info['name']) . '<br />' .
                     sprintf(_("The file %s should contain a %s setting."),
                             $GLOBALS['registry']->get('fileroot') . '/config/' . $info['file'],
                             sprintf("%s['%s']['params']['%s']", $info['variable'], $info['driver'], $info['field']));
@@ -227,8 +221,6 @@ HTML;
      * Adds the javascript code to the output (if output has already started)
      * or to the list of script files to include via includeScriptFiles().
      *
-     * @access public
-     *
      * @param string $file     The full javascript file name.
      * @param string $app      The application name. Defaults to the current
      *                         application.
@@ -267,8 +259,6 @@ HTML;
 
     /**
      * Includes javascript files that were needed before any headers were sent.
-     *
-     * @access public
      */
     function includeScriptFiles()
     {
@@ -293,8 +283,6 @@ HTML;
 
     /**
      * Includes javascript files that were needed before any headers were sent.
-     *
-     * @access public
      */
     function inlineScriptFiles()
     {
@@ -318,8 +306,6 @@ HTML;
     /**
      * Checks if link should be shown and return the nescessary code.
      *
-     * @access public
-     *
      * @param string  $type      Type of link to display
      * @param string  $app       The name of the current Horde application.
      * @param boolean $override  Override Horde settings?
@@ -346,10 +332,10 @@ HTML;
             break;
 
         case 'problem':
-            return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/problem.php?return_url=' . urlencode(Horde::selfUrl(true)));
+            return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/problem.php?return_url=' . urlencode(Horde::selfUrl(true, true, true)));
 
         case 'logout':
-            return Auth::addLogoutParameters(Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/login.php'), AUTH_REASON_LOGOUT);
+            return Horde::url(Auth::addLogoutParameters($GLOBALS['registry']->get('webroot', 'horde') . '/login.php', AUTH_REASON_LOGOUT));
 
         case 'login':
             return Auth::getLoginScreen('', $referrer ? Horde::selfUrl(true) : null);
@@ -366,8 +352,6 @@ HTML;
     }
 
     /**
-     * @access public
-     *
      * @param string $type  The type of link.
      *
      * @return boolean  True if the link is to be shown.
@@ -500,7 +484,7 @@ HTML;
         foreach ($fields as $field) {
             if (!isset($params[$field])) {
                 Horde::fatal(PEAR::raiseError(
-                    sprintf(_("Required '%s' not specified in %s configuration."), $field, $name) . "\n\n" .
+                    sprintf(_("Required \"%s\" not specified in %s configuration."), $field, $name) . "\n\n" .
                     sprintf(_("The file %s should contain a %s setting."),
                         $fileroot . '/config/' . $file,
                         sprintf("%s['%s']['params']['%s']", $variable, $driver, $field))),
@@ -514,11 +498,11 @@ HTML;
      * If a full URL is requested, all parameter separators get converted to
      * "&", otherwise to "&amp;".
      *
-     * @access public
-     *
-     * @param string $uri          The URI to be modified.
-     * @param bool $full           Generate a full (http://server/path/) URL.
-     * @param int $append_session  0 = only if needed, 1 = always, -1 = never.
+     * @param string $uri              The URI to be modified.
+     * @param boolean $full            Generate a full (http://server/path/)
+     *                                 URL.
+     * @param integer $append_session  0 = only if needed, 1 = always, -1 =
+     *                                 never.
      *
      * @return string  The URL with the session id appended (if needed).
      */
@@ -595,11 +579,11 @@ HTML;
      * Returns a session-id-ified version of $uri, using the current
      * application's webroot setting.
      *
-     * @access public
-     *
-     * @param string $uri          The URI to be modified.
-     * @param bool $full           Generate a full (http://server/path/) URL.
-     * @param int $append_session  0 = only if needed, 1 = always, -1 = never.
+     * @param string $uri              The URI to be modified.
+     * @param boolean $full            Generate a full (http://server/path/)
+     *                                 URL.
+     * @param integer $append_session  0 = only if needed, 1 = always, -1 =
+     *                                 never.
      *
      * @return string  The url with the session id appended.
      */
@@ -647,8 +631,6 @@ HTML;
      * Returns a URL to be used for downloading, that takes into account any
      * special browser quirks (i.e. IE's broken filename handling).
      *
-     * @access public
-     *
      * @param string $filename  The filename of the download data.
      * @param array $params     Any additional parameters needed.
      * @param string $url       The URL to alter. If none passed in, will use
@@ -671,9 +653,7 @@ HTML;
 
         /* Add parameters. */
         if (!is_null($params)) {
-            foreach ($params as $key => $val) {
-                $url = Util::addParameter($url, $key, $val);
-            }
+            $url = Util::addParameter($url, $params);
         }
 
         /* If we are using the default Horde download link, add the
@@ -681,7 +661,7 @@ HTML;
          * many browsers, this should allow every browser to download
          * correctly. */
         if ($horde_url) {
-            $url = Util::addParameter($url, 'fn=/' . rawurlencode($filename));
+            $url = Util::addParameter($url, 'fn', '/' . rawurlencode($filename));
         } elseif ($browser->hasQuirk('break_disposition_filename')) {
             /* Some browsers will only obtain the filename correctly
              * if the extension is the last argument in the query
@@ -715,7 +695,7 @@ HTML;
 
             /* Append the extension, if it exists. */
             if (($server == 'apache2') || !empty($ext)) {
-                $url = Util::addParameter($url, 'fn_ext=/' . $filename);
+                $url = Util::addParameter($url, 'fn_ext', '/' . $filename);
             }
         }
 
@@ -725,8 +705,6 @@ HTML;
     /**
      * Returns an anchor tag with the relevant parameters
      *
-     * @access public
-     *
      * @param string $url        The full URL to be linked to
      * @param string $status     The JavaScript mouse-over string
      * @param string $class      The CSS class of the link
@@ -742,12 +720,17 @@ HTML;
     function link($url, $status = '', $class = '', $target = '', $onclick = '',
                   $title = '', $accesskey = '', $attributes = array())
     {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
         $ret = "<a href=\"$url\"";
         if (!empty($onclick)) {
             $ret .= " onclick=\"$onclick\"";
         }
         if (!empty($status)) {
-            $ret .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' . @htmlspecialchars(strtr(addslashes($status), array("\r" => '', "\n" => '')), ENT_QUOTES, NLS::getCharset()) . '\'; return true;"';
+            $ret .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' . @htmlspecialchars(strtr(addslashes($status), array("\r" => '', "\n" => '')), ENT_QUOTES, $charset) . '\'; return true;"';
         }
         if (!empty($class)) {
             $ret .= " class=\"$class\"";
@@ -756,7 +739,7 @@ HTML;
             $ret .= " target=\"$target\"";
         }
         if (!empty($title)) {
-            $ret .= ' title="' . @htmlspecialchars($title, ENT_QUOTES, NLS::getCharset()) . '"';
+            $ret .= ' title="' . @htmlspecialchars($title, ENT_QUOTES, $charset) . '"';
         }
         if (!empty($accesskey)) {
             $ret .= ' accesskey="' . htmlspecialchars($accesskey) . '"';
@@ -773,8 +756,6 @@ HTML;
      * Returns an anchor sequence with the relevant parameters for a widget
      * with accesskey and text.
      *
-     * @access public
-     *
      * @param string $url       The full URL to be linked to
      * @param string $status    The JavaScript mouse-over string
      * @param string $class     The CSS class of the link
@@ -796,9 +777,7 @@ HTML;
     }
 
     /**
-     * Returns a session-id-ified version of $PHP_SELF.
-     *
-     * @access public
+     * Returns a session-id-ified version of $SCRIPT_NAME resp. $PHP_SELF.
      *
      * @param boolean $query_string  Include any QUERY_STRING?
      * @param boolean $nocache       Include a nocache parameter in the URL?
@@ -809,7 +788,16 @@ HTML;
     function selfUrl($query_string = false, $nocache = true, $full = false,
                      $force_ssl = false)
     {
-        $url = $_SERVER['PHP_SELF'];
+        if (substr(php_sapi_name(), 0, 3) == 'cgi') {
+            // When using CGI PHP, SCRIPT_NAME may contain the path to
+            // the PHP binary instead of the script being run; use
+            // PHP_SELF instead.
+            $url = $_SERVER['PHP_SELF'];
+        } else {
+            $url = isset($_SERVER['SCRIPT_NAME']) ?
+                $_SERVER['SCRIPT_NAME'] :
+                $_SERVER['PHP_SELF'];
+        }
 
         if ($query_string && !empty($_SERVER['QUERY_STRING'])) {
             $url .= '?' . $_SERVER['QUERY_STRING'];
@@ -827,22 +815,25 @@ HTML;
     /**
      * Constructs a correctly-pathed link to an image.
      *
-     * @access public
-     *
-     * @param          string $src   The image file.
-     * @param optional string $alt   Text describing the image.
-     * @param optional mixed  $attr  Any additional attributes for the image tag.
-     *                               Can be a pre-built string or an array of key/value
-     *                               pairs that will be assembled and html-encoded.
-     * @param optional string $dir   The root graphics directory.
+     * @param string $src   The image file.
+     * @param string $alt   Text describing the image.
+     * @param mixed  $attr  Any additional attributes for the image tag. Can be
+     *                      a pre-built string or an array of key/value pairs
+     *                      that will be assembled and html-encoded.
+     * @param string $dir   The root graphics directory.
      *
      * @return string  The full image tag.
      */
     function img($src, $alt = '', $attr = '', $dir = null)
     {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
         /* If browser does not support images, simply return the ALT text. */
         if (!$GLOBALS['browser']->hasFeature('images')) {
-            return @htmlspecialchars($alt, ENT_COMPAT, NLS::getCharset());
+            return @htmlspecialchars($alt, ENT_COMPAT, $charset);
         }
 
         /* If no directory has been specified, get it from the registry. */
@@ -866,7 +857,7 @@ HTML;
 
         $img = '<img';
         foreach ($attributes as $attribute => $value) {
-            $img .= ' ' . $attribute . '="' . ($attribute == 'src' ? $value : @htmlspecialchars($value, ENT_COMPAT, NLS::getCharset())) . '"';
+            $img .= ' ' . $attribute . '="' . ($attribute == 'src' ? $value : @htmlspecialchars($value, ENT_COMPAT, $charset)) . '"';
         }
 
         /* If the user supplied a pre-built string of attributes, add that. */
@@ -882,8 +873,6 @@ HTML;
      * Determines the location of the system temporary directory. If a specific
      * setting cannot be found, it defaults to /tmp.
      *
-     * @access public
-     *
      * @return string  A directory name which can be used for temp files.
      *                 Returns false if one could not be found.
      */
@@ -910,8 +899,6 @@ HTML;
      * Creates a temporary filename for the lifetime of the script, and
      * (optionally) registers it to be deleted at request shutdown.
      *
-     * @access public
-     *
      * @param string $prefix   Prefix to make the temporary name more
      *                         recognizable.
      * @param boolean $delete  Delete the file at the end of the request?
@@ -935,8 +922,6 @@ HTML;
     /**
      * Starts output compression, if requested.
      *
-     * @access public
-     *
      * @since Horde 2.2
      */
     function compressOutput()
@@ -951,6 +936,7 @@ HTML;
         if ($GLOBALS['conf']['compress_pages'] &&
             !$GLOBALS['browser']->hasQuirk('buggy_compression') &&
             ini_get('zlib.output_compression') == '' &&
+            ini_get('zend_accelerator.compress_all') == '' &&
             ini_get('output_handler') != 'ob_gzhandler') {
             if (ob_get_level()) {
                 ob_end_clean();
@@ -964,8 +950,6 @@ HTML;
     /**
      * Determines if output compression can be used.
      *
-     * @access public
-     *
      * @return boolean  True if output compression can be used, false if not.
      */
     function allowOutputCompression()
@@ -979,6 +963,7 @@ HTML;
         }
 
         return (ini_get('zlib.output_compression') == '' &&
+                ini_get('zend_accelerator.compress_all') == '' &&
                 ini_get('output_handler') != 'ob_gzhandler');
     }
 
@@ -986,8 +971,6 @@ HTML;
      * Returns the Web server being used.
      * PHP string list built from the PHP 'configure' script.
      *
-     * @access public
-     *
      * @return string  A web server identification string.
      * <pre>
      * 'aolserver' = AOL Server
@@ -1026,8 +1009,6 @@ HTML;
     /**
      * Returns the <link> tags for the CSS stylesheets.
      *
-     * @access public
-     *
      * @param string|array $app  The Horde application(s).
      * @param mixed $theme       The theme to use; specify an empty value to
      *                           retrieve the theme from user preferences, and
@@ -1096,14 +1077,21 @@ HTML;
         if ($GLOBALS['browser']->isBrowser('opera')) {
             $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/opera.css" rel="stylesheet" type="text/css" />' . "\n";
         }
+        if ($GLOBALS['browser']->isBrowser('mozilla') &&
+            $GLOBALS['browser']->getMajor() >= 5 &&
+            preg_match('/rv:(.*)\)/', $GLOBALS['browser']->getAgentString(), $revision) &&
+            $revision[1] <= 1.4) {
+            $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/moz14.css" rel="stylesheet" type="text/css" />' . "\n";
+        }
+        if (strpos(strtolower($GLOBALS['browser']->getAgentString()), 'safari') !== false) {
+            $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/safari.css" rel="stylesheet" type="text/css" />' . "\n";
+        }
 
         return $html;
     }
 
     /**
      * Sets a custom session handler up, if there is one.
-     *
-     * @access public
      */
     function setupSessionHandler()
     {
@@ -1147,8 +1135,6 @@ HTML;
     /**
      * Returns an un-used access key from the label given.
      *
-     * @access public
-     *
      * @param string $label     The label to choose an access key from.
      * @param boolean $nocheck  Don't check if the access key already has been
      *                          used?
@@ -1212,8 +1198,6 @@ HTML;
      * For multibyte charset strings the access key gets removed completely,
      * otherwise only the underscore gets removed.
      *
-     * @access public
-     *
      * @param string $label  The label containing an access key.
      *
      * @return string  The label with the access key being stripped.
@@ -1231,8 +1215,6 @@ HTML;
     /**
      * Highlights an access key in a label.
      *
-     * @access public
-     *
      * @param string $label      The label to to highlight the access key in.
      * @param string $accessKey  The access key to highlight.
      *
@@ -1306,8 +1288,6 @@ HTML;
 
     /**
      * Redirects to the main Horde login page on authentication failure.
-     *
-     * @access public
      */
     function authenticationFailureRedirect()
     {
@@ -1331,8 +1311,6 @@ HTML;
      * If using this function, the following function must be called:
      *   Horde::addScriptFile('tooltip.js', 'horde', true);
      *
-     * @access public
-     *
      * @param string $url        The full URL to be linked to
      * @param string $status     The JavaScript mouse-over string
      * @param string $class      The CSS class of the link
@@ -1359,8 +1337,10 @@ HTML;
         if (strlen($title)) {
             require_once 'Horde/Text/Filter.php';
             $title = Text_Filter::filter($title, 'text2html', array('parselevel' => TEXT_HTML_NOHTML, 'charset' => '', 'class' => ''));
-            $title = '<pre style="margin:0px;">' . strtr(addslashes($title), array("\r" => '', "\n" => '')) . '</pre>';
+            $title = '<pre>' . strtr(addslashes($title), array("\r" => '', "\n" => '')) . '</pre>';
             $url .= ' onmouseover="tooltipLink(\'' . @htmlspecialchars($title, ENT_QUOTES, NLS::getCharset()) . '\', \'' . $status . '\'); return true;" onmouseout="tooltipClose();"';
+        } else {
+            $url .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' . $status . '\'; return true;"';
         }
         $url .= '>';
 
@@ -1373,8 +1353,6 @@ HTML;
      * these two conditions are not satisfied it will return the specified
      * value (by default a PEAR error).
      *
-     * @access public
-     *
      * @param string $hook  The function to call.
      * @param array  $args  An array of any arguments to pass to the hook
      *                      function.
