@@ -4,10 +4,12 @@ require_once 'Horde/String.php';
 
 /**
  * The Browser:: class provides capability information for the current
- * web client. Browser identification is performed by examining the
- * HTTP_USER_AGENT environmental variable provide by the web server.
+ * web client.
  *
- * $Horde: framework/Browser/Browser.php,v 1.153.2.10 2005/02/04 16:35:47 chuck Exp $
+ * Browser identification is performed by examining the HTTP_USER_AGENT
+ * environment variable provided by the web server.
+ *
+ * $Horde: framework/Browser/Browser.php,v 1.153.2.25.2.1 2005/10/26 06:49:29 slusarz Exp $
  *
  * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
@@ -25,56 +27,56 @@ class Browser {
     /**
      * Major version number.
      *
-     * @var integer $_majorVersion
+     * @var integer
      */
     var $_majorVersion = 0;
 
     /**
      * Minor version number.
      *
-     * @var integer $_minorVersion
+     * @var integer
      */
     var $_minorVersion = 0;
 
     /**
      * Browser name.
      *
-     * @var string $_browser
+     * @var string
      */
     var $_browser = '';
 
     /**
      * Full user agent string.
      *
-     * @var string $_agent
+     * @var string
      */
     var $_agent = '';
 
     /**
      * Lower-case user agent string.
      *
-     * @var string $_agent
+     * @var string
      */
     var $_lowerAgent = '';
 
     /**
      * HTTP_ACCEPT string
      *
-     * @var string $_accept
+     * @var string
      */
     var $_accept = '';
 
     /**
      * Platform the browser is running on.
      *
-     * @var string $_platform
+     * @var string
      */
     var $_platform = '';
 
     /**
      * Known robots.
      *
-     * @var array $_robots
+     * @var array
      */
     var $_robots = array(
         /* The most common ones. */
@@ -87,6 +89,9 @@ class Browser {
         'ArchitextSpider',
         'Ask Jeeves',
         'B-l-i-t-z-Bot',
+        'Baiduspider',
+        'BecomeBot',
+        'cfetch',
         'ConveraCrawler',
         'ExtractorPro',
         'FAST-WebCrawler',
@@ -97,18 +102,24 @@ class Browser {
         'Girafabot',
         'grub-client',
         'Gulliver',
+        'HTTrack',
         'ia_archiver',
         'InfoSeek',
+        'kinjabot',
         'KIT-Fireball',
+        'larbin',
         'LEIA',
+        'lmspider',
         'Lycos_Spider',
         'Mediapartners-Google',
         'MuscatFerret',
         'NaverBot',
+        'OmniExplorer_Bot',
         'polybot',
         'Pompos',
         'Scooter',
         'Teoma',
+        'TheSuBot',
         'TurnitinBot',
         'Ultraseek',
         'ViolaBot',
@@ -120,14 +131,14 @@ class Browser {
     /**
      * Is this a mobile browser?
      *
-     * @var boolean $_mobile
+     * @var boolean
      */
     var $_mobile = false;
 
     /**
      * Features.
      *
-     * @var array $_features
+     * @var array
      */
     var $_features = array(
         'html'       => true,
@@ -152,7 +163,7 @@ class Browser {
     /**
      * Quirks
      *
-     * @var array $_quirks
+     * @var array
      */
     var $_quirks = array(
         'avoid_popup_windows'        => false,
@@ -171,15 +182,16 @@ class Browser {
         'png_transparency'           => false,
         'scrollbar_in_way'           => false,
         'scroll_tds'                 => false,
+        'windowed_controls'          => false,
     );
 
     /**
      * List of viewable image MIME subtypes.
      * This list of viewable images works for IE and Netscape/Mozilla.
      *
-     * @var array $_images
+     * @var array
      */
-    var $_images = array('jpeg', 'gif', 'png', 'pjpeg', 'x-png', 'bmp', 'tiff');
+    var $_images = array('jpeg', 'gif', 'png', 'pjpeg', 'x-png', 'bmp');
 
     /**
 
@@ -187,15 +199,13 @@ class Browser {
      * Returns a reference to the global Browser object, only creating it
      * if it doesn't already exist.
      *
-     * This method must be invoked as:
-     *   $browser = &Browser::singleton([$userAgent[, $accept]]);
+     * This method must be invoked as:<pre>
+     *   $browser = &Browser::singleton([$userAgent[, $accept]]);</pre>
      *
-     * @access public
+     * @param string $userAgent  The browser string to parse.
+     * @param string $accept     The HTTP_ACCEPT settings to use.
      *
-     * @param optional string $userAgent  The browser string to parse.
-     * @param optional string $accept     The HTTP_ACCEPT settings to use.
-     *
-     * @return object Browser  The Browser object.
+     * @return Browser  The Browser object.
      */
     function &singleton($userAgent = null, $accept = null)
     {
@@ -216,10 +226,8 @@ class Browser {
     /**
      * Create a browser instance (Constructor).
      *
-     * @access public
-     *
-     * @param optional string $userAgent  The browser string to parse.
-     * @param optional string $accept     The HTTP_ACCEPT settings to use.
+     * @param string $userAgent  The browser string to parse.
+     * @param string $accept     The HTTP_ACCEPT settings to use.
      */
     function Browser($userAgent = null, $accept = null)
     {
@@ -230,10 +238,8 @@ class Browser {
      * Parses the user agent string and inititializes the object with
      * all the known features and quirks for the given browser.
      *
-     * @access public
-     *
-     * @param optional string $userAgent  The browser string to parse.
-     * @param optional string $accept     The HTTP_ACCEPT settings to use.
+     * @param string $userAgent  The browser string to parse.
+     * @param string $accept     The HTTP_ACCEPT settings to use.
      */
     function match($userAgent = null, $accept = null)
     {
@@ -270,14 +276,12 @@ class Browser {
                 $this->setFeature('javascript', true);
                 $this->setQuirk('no_filename_spaces');
 
-                switch ($this->_majorVersion) {
-                case 7:
+                if ($this->_majorVersion >= 7) {
                     $this->setFeature('dom');
                     $this->setFeature('iframes');
                     $this->setFeature('accesskey');
                     $this->setFeature('optgroup');
                     $this->setQuirk('double_linebreak_textarea');
-                    break;
                 }
             } elseif (strpos($this->_lowerAgent, 'elaine/') !== false ||
                       strpos($this->_lowerAgent, 'palmsource') !== false ||
@@ -340,7 +344,7 @@ class Browser {
                 /* Some Handhelds have their screen resolution in the
                  * user agent string, which we can use to look for
                  * mobile agents. */
-                if (preg_match('/; (120x160|240x280|240x320)\)/', $this->_agent)) {
+                if (preg_match('/; (120x160|240x280|240x320|320x320)\)/', $this->_agent)) {
                     $this->_mobile = true;
                 }
 
@@ -357,6 +361,7 @@ class Browser {
                     $this->setFeature('xmlhttpreq');
                     $this->setQuirk('scrollbar_in_way');
                     $this->setQuirk('broken_multipart_form');
+                    $this->setQuirk('windowed_controls');
                     break;
 
                 case 5:
@@ -370,6 +375,7 @@ class Browser {
                         $this->setFeature('xmlhttpreq');
                         if ($this->_minorVersion >= 5) {
                             $this->setFeature('rte');
+                            $this->setQuirk('windowed_controls');
                         }
                     }
                     $this->setFeature('iframes');
@@ -430,7 +436,6 @@ class Browser {
                     }
                 } else {
                     // Konqueror.
-                    $this->setBrowser('konqueror');
                     $this->setFeature('javascript', 1.1);
                     switch ($this->_majorVersion) {
                     case 3:
@@ -542,6 +547,13 @@ class Browser {
                 $this->setBrowser('imode');
                 $this->setFeature('images', false);
                 $this->_mobile = true;
+            } elseif (strpos($this->_agent, 'MOT-V3/') !== false) {
+                $this->setBrowser('motorola');
+                $this->setFeature('html', false);
+                $this->setFeature('javascript', false);
+                $this->setFeature('hdml');
+                $this->setFeature('wml');
+                $this->_mobile = true;
             } elseif (strpos($this->_lowerAgent, 'j-') !== false) {
                 $this->setBrowser('mml');
                 $this->_mobile = true;
@@ -556,8 +568,6 @@ class Browser {
      * to let us tell what line breaks to send, so it's good enough
      * for its purpose.
      *
-     * @access public
-     *
      * @since Horde 2.2
      */
     function _setPlatform()
@@ -586,8 +596,6 @@ class Browser {
     /**
      * Sets the current browser.
      *
-     * @access public
-     *
      * @param string $browser  The browser to set as current.
      */
     function setBrowser($browser)
@@ -598,8 +606,6 @@ class Browser {
     /**
      * Determine if the given browser is the same as the current.
      *
-     * @access public
-     *
      * @param string $browser  The browser to check.
      *
      * @return boolean  Is the given browser the same as the current?
@@ -622,8 +628,6 @@ class Browser {
     /**
      * Determines if the browser is a robot or not.
      *
-     * @access public
-     *
      * @return boolean  True if browser is a known robot.
      */
     function isRobot()
@@ -639,8 +643,6 @@ class Browser {
     /**
      * Retrieve the current browser.
      *
-     * @access public
-     *
      * @return string  The current browser.
      */
     function getBrowser()
@@ -651,9 +653,7 @@ class Browser {
     /**
      * Retrieve the current browser's major version.
      *
-     * @access public
-     *
-     * @return int  The current browser's major version.
+     * @return integer  The current browser's major version.
      */
     function getMajor()
     {
@@ -663,9 +663,7 @@ class Browser {
     /**
      * Retrieve the current browser's minor version.
      *
-     * @access public
-     *
-     * @return int  The current browser's minor version.
+     * @return integer  The current browser's minor version.
      */
     function getMinor()
     {
@@ -675,8 +673,6 @@ class Browser {
     /**
      * Retrieve the current browser's version.
      *
-     * @access public
-     *
      * @return string  The current browser's version.
      */
     function getVersion()
@@ -687,8 +683,6 @@ class Browser {
     /**
      * Return the full browser agent string.
      *
-     * @access public
-     *
      * @return string  The browser agent string.
      */
     function getAgentString()
@@ -699,10 +693,8 @@ class Browser {
     /**
      * Set unique behavior for the current browser.
      *
-     * @access public
-     *
-     * @param string $quirk           The behavior to set.
-     * @param optional string $value  Special behavior parameter.
+     * @param string $quirk  The behavior to set.
+     * @param string $value  Special behavior parameter.
      */
     function setQuirk($quirk, $value = true)
     {
@@ -712,8 +704,6 @@ class Browser {
     /**
      * Check unique behavior for the current browser.
      *
-     * @access public
-     *
      * @param string $quirk  The behavior to check.
      *
      * @return boolean  Does the browser have the behavior set?
@@ -724,11 +714,9 @@ class Browser {
     }
 
     /**
-     * Retreive unique behavior for the current browser.
+     * Retrieve unique behavior for the current browser.
      *
-     * @access public
-     *
-     * @param string $quirk  The behavior to retreive.
+     * @param string $quirk  The behavior to retrieve.
      *
      * @return string  The value for the requested behavior.
      */
@@ -742,10 +730,8 @@ class Browser {
     /**
      * Set capabilities for the current browser.
      *
-     * @access public
-     *
-     * @param string $feature         The capability to set.
-     * @param optional string $value  Special capability parameter.
+     * @param string $feature  The capability to set.
+     * @param string $value    Special capability parameter.
      */
     function setFeature($feature, $value = true)
     {
@@ -755,8 +741,6 @@ class Browser {
     /**
      * Check the current browser capabilities.
      *
-     * @access public
-     *
      * @param string $feature  The capability to check.
      *
      * @return boolean  Does the browser have the capability set?
@@ -767,11 +751,9 @@ class Browser {
     }
 
     /**
-     * Retreive the current browser capability.
+     * Retrieve the current browser capability.
      *
-     * @access public
-     *
-     * @param string $feature  The capability to retreive.
+     * @param string $feature  The capability to retrieve.
      *
      * @return string  The value of the requested capability.
      */
@@ -785,8 +767,6 @@ class Browser {
     /**
      * Determine if we are using a secure (SSL) connection.
      *
-     * @access public
-     *
      * @return boolean  True if using SSL, false if not.
      */
     function usingSSLConnection()
@@ -799,8 +779,6 @@ class Browser {
     /**
      * Returns the server protocol in use on the current server.
      *
-     * @access public
-     *
      * @return string  The HTTP server protocol version.
      */
     function getHTTPProtocol()
@@ -817,8 +795,6 @@ class Browser {
     /**
      * Determine if files can be uploaded to the system.
      *
-     * @access public
-     *
      * @return integer  If uploads allowed, returns the maximum size of the
      *                  upload in bytes.  Returns 0 if uploads are not
      *                  allowed.
@@ -830,21 +806,35 @@ class Browser {
                 !is_writable($dir)) {
                 return 0;
             }
-            $size = ini_get('upload_max_filesize');
-            switch (strtolower(substr($size, -1, 1))) {
+            $filesize = ini_get('upload_max_filesize');
+            switch (strtolower(substr($filesize, -1, 1))) {
             case 'k':
-                $size = intval(floatval($size) * 1024);
+                $filesize = intval(floatval($filesize) * 1024);
                 break;
 
             case 'm':
-                $size = intval(floatval($size) * 1024 * 1024);
+                $filesize = intval(floatval($filesize) * 1024 * 1024);
                 break;
 
             default:
-                $size = intval($size);
+                $filesize = intval($filesize);
                 break;
             }
-            return $size;
+            $postsize = ini_get('post_max_size');
+            switch (strtolower(substr($postsize, -1, 1))) {
+            case 'k':
+                $postsize = intval(floatval($postsize) * 1024);
+                break;
+
+            case 'm':
+                $postsize = intval(floatval($postsize) * 1024 * 1024);
+                break;
+
+            default:
+                $postsize = intval($postsize);
+                break;
+            }
+            return min($filesize, $postsize);
         } else {
             return 0;
         }
@@ -854,12 +844,10 @@ class Browser {
      * Determines if the file was uploaded or not.  If not, will return the
      * appropriate error message.
      *
-     * @access public
-     *
-     * @param string $field           The name of the field containing the
-     *                                uploaded file.
-     * @param optional string $name   The file description string to use in the
-     *                                error message.  Default: 'file'.
+     * @param string $field  The name of the field containing the uploaded
+     *                       file.
+     * @param string $name   The file description string to use in the error
+     *                       message.  Default: 'file'.
      *
      * @return mixed  True on success, PEAR_Error on error.
      */
@@ -911,19 +899,19 @@ class Browser {
     /**
      * Returns the headers for a browser download.
      *
-     * @access public
-     *
-     * @param optional string $filename  The filename of the download.
-     * @param optional string $cType     The content-type description of the
-     *                                   file.
-     * @param optional boolean $inline   True if inline, false if attachment.
-     * @param optional string $cLength   The content-length of this file.
+     * @param string $filename  The filename of the download.
+     * @param string $cType     The content-type description of the file.
+     * @param boolean $inline   True if inline, false if attachment.
+     * @param string $cLength   The content-length of this file.
      *
      * @since Horde 2.2
      */
     function downloadHeaders($filename = 'unknown', $cType = null,
                              $inline = false, $cLength = null)
     {
+        /* Remove linebreaks from file names. */
+        $filename = str_replace(array("\r\n", "\r", "\n"), ' ', $filename);
+
         /* Some browsers don't like spaces in the filename. */
         if ($this->hasQuirk('no_filename_spaces')) {
             $filename = strtr($filename, ' ', '_');
@@ -980,8 +968,6 @@ class Browser {
     /**
      * Determines if a browser can display a given MIME type.
      *
-     * @access public
-     *
      * @param string $mimetype  The MIME type to check.
      *
      * @return boolean  True if the browser can display the MIME type.
@@ -1033,8 +1019,6 @@ class Browser {
      * jscript will interpret it as a single quote, pound sign, or
      * ampersand and refuse to work.
      *
-     * @access public
-     *
      * @param string $code  The JS code to escape.
      *
      * @return string  The escaped code.
@@ -1057,8 +1041,6 @@ class Browser {
     /**
      * Set the IE version in the session.
      *
-     * @access public
-     *
      * @param string $ver  The IE Version string.
      */
     function setIEVersion($ver)
@@ -1071,8 +1053,6 @@ class Browser {
     /**
      * Return the IE version stored in the session, if available.
      *
-     * @access public
-     *
      * @return mixed  The IE Version string or null if no string is stored.
      */
     function getIEVersion()
