@@ -13,7 +13,7 @@
  *
  *   http://alexandre.alapetite.net/doc-alex/domxml-php4-php5/
  *
- * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.7 2005/03/25 11:10:16 jan Exp $
+ * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.10 2005/06/23 20:18:07 selsky Exp $
  *
  * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
  *
@@ -29,7 +29,7 @@ class Horde_Config {
     /**
      * The name of the configured application.
      *
-     * @var string $_app
+     * @var string
      */
     var $_app;
 
@@ -37,65 +37,64 @@ class Horde_Config {
      * The XML tree of the configuration file traversed to an
      * associative array.
      *
-     * @var array $_xmlConfigTree
+     * @var array
      */
     var $_xmlConfigTree = null;
 
     /**
      * The content of the generated configuration file.
      *
-     * @var string $_phpConfig
+     * @var string
      */
     var $_phpConfig;
 
     /**
      * The content of the old configuration file.
      *
-     * @var string $_oldConfig
+     * @var string
      */
     var $_oldConfig;
 
     /**
-     * The manual configuration in front of the generated
-     * configuration.
+     * The manual configuration in front of the generated configuration.
      *
-     * @var string $_preConfig
+     * @var string
      */
     var $_preConfig;
 
     /**
      * The manual configuration after the generated configuration.
      *
-     * @var string $_preConfig
+     * @var string
      */
     var $_postConfig;
 
     /**
      * The current $conf array of the configured application.
      *
-     * @var array $_currentConfig
+     * @var array
      */
     var $_currentConfig = array();
 
     /**
-     * The CVS version tag of the conf.xml file which will be copied
-     * into the conf.php file.
+     * The CVS version tag of the conf.xml file which will be copied into the
+     * conf.php file.
      *
-     * @var string $_versionTag
+     * @var string
      */
     var $_versionTag = '';
 
     /**
      * The line marking the begin of the generated configuration.
      *
-     * @var string $_configBegin
+     * @var string
      */
     var $_configBegin = "/* CONFIG START. DO NOT CHANGE ANYTHING IN OR AFTER THIS LINE. */\n";
 
     /**
      * The line marking the end of the generated configuration.
      *
-     * @var string $_configEnd
+     * @var string
      */
     var $_configEnd = "/* CONFIG END. DO NOT CHANGE ANYTHING IN OR BEFORE THIS LINE. */\n";
 
@@ -371,7 +370,7 @@ class Horde_Config {
                     $name = md5(microtime());
                 }
                 $conf[$name] = array('_type' => 'description',
-                                     'desc' => Text_Filter::filter($this->_default($curctx, $this->_getNodeOnlyText($node))), 'linkurls');
+                                     'desc' => Text_Filter::filter($this->_default($curctx, $this->_getNodeOnlyText($node)), 'linkurls', array('callback' => 'Horde::externalUrl')));
                 break;
 
             case 'configheader':
@@ -445,7 +444,7 @@ class Horde_Config {
                                      'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)));
                 if ($conf[$name]['default'] === false) {
                     $conf[$name]['default'] = 'false';
-                } else if ($conf[$name]['default'] === true) {
+                } elseif ($conf[$name]['default'] === true) {
                     $conf[$name]['default'] = 'true';
                 }
                 break;
@@ -552,22 +551,22 @@ class Horde_Config {
             '_type' => 'text',
             'required' => true,
             'desc' => 'Database server/host',
-            'default' => $this->_default($ctx . '|hostspec', 'localhost'));
+            'default' => $this->_default($ctx . '|hostspec', ''));
         $username = array(
             '_type' => 'text',
             'required' => true,
             'desc' => 'Username to connect to the database as',
-            'default' => $this->_default($ctx . '|username', 'horde'));
+            'default' => $this->_default($ctx . '|username', ''));
         $password = array(
             '_type' => 'text',
             'required' => false,
             'desc' => 'Password to connect with',
-            'default' => $this->_default($ctx . '|password', '*****'));
+            'default' => $this->_default($ctx . '|password', ''));
         $database = array(
             '_type' => 'text',
             'required' => true,
             'desc' => 'Database name to use',
-            'default' => $this->_default($ctx . '|database', 'horde'));
+            'default' => $this->_default($ctx . '|database', ''));
         $socket = array(
             '_type' => 'text',
             'required' => false,
@@ -1122,21 +1121,21 @@ class ConfigForm extends Horde_Form {
      * generating configuration info, things like the Token system
      * might not work correctly. This saves some headaches.
      *
-     * @var boolean $_useFormToken
+     * @var boolean
      */
     var $_useFormToken = false;
 
     /**
      * Contains the Horde_Config object that this form represents.
      *
-     * @var Horde_Config $_xmlConfig
+     * @var Horde_Config
      */
     var $_xmlConfig;
 
     /**
      * Contains the Variables object of this form.
      *
-     * @var Variables $_vars
+     * @var Variables
      */
     var $_vars;
 
