@@ -6,7 +6,7 @@ require_once 'Horde/iCalendar.php';
 /**
  * Implement the Horde_Data:: API for vCard data.
  *
- * $Horde: framework/Data/Data/vcard.php,v 1.33.10.1 2005/01/03 12:18:57 jan Exp $
+ * $Horde: framework/Data/Data/vcard.php,v 1.33.10.5 2005/07/03 04:39:57 selsky Exp $
  *
  * Copyright 1999-2005 Jan Schneider <jan@horde.org>
  *
@@ -14,7 +14,6 @@ require_once 'Horde/iCalendar.php';
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jan Schneider <jan@horde.org>
- * @version $Horde: framework/Data/Data/vcard.php,v 1.33.10.1 2005/01/03 12:18:57 jan Exp $
  * @since   Horde 3.0
  * @package Horde_Data
  */
@@ -24,23 +23,17 @@ class Horde_Data_vcard extends Horde_Data_imc {
      * Exports vcalendar data as a string. Unlike vEvent, vCard data
      * is not enclosed in BEGIN|END:vCalendar.
      *
-     * @access public
-     *
-     * @param array  $data   An array containing Horde_iCalendar_vcard objects.
-     * @param string $method (optional) The iTip method to use.
+     * @param array $data     An array containing Horde_iCalendar_vcard
+     *                        objects.
+     * @param string $method  The iTip method to use.
      *
      * @return string  The iCalendar data.
      */
     function exportData($data, $method = 'REQUEST')
     {
-        global $prefs;
-
-        $this->_iCal = &new Horde_iCalendar();
-
-        $this->_iCal->setAttribute('METHOD', $method);
         $s = '';
-        foreach ($data as $event) {
-            $s.= $event->exportvCalendar();
+        foreach ($data as $vcard) {
+            $s.= $vcard->exportvCalendar();
         }
         return $s;
     }
