@@ -24,10 +24,10 @@ define('DATATREE_ROOT', -1);
  * everywhere. Every stored object must have a different name (inside
  * each groupid).
  *
- * Required values for $params:
- * group: define each group of objects we want to build.
+ * Required values for $params:<pre>
+ *   'group'         Define each group of objects we want to build.</pre>
  *
- * $Horde: framework/DataTree/DataTree.php,v 1.151.2.2 2005/01/03 12:18:58 jan Exp $
+ * $Horde: framework/DataTree/DataTree.php,v 1.151.2.13 2005/09/21 13:02:08 jan Exp $
  *
  * Copyright 1999-2003 Stephane Huther <shuther@bigfoot.com>
  * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
@@ -46,7 +46,7 @@ class DataTree {
      * Array of all data: indexed by id. The format is: array(id =>
      * 'name' => name, 'parent' => parent).
      *
-     * @var array $_data
+     * @var array
      */
     var $_data = array();
 
@@ -54,14 +54,14 @@ class DataTree {
      * A hash that can be used to map a full object name
      * (parent:child:object) to that object's unique ID.
      *
-     * @var array $_nameMap
+     * @var array
      */
     var $_nameMap = array();
 
     /**
      * Hash containing connection parameters.
      *
-     * @var array $_params
+     * @var array
      */
     var $_params = array();
 
@@ -294,12 +294,14 @@ class DataTree {
     function &getObject($object, $class = 'DataTreeObject')
     {
         if (empty($object)) {
-            return PEAR::raiseError('No object requested.');
+            $error = PEAR::raiseError('No object requested.');
+            return $error;
         }
 
         $this->_load($object);
         if (!$this->exists($object)) {
-            return PEAR::raiseError($object . ' not found.');
+            $error = PEAR::raiseError($object . ' not found.');
+            return $error;
         }
 
         $use_attributes = is_null($class) || is_callable(array($class, '_fromAttributes'));
@@ -315,7 +317,8 @@ class DataTree {
         }
 
         if (!class_exists($class)) {
-            return PEAR::raiseError($class . ' not found.');
+            $error = PEAR::raiseError($class . ' not found.');
+            return $error;
         }
 
         $dataOb = &new $class($object);
@@ -347,7 +350,8 @@ class DataTree {
     function &getObjectById($id, $class = 'DataTreeObject')
     {
         if (empty($id)) {
-            return PEAR::raiseError('No id requested.');
+            $object = PEAR::raiseError('No id requested.');
+            return $object;
         }
 
         $result = $this->_loadById($id);
@@ -571,10 +575,8 @@ class DataTree {
     }
 
     /**
-     * Import a list of objects. Used by drivers to populate the
-     * internal $_data array.
-     *
-     * @access private
+     * Import a list of objects. Used by drivers to populate the internal
+     * $_data array.
      *
      * @param array   $data     The data to import.
      * @param string  $charset  The charset to convert the object name from.
@@ -629,7 +631,7 @@ class DataTree {
      * their order and then by name. This function is a way to get a
      * collection of $leaf's children.
      *
-     * @param optional string $leaf  Name of the parent from which to start.
+     * @param string $leaf  Name of the parent from which to start.
      *
      * @return array
      */
@@ -650,12 +652,10 @@ class DataTree {
      * Extract all levels of data, starting from a given parent
      * leaf in the datatree.
      *
-     * @param array $out                  This is an iterating function, so $out
-     *                                    is passed by reference to contain the
-     *                                    result.
-     * @param optional string  $parent    The name of the parent from which to
-     *                                    begin.
-     * @param optional integer $maxdepth  Max of levels of depth to check.
+     * @param array $out         This is an iterating function, so $out is
+     *                           passed by reference to contain the result.
+     * @param string  $parent    The name of the parent from which to begin.
+     * @param integer $maxdepth  Max of levels of depth to check.
      *
      * @note If nothing is returned that means there is no child, but
      * don't forget to add the parent if any subsequent operations are
@@ -685,12 +685,10 @@ class DataTree {
      *
      * Returned array format: array(parent => array(child => true))
      *
-     * @param array $out                  This is an iterating function, so $out
-     *                                    is passed by reference to contain the
-     *                                    result.
-     * @param optional string  $parent    The name of the parent from which to
-     *                                    begin.
-     * @param optional integer $maxdepth  Max number of levels of depth to check.
+     * @param array $out         This is an iterating function, so $out is
+     *                           passed by reference to contain the result.
+     * @param string  $parent    The name of the parent from which to begin.
+     * @param integer $maxdepth  Max number of levels of depth to check.
      */
     function _extractAllLevelList(&$out, $parent = DATATREE_ROOT, $maxdepth = -1)
     {
@@ -787,7 +785,7 @@ class DataTree {
      * $child.
      *
      * @param integer $childId  The id of the child.
-     * @param array   $parents  (optional) The array, as we build it up.
+     * @param array $parents    The array, as we build it up.
      *
      * @return array  A flat list of all of the parents of $child,
      *                hashed in $id => $name format.
@@ -835,9 +833,9 @@ class DataTree {
      * Get the number of children an object has, only counting immediate
      * children, not grandchildren, etc.
      *
-     * @param optional mixed $parent  Either the object or the
-     *                                name for which to count the
-     *                                children, defaults to the root (DATATREE_ROOT).
+     * @param mixed $parent  Either the object or the name for which to count
+     *                       the children, defaults to the root
+     *                       (DATATREE_ROOT).
      *
      * @return integer
      */
@@ -978,8 +976,6 @@ class DataTree {
     /**
      * Replace all occurences of ':' in an object name with '.'.
      *
-     * @access public
-     *
      * @param string $name  The name of the object.
      *
      * @return string  The encoded name.
@@ -990,10 +986,9 @@ class DataTree {
     }
 
     /**
-     * Get the short name of an object, returns only the last portion
-     * of the full name. For display purposes only.
+     * Get the short name of an object, returns only the last portion of the
+     * full name. For display purposes only.
      *
-     * @access public
      * @static
      *
      * @param string $name  The name of the object.
@@ -1002,13 +997,13 @@ class DataTree {
      */
     function getShortName($name)
     {
-        /* If there are several components to the name, explode and
-         * get the last one, otherwise just return the name. */
+        /* If there are several components to the name, explode and get the
+         * last one, otherwise just return the name. */
         if (strpos($name, ':') !== false) {
-            return array_pop(explode(':', $name));
-        } else {
-            return $name;
+            $name = explode(':', $name);
+            $name = array_pop($name);
         }
+        return $name;
     }
 
     /**
@@ -1016,11 +1011,12 @@ class DataTree {
      *
      * @abstract
      *
-     * @param mixed $object     The object to add (string or DataTreeObject).
-     * @param bool $id_as_name  True or false to indicate if object ID is to be
-     *                          used as object name. Used in situations where
-     *                          there is no available unique input for object
-     *                          name. Defaults to false.
+     * @param mixed $object        The object to add (string or
+     *                             DataTreeObject).
+     * @param boolean $id_as_name  True or false to indicate if object ID is to
+     *                             be used as object name. Used in situations
+     *                             where there is no available unique input for
+     *                             object name.
      */
     function add($object, $id_as_name = false)
     {
@@ -1067,37 +1063,47 @@ class DataTree {
         return PEAR::raiseError('not supported');
     }
 
+    /**
+     * Returns the number of objects matching a set of attribute criteria.
+     *
+     * @see buildAttributeQuery()
+     *
+     * @param array   $criteria   The array of criteria.
+     * @param string  $parent     The parent node to start searching from.
+     * @param boolean $allLevels  Return all levels, or just the direct
+     *                            children of $parent? Defaults to all levels.
+     * @param string  $restrict   Only return attributes with the same
+     *                            attribute_name or attribute_id.
+     */
+    function countByAttributes($criteria, $parent = DATATREE_ROOT,
+                               $allLevels = true, $restrict = 'name')
+    {
+        return PEAR::raiseError('not supported');
+    }
+
     /**
      * Returns a set of object ids based on a set of attribute criteria.
      *
-     * @abstract
+     * @see buildAttributeQuery()
      *
-     * @param array $criteria      The array of criteria. Example:
-     *                             $criteria['OR'] = array(
-     *                                 array('AND' => array(
-     *                                     array('field' => 'name',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'foo'),
-     *                                     array('field' => 'key',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'abc'))),
-     *                                 array('AND' => array(
-     *                                     array('field' => 'name',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'bar'),
-     *                                     array('field' => 'key',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'xyz'))));
-     *                             This would fetch all object ids where
-     *                             attribute name is "foo" AND key is "abc", OR
-     *                             "bar" AND "xyz".
-     * @param string $parent       The parent node to start searching from.
-     * @param bool $allLevels      Return all levels, or just the direct
-     *                             children of $parent? Defaults to all levels.
-     * @param bool $restrictNames  Only return attributes with the same
-     *                             attribute_name.
+     * @param array   $criteria     The array of criteria.
+     * @param string  $parent       The parent node to start searching from.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all levels.
+     * @param string  $restrict     Only return attributes with the same
+     *                              attribute_name or attribute_id.
+     * @param integer $from         The object to start to fetching
+     * @param integer $count        The number of objects to fetch
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      */
-    function getByAttributes($criteria, $parent = DATATREE_ROOT, $allLevels = true, $restrictNames = true)
+    function getByAttributes($criteria, $parent = DATATREE_ROOT,
+                             $allLevels = true, $restrict = 'name', $from = 0,
+                             $count = 0, $sortby_name = null,
+                             $sortby_key = null, $direction = 0)
     {
         return PEAR::raiseError('not supported');
     }
@@ -1234,7 +1240,7 @@ class DataTreeObject {
     /**
      * This object's DataTree instance.
      *
-     * @var DataTree $datatree
+     * @var DataTree
      */
     var $datatree;
 
@@ -1242,7 +1248,7 @@ class DataTreeObject {
      * Key-value hash that will be serialized.
      *
      * @see getData()
-     * @var array $data
+     * @var array
      */
     var $data = array();
 
@@ -1251,14 +1257,14 @@ class DataTreeObject {
      * These names have the same requirements as other object names - they must
      * be unique, etc.
      *
-     * @var string $name.
+     * @var string
      */
     var $name;
 
     /**
      * If this object has ordering data, store it here.
      *
-     * @var integer $order
+     * @var integer
      */
     var $order = null;
 
@@ -1350,7 +1356,7 @@ class DataTreeObject {
     /**
      * Sets the order of this object in its object collection.
      *
-     * @var integer $order
+     * @param integer $order
      */
     function setOrder($order)
     {
@@ -1403,10 +1409,8 @@ class DataTreeObject {
     }
 
     /**
-     * Saves any changes to this object to the backend
-     * permanently. New objects are added instead.
-     *
-     * @access public
+     * Saves any changes to this object to the backend permanently. New objects
+     * are added instead.
      *
      * @return boolean|PEAR_Error  PEAR_Error on failure.
      */
@@ -1422,8 +1426,6 @@ class DataTreeObject {
     /**
      * Delete this object from the backend permanently.
      *
-     * @access public
-     *
      * @return boolean|PEAR_Error  PEAR_Error on failure.
      */
     function delete()
