@@ -20,29 +20,32 @@ if (!defined('LATT_HASCHILDREN')) {
 
 /* Start at 128 for our local bitmasks to allow for the c-client LATT_*
    constants. */
-/** @const IMAPTREE_ELT_HAS_CHILDREN */  define('IMAPTREE_ELT_HAS_CHILDREN', 128);
-/** @const IMAPTREE_ELT_IS_OPEN */       define('IMAPTREE_ELT_IS_OPEN', 256);
-/** @const IMAPTREE_ELT_IS_SUBSCRIBED */ define('IMAPTREE_ELT_IS_SUBSCRIBED', 512);
-/** @const IMAPTREE_ELT_IS_DISCOVERED */ define('IMAPTREE_ELT_IS_DISCOVERED', 1024);
-/** @const IMAPTREE_ELT_IS_POLLED */     define('IMAPTREE_ELT_IS_POLLED', 2048);
-/** @const IMAPTREE_ELT_NEED_SORT */     define('IMAPTREE_ELT_NEED_SORT', 4096);
+define('IMAPTREE_ELT_HAS_CHILDREN', 128);
+define('IMAPTREE_ELT_IS_OPEN', 256);
+define('IMAPTREE_ELT_IS_SUBSCRIBED', 512);
+define('IMAPTREE_ELT_IS_DISCOVERED', 1024);
+define('IMAPTREE_ELT_IS_POLLED', 2048);
+define('IMAPTREE_ELT_NEED_SORT', 4096);
 
 /* The isOpen() expanded mode constants. */
-/** @const IMAPTREE_OPEN_NONE */ define('IMAPTREE_OPEN_NONE', 0);
-/** @const IMAPTREE_OPEN_ALL */  define('IMAPTREE_OPEN_ALL', 1);
-/** @const IMAPTREE_OPEN_USER */ define('IMAPTREE_OPEN_USER', 2);
+define('IMAPTREE_OPEN_NONE', 0);
+define('IMAPTREE_OPEN_ALL', 1);
+define('IMAPTREE_OPEN_USER', 2);
 
 /* Which mode of IMAP access are we using. */
-/** @const IMAPTREE_MODE_MAIL */ define('IMAPTREE_MODE_MAIL', 0);
-/** @const IMAPTREE_MODE_NEWS */ define('IMAPTREE_MODE_NEWS', 1);
+define('IMAPTREE_MODE_MAIL', 0);
+define('IMAPTREE_MODE_NEWS', 1);
 
 /* The initalization mode to use when creating the tree. */
-/** @const IMAPTREE_INIT_SUB */            define('IMAPTREE_INIT_SUB', 1);
-/** @const IMAPTREE_INIT_UNSUB */          define('IMAPTREE_INIT_UNSUB', 2);
-/** @const IMAPTREE_INIT_SUB_FETCHALL */   define('IMAPTREE_INIT_FETCHALL', 4);
+define('IMAPTREE_INIT_SUB', 1);
+define('IMAPTREE_INIT_UNSUB', 2);
+define('IMAPTREE_INIT_FETCHALL', 4);
 
 /* The manner to which to traverse the tree when calling next(). */
-/** @const IMAPTREE_NEXT_SHOWCLOSED */   define('IMAPTREE_NEXT_SHOWCLOSED', 1);
+define('IMAPTREE_NEXT_SHOWCLOSED', 1);
+
+/* The string used to indicate the base of the tree. */
+define('IMAPTREE_BASE_ELT', null);
 
 /**
  * The IMAP_Tree class provides a tree view of the folders supported with
@@ -50,7 +53,7 @@ if (!defined('LATT_HASCHILDREN')) {
  * functions to iterate through this tree and query information about
  * individual mailboxes/folders.
  *
- * $Horde: framework/IMAP/IMAP/Tree.php,v 1.48.2.6 2005/02/01 23:54:34 slusarz Exp $
+ * $Horde: framework/IMAP/IMAP/Tree.php,v 1.48.2.28 2005/08/24 05:22:03 slusarz Exp $
  *
  * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2000-2005 Jon Parise <jon@horde.org>
@@ -64,7 +67,6 @@ if (!defined('LATT_HASCHILDREN')) {
  * @author  Jon Parise <jon@horde.org>
  * @author  Anil Madhavapeddy <avsm@horde.org>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.48.2.6 $
  * @since   Horde 3.0
  * @package Horde_IMAP
  */
@@ -73,88 +75,91 @@ class IMAP_Tree {
     /**
      * Associative array containing the mailbox tree.
      *
-     * @var array $_tree
+     * @var array
      */
     var $_tree;
 
     /**
      * Location of current element in the tree.
      *
-     * @var string $_currparent
-     * @var integer $_currkey
-     * @var array $_currstack
+     * @var string
      */
     var $_currparent = null;
-    var $_currkey = null;
-    var $_currstack = array();
 
     /**
-     * Key of first (top?) element in the tree (for traversal).
+     * Location of current element in the tree.
      *
-     * @var string $_first
+     * @var integer
      */
-    var $_first = '';
+    var $_currkey = null;
+
+    /**
+     * Location of current element in the tree.
+     *
+     * @var array
+     */
+    var $_currstack = array();
 
     /**
      * Show unsubscribed mailboxes?
      *
-     * @var boolean $_showunsub
+     * @var boolean
      */
     var $_showunsub = false;
 
     /**
      * The prefix without a trailing delimiter.
      *
-     * @var string $_prefixnodelim
+     * @var string
      */
     var $_prefixnodelim = '';
 
     /**
      * Parent list.
      *
-     * @var array $_sortcache
+     * @var array
      */
     var $_parent = array();
 
     /**
      * The cached list of mailboxes to poll.
      *
-     * @var array $_poll
+     * @var array
      */
     var $_poll = null;
 
     /**
      * The cached list of expanded folders.
      *
-     * @var array $_expanded
+     * @var array
      */
     var $_expanded = null;
 
     /**
      * Cached list of subscribed mailboxes.
      *
-     * @var array $_subscribed
+     * @var array
      */
     var $_subscribed = null;
 
     /**
      * Cached list of unsubscribed mailboxes.
      *
-     * @var array $_unsubscribed
+     * @var array
      */
     var $_unsubscribed = null;
 
     /**
      * Init mode flag.
      *
-     * @var integer $_initmode
+     * @var integer
      */
     var $_initmode = 0;
 
     /**
      * Tree changed flag.  Set when something in the tree has been altered.
      *
-     * @var boolean $_changed
+     * @var boolean
      */
     var $_changed = false;
 
@@ -162,28 +167,28 @@ class IMAP_Tree {
      * Does this server support the LATT_HASCHILDREN/LATT_HASNOCHILDREN
      * constants?
      *
-     * @var boolean $_children
+     * @var boolean
      */
     var $_children = null;
 
     /**
      * Have we shown unsubscribed folders previously?
      *
-     * @var boolean $_unsubview
+     * @var boolean
      */
     var $_unsubview = false;
 
     /**
      * Cached results of hasChildren().
      *
-     * @var array $_hasChildrenCache
+     * @var array
      */
     var $_hasChildrenCache = array();
 
     /**
      * The IMAP_Sort object.
      *
-     * @var object IMAP_Sort $_imap_sort
+     * @var IMAP_Sort
      */
     var $_imap_sort = null;
 
@@ -192,14 +197,14 @@ class IMAP_Tree {
      * If set, IMAP_Tree:: will call _getNonIMAPElt() to obtain the element
      * to add to the tree.
      *
-     * @var boolean $_nonimapelt
+     * @var boolean
      */
     var $_nonimapelt = false;
 
     /**
      * The name to use when storing the object in the session.
      *
-     * @var string $_cachename
+     * @var string
      */
     var $_cachename;
 
@@ -207,7 +212,7 @@ class IMAP_Tree {
      * The application that generated this tree.
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var string $_app
+     * @var string
      */
     var $_app = null;
 
@@ -215,7 +220,7 @@ class IMAP_Tree {
      * The server string for the current server.
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var string $_server
+     * @var string
      */
     var $_server = '';
 
@@ -223,7 +228,7 @@ class IMAP_Tree {
      * The server string used for the delimiter.
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var string $_delimiter
+     * @var string
      */
     var $_delimiter = '/';
 
@@ -231,7 +236,7 @@ class IMAP_Tree {
      * Where we start listing folders.
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var string $_prefix
+     * @var string
      */
     var $_prefix = '';
 
@@ -239,7 +244,7 @@ class IMAP_Tree {
      * The location of the first level of folders below the INBOX.
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var string $_namespace
+     * @var string
      */
     var $_namespace = '';
 
@@ -247,7 +252,7 @@ class IMAP_Tree {
      * Should dotfiles be shown?
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var boolean $_dotfiles
+     * @var boolean
      */
     var $_dotfiles = false;
 
@@ -255,32 +260,30 @@ class IMAP_Tree {
      * Should we use 'mail' mode or 'news' mode?
      * THIS SHOULD BE SET IN EVERY SUBCLASS CONSTRUCTOR.
      *
-     * @var integer $_mode
+     * @var integer
      */
     var $_mode = null;
 
     /**
      * Attempts to return a reference to a concrete IMAP_Tree instance.
+     *
      * If an IMAP_Tree object is currently stored in the local session,
      * recreate that object.  Else, if $create is true, will create a new
      * instance.  Ensures that only one IMAP_Tree instance is available
      * at any time.
      *
-     * This method must be invoked as:
-     *   $imap_tree = &IMAP_Tree::singleton($app, $classname, [$create[, $unsub]]);
+     * This method must be invoked as:<pre>
+     *   $imap_tree = &IMAP_Tree::singleton($app, $classname, [$create[, $unsub]]);</pre>
      *
-     * @access public
+     * @param string $app        The current application name.
+     * @param string $classname  The class name to use when instantiating a new
+     *                           object.
+     * @param boolean $create    Create a new IMAP_Tree if it doesn't exist in
+     *                           the session?
+     * @param integer $init      The initialization mode to use.
+     * @param string $cachename  The name to use when storing in the session.
      *
-     * @param string $app                 The current application name.
-     * @param string $classname           The class name to use when
-     *                                    instantiating a new object.
-     * @param optional boolean $create    Create a new IMAP_Tree if it doesn't
-     *                                    exist in the session?
-     * @param optional integer $init      The initialization mode to use.
-     * @param optional string $cachename  The name to use when storing in the
-     *                                    session.
-     *
-     * @return object IMAP_Tree  The IMAP_Tree object or null.
+     * @return IMAP_Tree  The IMAP_Tree object or null.
      */
     function &singleton($app, $classname, $create = false,
                         $init = IMAPTREE_INIT_SUB, $cachename = 'imaptree')
@@ -306,11 +309,8 @@ class IMAP_Tree {
     /**
      * Constructor.
      *
-     * @access public
-     *
-     * @param optional integer $init      The initialization mode to use.
-     * @param optional string $cachename  The name to use when storing in the
-     *                                    session.
+     * @param integer $init      The initialization mode to use.
+     * @param string $cachename  The name to use when storing in the session.
      */
     function IMAP_Tree($init = IMAPTREE_INIT_SUB, $cachename = 'imaptree')
     {
@@ -368,21 +368,16 @@ class IMAP_Tree {
      */
     function _getList($path)
     {
-        $newboxes = call_user_func(($this->_showunsub) ? 'imap_getmailboxes' : 'imap_getsubscribed', $this->_getStream(), $this->_server, $path);
-
         $unique = array();
 
+        $newboxes = call_user_func(($this->_showunsub) ? 'imap_getmailboxes' : 'imap_getsubscribed', $this->_getStream(), $this->_server, $path);
         if (is_array($newboxes)) {
             foreach ($newboxes as $box) {
-                if (!empty($box->name)) {
+                if (!empty($box->name) && !empty($box->delimiter)) {
                     /* Strip off server string. */
                     $box = $this->_removeServerString($box);
-                    if ($box->name) {
-                        if (strcasecmp($box->name, 'INBOX') == 0) {
-                            $unique['INBOX'] = $box;
-                        } else {
-                            $unique[$box->name] = $box;
-                        }
+                    if ($box->name && !isset($unique[$box->name])) {
+                        $unique[$box->name] = $box;
                     }
                 }
             }
@@ -403,7 +398,7 @@ class IMAP_Tree {
      */
     function _getMailbox($path)
     {
-        $box = (!$this->_showunsub && (($this->_mode == IMAPTREE_MODE_NEWS) || (strcasecmp($path, 'INBOX') != 0))) ? @imap_getsubscribed($this->_getStream(), $this->_server, $path) : @imap_getmailboxes($this->_getStream(), $this->_server, $path);
+        $box = (!$this->_showunsub && ($this->_convertName($path) != 'INBOX')) ? @imap_getsubscribed($this->_getStream(), $this->_server, $path) : @imap_getmailboxes($this->_getStream(), $this->_server, $path);
 
         if (empty($box)) {
             return false;
@@ -440,14 +435,13 @@ class IMAP_Tree {
     function _makeMailboxTreeElt($ob)
     {
         $elt['a'] = $ob->attributes;
-        $elt['p'] = null;
+        $elt['p'] = IMAPTREE_BASE_ELT;
         $elt['v'] = $ob->name;
 
         /* Set subscribed values. Make sure INBOX is always subscribed
          * to (if working with mail) so there is always at least 1
          * viewable element. */
-        if (($this->_mode == IMAPTREE_MODE_MAIL) &&
-            (strcasecmp($ob->name, 'INBOX') == 0)) {
+        if ($elt['v'] == 'INBOX') {
             $this->_initSubscribed();
             $this->_setSubscribed($elt, true);
         } elseif (!$this->isSubscribed($elt)) {
@@ -483,8 +477,6 @@ class IMAP_Tree {
     /**
      * Initalize the list at the top level of the hierarchy.
      *
-     * @access public
-     *
      * @param integer $init  The initialization mode.
      */
     function init($init)
@@ -494,7 +486,6 @@ class IMAP_Tree {
         $this->_currkey = null;
         $this->_currparent = null;
         $this->_currstack = array();
-        $this->_first = null;
         $this->_tree = array();
         $this->_parent = array();
         $this->_showunsub = $this->_unsubview = ($init & IMAPTREE_INIT_UNSUB);
@@ -509,6 +500,10 @@ class IMAP_Tree {
         /* Get the initial list of mailboxes from the subclass. */
         $boxes = $this->_init();
 
+        /* Create a placeholder element to the base of the tree list so we can
+         * keep track of whether the base level needs to be sorted. */
+        $this->_tree[IMAPTREE_BASE_ELT] = array('a' => 0);
+
         /* Create the list (INBOX and any other hierarchies). */
         $this->_addLevel($boxes);
 
@@ -534,8 +529,6 @@ class IMAP_Tree {
     /**
      * Make sure there is no trailing delimiter on the element name.
      *
-     * @access public
-     *
      * @param string $name  The element name.
      *
      * @return string  The element name with any trailing delimiter stripped
@@ -553,13 +546,13 @@ class IMAP_Tree {
     /**
      * Expand a mail folder.
      *
-     * @access public
-     *
-     * @param string $folder               The folder name to expand.
-     * @param optional boolean $expandall  Expand all folders under this one?
+     * @param string $folder      The folder name to expand.
+     * @param boolean $expandall  Expand all folders under this one?
      */
     function expand($folder, $expandall = false)
     {
+        $folder = $this->_convertName($folder);
+
         if (!isset($this->_tree[$folder])) {
             return;
         }
@@ -598,12 +591,12 @@ class IMAP_Tree {
     /**
      * Collapse a mail folder.
      *
-     * @access public
-     *
      * @param string $folder  The folder name to collapse.
      */
     function collapse($folder)
     {
+        $folder = $this->_convertName($folder);
+
         if (!isset($this->_tree[$folder])) {
             return;
         }
@@ -618,11 +611,9 @@ class IMAP_Tree {
      * Sets the internal array pointer to the next element, and returns the
      * next object.
      *
-     * @access public
-     *
-     * @param optional integer $mask  A mask with the following elements:
+     * @param integer $mask  A mask with the following elements:
      * <pre>
-     * IMAPTREE_NEXT_SHOWCLOSED - Dont't ignore closed elements.
+     * IMAPTREE_NEXT_SHOWCLOSED - Don't ignore closed elements.
      * </pre>
      *
      * @return mixed  Returns the next element or false if the element doesn't
@@ -638,15 +629,15 @@ class IMAP_Tree {
 
         if ($this->_activeElt($curr) &&
             (($mask & IMAPTREE_NEXT_SHOWCLOSED) || $this->isOpen($curr)) &&
-            ($this->_currparent != $curr['value'])) {
+            ($this->_currparent != $curr['v'])) {
             /* If the current element is open, and children exist, move into
              * it. */
-            array_push($this->_currstack, array('k' => $this->_currkey, 'p' => $this->_currparent));
+            $this->_currstack[] = array('k' => $this->_currkey, 'p' => $this->_currparent);
             $this->_currkey = 0;
-            $this->_currparent = $curr['value'];
-            if ($this->_needSort($this->_tree[$curr['value']])) {
-                $this->_sortList($this->_parent[$curr['value']]);
-                $this->_setNeedSort($this->_tree[$curr['value']], false);
+            $this->_currparent = $curr['v'];
+            if ($this->_needSort($this->_tree[$curr['v']])) {
+                $this->_sortList($this->_parent[$curr['v']]);
+                $this->_setNeedSort($this->_tree[$curr['v']], false);
                 $this->_changed = true;
             }
         } else {
@@ -689,26 +680,29 @@ class IMAP_Tree {
      * Set internal pointer to the head of the tree.
      * This MUST be called before you can traverse the tree with next().
      *
-     * @access public
-     *
      * @return mixed  Returns the element at the head of the tree or false
      *                if the element doesn't exist.
      */
     function reset()
     {
-        foreach ($this->_parent as $key => $val) {
-            $key2 = array_search($this->_first, $val);
-            if ($key2 !== null) {
-                $this->_currkey = $key2;
-                $this->_currparent = $key;
-                $this->_currstack = array();
-                if (isset($this->_tree[$key]) &&
-                    $this->_needSort($this->_tree[$key])) {
-                    $this->_sortList($this->_parent[$key]);
-                    $this->_setNeedSort($this->_tree[$key], false);
-                    $this->_changed = true;
-                }
-                break;
+        $this->_currkey = 0;
+        $this->_currparent = null;
+        $this->_currstack = array();
+
+        if ($this->_needSort($this->_tree[$this->_currparent])) {
+            $this->_sortList($this->_parent[$this->_currparent]);
+            $this->_setNeedSort($this->_tree[$this->_currparent], false);
+            $this->_changed = true;
+        }
+
+        /* When resetting in news mode, the first element may not be
+         * viewable (unlike mail mode, where the first element is INBOX and
+         * is always viewable).  Therefore, we need to search for the first
+         * viewable item and that will be the base of the viewable tree. */
+        if ($this->_mode == IMAPTREE_MODE_NEWS) {
+            $curr = $this->current();
+            if (!$this->_activeElt($curr)) {
+                $this->next();
             }
         }
 
@@ -718,8 +712,6 @@ class IMAP_Tree {
     /**
      * Return the current tree element.
      *
-     * @access public
-     *
      * @return array  The current tree element or false if there is no
      *                element.
      */
@@ -735,8 +727,6 @@ class IMAP_Tree {
     /**
      * Determines if there are more elements in the current tree level.
      *
-     * @access public
-     *
      * @return boolean  True if there are more elements, false if this is the
      *                  last element.
      */
@@ -755,7 +745,7 @@ class IMAP_Tree {
     /**
      * Adds aliases to a tree element and returns the resulting array.
      *
-     * @access private
+     * @access protected
      *
      * @param array $elt  A tree element.
      *
@@ -774,14 +764,14 @@ class IMAP_Tree {
     /**
      * Returns the requested element.
      *
-     * @access public
-     *
      * @param string $name  The name of the tree element.
      *
      * @return array  Returns the requested element or false if not found.
      */
     function get($name)
     {
+        $name = $this->_convertName($name);
+
         if (isset($this->_tree[$name])) {
             return $this->_addAliases($this->_tree[$name]);
         } else {
@@ -792,8 +782,6 @@ class IMAP_Tree {
     /**
      * Insert a folder/mailbox into the tree.
      *
-     * @access public
-     *
      * @param mixed $id  The name of the folder (or a list of folder names)
      *                   to add (must be present on the mail server).
      *
@@ -840,14 +828,11 @@ class IMAP_Tree {
         /* Break apart the name via the delimiter and go step by step through
          * the name to make sure all subfolders exist in the tree. */
         $parts = explode($this->_delimiter, $id);
+        $parts[0] = $this->_convertName($parts[0]);
         $begin = implode($this->_delimiter, array_slice($parts, 0, $pos));
         $more = (count($parts) > $pos);
 
-        if (isset($this->_tree[$begin])) {
-            if (!$more) {
-                $this->_setContainer($this->_tree[$begin], false);
-            }
-        } elseif ($begin != $this->_prefixnodelim) {
+        if (!isset($this->_tree[$begin]) && ($begin != $this->_prefixnodelim)) {
             if ($this->_nonimapelt) {
                 $elt = $this->_getNonIMAPElt($begin);
             } else {
@@ -909,8 +894,6 @@ class IMAP_Tree {
     /**
      * Delete an element from the tree.
      *
-     * @access public
-     *
      * @param mixed $id  The element name or an array of element names.
      *
      * @return boolean  Return true on success, false on error.
@@ -931,10 +914,11 @@ class IMAP_Tree {
                 }
             }
             return $success;
+        } else {
+            $id = $this->_convertName($id);
         }
 
-        if ((($this->_mode == IMAPTREE_MODE_MAIL) &&
-             (strcasecmp($id, 'INBOX') == 0)) ||
+        if (($id == 'INBOX') ||
             !isset($this->_tree[$id]) ||
             ($id == $this->_prefixnodelim)) {
             return false;
@@ -966,9 +950,13 @@ class IMAP_Tree {
              * the tree. */
             unset($this->_parent[$parent]);
             if (isset($this->_tree[$parent])) {
-                $this->_setChildren($this->_tree[$parent], false);
                 if ($this->isContainer($this->_tree[$parent])) {
                     $this->delete($parent);
+                } elseif (($parent != 'INBOX') ||
+                          !$this->hasChildren($this->_tree['INBOX'])) {
+                    $this->_removeExpandedList($parent);
+                    $this->_setChildren($this->_tree[$parent], false);
+                    $this->_setOpen($this->_tree[$parent], false);
                 }
             }
         } else {
@@ -988,8 +976,6 @@ class IMAP_Tree {
     /**
      * Subscribe an element to the tree.
      *
-     * @access public
-     *
      * @param mixed $id  The element name or an array of element names.
      */
     function subscribe($id)
@@ -999,6 +985,7 @@ class IMAP_Tree {
         }
 
         foreach ($id as $val) {
+            $val = $this->_convertName($val);
             if (isset($this->_tree[$val])) {
                 $this->_changed = true;
                 $this->_setSubscribed($this->_tree[$val], true);
@@ -1016,8 +1003,6 @@ class IMAP_Tree {
     /**
      * Unsubscribe an element from the tree.
      *
-     * @access public
-     *
      * @param mixed $id  The element name or an array of element names.
      */
     function unsubscribe($id)
@@ -1032,11 +1017,12 @@ class IMAP_Tree {
         }
 
         foreach ($id as $val) {
+            $val = $this->_convertName($val);
+
             /* INBOX can never be unsubscribed to (if in mail mode). */
-            if (isset($this->_tree[$val]) &&
-                (($this->_mode != IMAPTREE_MODE_MAIL) ||
-                 (strcasecmp($val, 'INBOX') != 0))) {
+            if (isset($this->_tree[$val]) && ($val != 'INBOX')) {
                 $this->_changed = true;
+                $this->_unsubview = true;
 
                 $elt = &$this->_tree[$val];
 
@@ -1068,22 +1054,13 @@ class IMAP_Tree {
     function _addLevel($list)
     {
         $expandall = ($this->_initmode & IMAPTREE_INIT_FETCHALL);
-        foreach ($list as $key => $val) {
-            $elt = $this->_makeMailboxTreeElt($val);
-            if ($key != $this->_prefix) {
-                /* We need to convert $parent to a string because, if there
-                 * is no parent (NULL), this is stored as the empty string in
-                 * the parent array. */
-                $parent = strval($elt['p']);
-                if ($this->_insertElt($elt) && empty($this->_first)) {
-                    $this->_first = $elt['l'];
+        $isparent = false;
 
-                    /* If this is the base level, add a placeholder element to
-                     * the tree list so we can keep track of whether the base
-                     * level needs to be sorted. */
-                    $this->_tree[$parent] = array('a' => 0);
-                }
-            }
+        foreach ($list as $val) {
+            $elt = $this->_makeMailboxTreeElt($val);
+            $isparent = true;
+            $parent = $elt['p'];
+            $this->_insertElt($elt);
 
             if ($expandall || $this->isOpen($elt)) {
                 $this->expand($elt['v']);
@@ -1093,7 +1070,7 @@ class IMAP_Tree {
         }
 
         /* Sort the list. */
-        if (isset($parent) &&
+        if ($isparent &&
             !empty($this->_parent[$parent]) &&
             (count($this->_parent[$parent]) > 1)) {
             $this->_setNeedSort($this->_tree[$parent], true);
@@ -1121,11 +1098,9 @@ class IMAP_Tree {
     /**
      * Does the element have any children?
      *
-     * @access public
-     *
-     * @param array $elt                  A tree element.
-     * @param optional boolean $viewable  Only return true if this element
-     *                                    has viewable children?
+     * @param array $elt         A tree element.
+     * @param boolean $viewable  Only return true if this element has viewable
+     *                           children?
      *
      * @return integer  Non-zero if the element has children.
      */
@@ -1134,6 +1109,17 @@ class IMAP_Tree {
         $ret = (($elt['a'] & IMAPTREE_ELT_HAS_CHILDREN) ||
                 ($this->_children && ($elt['a'] & LATT_HASCHILDREN)));
 
+        /* CHECK: If we are viewing all folders, and there is a folder listed
+         * as expanded but it does not contain any children, then we should
+         * remove it from the expanded list since it doesn't exist anymore. */
+        if ($this->_showunsub && !$ret) {
+            $this->_initExpandedList();
+            if (!empty($this->_expanded[$elt['v']])) {
+                $this->_removeExpandedList($elt['v']);
+                $this->_setOpen($this->_tree[$elt['v']], false);
+            }
+        }
+
         if (!$viewable || !$ret) {
             return $ret;
         }
@@ -1154,7 +1140,7 @@ class IMAP_Tree {
 
         $retvalue = false;
         if (!$this->isDiscovered($elt) ||
-            ($this->hasChildren($elt) &&
+            ($ret &&
              empty($this->_parent[$elt['v']]))) {
             $info = $this->_childrenInfo($elt['v']);
             $retvalue = (!empty($info['haschildren']));
@@ -1195,8 +1181,6 @@ class IMAP_Tree {
     /**
      * Has the tree element been discovered?
      *
-     * @access public
-     *
      * @param array $elt  A tree element.
      *
      * @return integer  Non-zero if the element has been discovered.
@@ -1222,8 +1206,6 @@ class IMAP_Tree {
     /**
      * Is the tree element open?
      *
-     * @access public
-     *
      * @param array $elt  A tree element.
      *
      * @return integer  True if the element is open.
@@ -1275,8 +1257,6 @@ class IMAP_Tree {
      * Is this element a container only, not a mailbox (meaning you can
      * not open it)?
      *
-     * @access public
-     *
      * @param array $elt  A tree element.
      *
      * @return integer  True if the element is a container.
@@ -1302,8 +1282,6 @@ class IMAP_Tree {
     /**
      * Is the user subscribed to this element?
      *
-     * @access public
-     *
      * @param array $elt  A tree element.
      *
      * @return integer  True if the user is subscribed to the element.
@@ -1329,8 +1307,6 @@ class IMAP_Tree {
     /**
      * Return the prefix.
      *
-     * @access public
-     *
      * @return string  The prefix where folders begin to be listed.
      */
     function getPrefix()
@@ -1352,7 +1328,7 @@ class IMAP_Tree {
     function _removeServerString($ob)
     {
         $ob->fullServerPath = $ob->name;
-        $ob->name = substr($ob->name, strpos($ob->name, '}') + 1);
+        $ob->name = $this->_convertName(substr($ob->name, strpos($ob->name, '}') + 1));
         return $ob;
     }
 
@@ -1403,8 +1379,6 @@ class IMAP_Tree {
      *
      * @abstract
      *
-     * @access public
-     *
      * @return array  The list of elements to poll (name in key field).
      */
     function getPollList()
@@ -1419,8 +1393,6 @@ class IMAP_Tree {
      *
      * @abstract
      *
-     * @access public
-     *
      * @param mixed $id  The element name or a list of element names to add.
      */
     function addPollList($id)
@@ -1433,8 +1405,6 @@ class IMAP_Tree {
      *
      * @abstract
      *
-     * @access public
-     *
      * @param string $id  The folder/mailbox or a list of folders/mailboxes
      *                    to remove.
      */
@@ -1445,8 +1415,6 @@ class IMAP_Tree {
     /**
      * Does the user want to poll this mailbox for new/unseen messages?
      *
-     * @access public
-     *
      * @param array $elt  A tree element.
      *
      * @return integer  True if the user wants to poll the element.
@@ -1485,8 +1453,6 @@ class IMAP_Tree {
     /**
      * Does this element's children need sorting?
      *
-     * @access public
-     *
      * @param array $elt  A tree element.
      *
      * @return integer  True if the children need to be sorted.
@@ -1545,8 +1511,6 @@ class IMAP_Tree {
 
     /**
      * Should we expand all elements?
-     *
-     * @access public
      */
     function expandAll()
     {
@@ -1557,14 +1521,21 @@ class IMAP_Tree {
 
     /**
      * Should we collapse all elements?
-     *
-     * @access public
      */
     function collapseAll()
     {
-        foreach ($this->_tree as $val) {
-            $this->collapse($val['v']);
+        foreach ($this->_tree as $key => $val) {
+            if ($key != IMAPTREE_BASE_ELT) {
+                $this->collapse($val['v']);
+            }
         }
+
+        /* Clear all entries from the expanded list. */
+        $this->_initExpandedList();
+        foreach ($this->_expanded as $key => $val) {
+            $this->_removeExpandedList($key);
+        }
+
     }
 
     /**
@@ -1572,11 +1543,10 @@ class IMAP_Tree {
      *
      * @access private
      *
-     * @param string $id              The current mailbox.
-     * @param optional boolean $list  Should we return the list of mailboxes,
-     *                                even if we can use the LATT_HASCHILDREN
-     *                                constant?
-     * @param optional boolean $set   Should we set the has children flag?
+     * @param string $id     The current mailbox.
+     * @param boolean $list  Should we return the list of mailboxes, even if we
+     *                       can use the LATT_HASCHILDREN constant?
+     * @param boolean $set   Should we set the has children flag?
      *
      * @return array  A list of mailbox objects or the empty list.
      *                See _getList() for format.
@@ -1594,8 +1564,7 @@ class IMAP_Tree {
         }
 
         if (!$tried || ($info['haschildren'] && $list)) {
-            if (($this->_mode == IMAPTREE_MODE_MAIL) &&
-                (strcasecmp($id, 'INBOX') == 0) &&
+            if (($id == 'INBOX') &&
                 (!empty($this->_prefix) || !empty($this->_namespace))) {
                 if (!empty($this->_prefix)) {
                     $query = $this->_prefix . '%';
@@ -1610,7 +1579,7 @@ class IMAP_Tree {
             if (!$info['haschildren']) {
                 if (!empty($info['list'])) {
                     $info['haschildren'] = true;
-                } elseif (strcasecmp($id, 'INBOX') == 0) {
+                } elseif ($id == 'INBOX') {
                     /* Check to see if INBOX's children reside under the
                      * same level as the INBOX. */
                     $ret = $this->_getList('%');
@@ -1629,8 +1598,6 @@ class IMAP_Tree {
     /**
      * Switch subscribed/unsubscribed viewing.
      *
-     * @access public
-     *
      * @param boolean $unsub  Show unsubscribed elements?
      */
     function showUnsubscribed($unsub)
@@ -1640,6 +1607,7 @@ class IMAP_Tree {
         }
 
         $this->_showunsub = $unsub;
+        $this->_changed = true;
 
         /* If we are switching from unsubscribed to subscribed, no need
          * to do anything (we just ignore unsubscribed stuff). */
@@ -1692,10 +1660,7 @@ class IMAP_Tree {
     }
 
     /**
-     * Get information about new/unseen/total messages for the given
-     * element.
-     *
-     * @access public
+     * Get information about new/unseen/total messages for the given element.
      *
      * @param string $name  The element name.
      *
@@ -1725,9 +1690,9 @@ class IMAP_Tree {
      *
      * @access private
      *
-     * @param array &$mbox           The list of mailboxes to sort.
-     * @param optional boolean $key  Are the list of mailbox names in the key
-     *                               field of $mbox?
+     * @param array &$mbox  The list of mailboxes to sort.
+     * @param boolean $key  Are the list of mailbox names in the key field of
+     *                      $mbox?
      */
     function _sortList(&$mbox, $key = false)
     {
@@ -1766,15 +1731,30 @@ class IMAP_Tree {
      * @access private
      *
      * @param array $elt  A tree element.
-     * 
+     *
      * @return boolean  True if it is an active element.
      */
     function _activeElt($elt)
     {
         return ($this->_showunsub ||
                 $this->isSubscribed($elt) ||
-                ($this->isContainer($elt) &&
+                (($this->_unsubview || $this->isContainer($elt)) &&
                  $this->hasChildren($elt, true)));
     }
 
+    /**
+     * Convert a mailbox name to the correct, internal name (i.e. make sure
+     * INBOX is always capitalized for IMAP servers).
+     *
+     * @access private
+     *
+     * @param string $name  The mailbox name.
+     *
+     * @return string  The converted name.
+     */
+    function _convertName($name)
+    {
+        return (($this->_mode == IMAPTREE_MODE_MAIL) && (strcasecmp($name, 'INBOX') == 0)) ? 'INBOX' : $name;
+    }
+
 }
