@@ -3,30 +3,44 @@
 require_once 'Horde/String.php';
 require_once dirname(__FILE__) . '/../MIME.php';
 
-/** @constant MIME_PART_EOL The character(s) used internally for EOLs. */
+/**
+ * The character(s) used internally for EOLs.
+ */
 define('MIME_PART_EOL', "\n");
 
-/** @constant MIME_PART_RFC_EOL The character string designated by RFCs 822/2045 to designate EOLs in MIME messages. */
+/**
+ * The character string designated by RFCs 822/2045 to designate EOLs in MIME
+ * messages.
+ */
 define('MIME_PART_RFC_EOL', "\r\n");
 
 /* Default MIME parameters. */
-/** @constant MIME_DEFAULT_CHARSET The default MIME character set. */
+
+/**
+ * The default MIME character set.
+ */
 define('MIME_DEFAULT_CHARSET', 'us-ascii');
 
-/** @constant MIME_DEFAULT_DESCRIPTION The default MIME description. */
+/**
+ * The default MIME description.
+ */
 define('MIME_DEFAULT_DESCRIPTION', _("unnamed"));
 
-/** @constant MIME_DEFAULT_DISPOSITION The default MIME disposition. */
+/**
+ * The default MIME disposition.
+ */
 define('MIME_DEFAULT_DISPOSITION', 'inline');
 
-/** @constant MIME_DEFAULT_ENCODING The default MIME encoding. */
+/**
+ * The default MIME encoding.
+ */
 define('MIME_DEFAULT_ENCODING', '7bit');
 
 /**
  * The MIME_Part:: class provides a wrapper around MIME parts and methods
  * for dealing with them.
  *
- * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.2 2005/02/07 00:37:37 chuck Exp $
+ * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.8 2005/07/11 17:10:15 jan Exp $
  *
  * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
@@ -45,7 +59,7 @@ class MIME_Part {
      * The type (ex.: text) of this part.
      * Per RFC 2045, the default is 'application'.
      *
-     * @var string $_type
+     * @var string
      */
     var $_type = 'application';
 
@@ -53,91 +67,91 @@ class MIME_Part {
      * The subtype (ex.: plain) of this part.
      * Per RFC 2045, the default is 'octet-stream'.
      *
-     * @var string $_subtype
+     * @var string
      */
     var $_subtype = 'octet-stream';
 
     /**
      * The body of the part.
      *
-     * @var string $_contents
+     * @var string
      */
     var $_contents = '';
 
     /**
      * The desired transfer encoding of this part.
      *
-     * @var string $_transferEncoding
+     * @var string
      */
     var $_transferEncoding = MIME_DEFAULT_ENCODING;
 
     /**
      * The current transfer encoding of the contents of this part.
      *
-     * @var string $_currentEncoding
+     * @var string
      */
     var $_currentEncoding = null;
 
     /**
      * Should the message be encoded via 7-bit?
      *
-     * @var boolean $_encode7bit
+     * @var boolean
      */
     var $_encode7bit = true;
 
     /**
      * The description of this part.
      *
-     * @var string $_description
+     * @var string
      */
     var $_description = '';
 
     /**
      * The disposition of this part (inline or attachment).
      *
-     * @var string $_disposition
+     * @var string
      */
     var $_disposition = MIME_DEFAULT_DISPOSITION;
 
     /**
      * The disposition parameters of this part.
      *
-     * @var array $_dispositionParameters
+     * @var array
      */
     var $_dispositionParameters = array();
 
     /**
      * The content type parameters of this part.
      *
-     * @var array $_contentTypeParameters
+     * @var array
      */
     var $_contentTypeParameters = array();
 
     /**
      * The subparts of this part.
      *
-     * @var array $_parts
+     * @var array
      */
     var $_parts = array();
 
     /**
      * Information/Statistics on the subpart.
      *
-     * @var array $_information
+     * @var array
      */
     var $_information = array();
 
     /**
      * The list of CIDs for this part.
      *
-     * @var array $_cids
+     * @var array
      */
     var $_cids = array();
 
     /**
      * The MIME ID of this part.
      *
-     * @var string $_mimeid
+     * @var string
      */
     var $_mimeid = null;
 
@@ -150,57 +164,53 @@ class MIME_Part {
      * It is not crucial here since the PHP/PEAR mailing functions will handle
      * the EOL details.
      *
-     * @var string $_eol
+     * @var string
      */
     var $_eol = MIME_PART_EOL;
 
     /**
      * Internal class flags.
      *
-     * @var array $_flags
+     * @var array
      */
     var $_flags = array();
 
     /**
      * Part -> ID mapping cache.
      *
-     * @var array $_idmap
+     * @var array
      */
     var $_idmap = array();
 
     /**
      * Unique MIME_Part boundary string.
      *
-     * @var string $_boundary
+     * @var string
      */
     var $_boundary = null;
 
     /**
      * Default value for this Part's size.
      *
-     * @var integer $_bytes
+     * @var integer
      */
     var $_bytes = 0;
 
     /**
      * The content-ID for this part.
      *
-     * @var string $_contentid
+     * @var string
      */
     var $_contentid = null;
 
     /**
      * MIME_Part constructor.
      *
-     * @access public
-     *
-     * @param optional string $mimetype     The content type of the part.
-     * @param optional string $contents     The body of the part.
-     * @param optional string $charset      The character set of the part.
-     * @param optional string $disposition  The content disposition of the
-     *                                      part.
-     * @param optional string $encoding     The content encoding of the
-     *                                      contents.
+     * @param string $mimetype     The content type of the part.
+     * @param string $contents     The body of the part.
+     * @param string $charset      The character set of the part.
+     * @param string $disposition  The content disposition of the part.
+     * @param string $encoding     The content encoding of the contents.
      */
     function MIME_Part($mimetype = null, $contents = null,
                        $charset = MIME_DEFAULT_CHARSET, $disposition = null,
@@ -227,8 +237,6 @@ class MIME_Part {
     /**
      * Set the content-disposition of this part.
      *
-     * @access public
-     *
      * @param string $disposition  The content-disposition to set (inline or
      *                             attachment).
      */
@@ -244,8 +252,6 @@ class MIME_Part {
     /**
      * Get the content-disposition of this part.
      *
-     * @access public
-     *
      * @return string  The part's content-disposition.
      */
     function getDisposition()
@@ -256,8 +262,6 @@ class MIME_Part {
     /**
      * Set the name of this part.
      *
-     * @access public
-     *
      * @param string $name The name to set.
      */
     function setName($name)
@@ -268,12 +272,10 @@ class MIME_Part {
     /**
      * Get the name of this part.
      *
-     * @access public
-     *
-     * @param optional boolean $decode   MIME decode description?
-     * @param optional boolean $default  If the name parameter doesn't exist,
-     *                                   should we use the default name from
-     *                                   the description parameter?
+     * @param boolean $decode   MIME decode description?
+     * @param boolean $default  If the name parameter doesn't exist, should we
+     *                          use the default name from the description
+     *                          parameter?
      *
      * @return string  The name of the part.
      */
@@ -295,10 +297,8 @@ class MIME_Part {
     /**
      * Set the body contents of this part.
      *
-     * @access public
-     *
-     * @param string $contents           The part body.
-     * @param optional string $encoding  The current encoding of the contents.
+     * @param string $contents  The part body.
+     * @param string $encoding  The current encoding of the contents.
      */
     function setContents($contents, $encoding = null)
     {
@@ -310,13 +310,11 @@ class MIME_Part {
     /**
      * Add to the body contents of this part.
      *
-     * @access public
-     *
-     * @param string $contents           The contents to append to the current
-     *                                   part body.
-     * @param optional string $encoding  The current encoding of the contents.
-     *                                   If not specified, will try to auto
-     *                                   determine the encoding.
+     * @param string $contents  The contents to append to the current part
+     *                          body.
+     * @param string $encoding  The current encoding of the contents. If not
+     *                          specified, will try to auto determine the
+     *                          encoding.
      */
     function appendContents($contents, $encoding = null)
     {
@@ -325,8 +323,6 @@ class MIME_Part {
 
     /**
      * Clears the body contents of this part.
-     *
-     * @access public
      */
     function clearContents()
     {
@@ -338,8 +334,6 @@ class MIME_Part {
     /**
      * Return the body of the part.
      *
-     * @access public
-     *
      * @return string  The raw body of the part.
      */
     function getContents()
@@ -351,8 +345,6 @@ class MIME_Part {
      * Returns the contents in strict RFC 822 & 2045 output - namely, all
      * newlines end with the canonical <CR><LF> sequence.
      *
-     * @access public
-     *
      * @return string  The entire MIME part.
      */
     function getCanonicalContents()
@@ -363,8 +355,6 @@ class MIME_Part {
     /**
      * Transfer encode the contents (to the transfer encoding identified via
      * getTransferEncoding()) and set as the part's new contents.
-     *
-     * @access public
      */
     function transferEncodeContents()
     {
@@ -376,8 +366,6 @@ class MIME_Part {
 
     /**
      * Transfer decode the contents and set them as the new contents.
-     *
-     * @access public
      */
     function transferDecodeContents()
     {
@@ -397,8 +385,6 @@ class MIME_Part {
     /**
      * Set the mimetype of this part.
      *
-     * @access public
-     *
      * @param string $mimetype  The mimetype to set (ex.: text/plain).
      */
     function setType($mimetype)
@@ -434,11 +420,8 @@ class MIME_Part {
      /**
       * Get the full MIME Content-Type of this part.
       *
-      * @access public
-      *
-      * @param optional boolean $charset  Append character set information to
-      *                                   the end of the content type if this
-      *                                   is a text/* part.
+      * @param boolean $charset  Append character set information to the end of
+      *                          the content type if this is a text/* part.
       *
       * @return string  The mimetype of this part
       *                 (ex.: text/plain; charset=us-ascii).
@@ -461,8 +444,6 @@ class MIME_Part {
      * default type should be used instead (See RFC 2046).  This method
      * returns the default subtype for a particular primary MIME Type.
      *
-     * @access public
-     *
      * @return string  The default mimetype of this part (ex.: text/plain).
      */
     function getDefaultType()
@@ -486,8 +467,6 @@ class MIME_Part {
     /**
      * Get the primary type of this part.
      *
-     * @access public
-     *
      * @return string  The primary MIME type of this part.
      */
     function getPrimaryType()
@@ -498,8 +477,6 @@ class MIME_Part {
     /**
      * Get the subtype of this part.
      *
-     * @access public
-     *
      * @return string  The MIME subtype of this part.
      */
     function getSubType()
@@ -510,9 +487,7 @@ class MIME_Part {
     /**
      * Set the character set of this part.
      *
-     * @access public
-     *
-     * @param optional string $charset  The character set of this part.
+     * @param string $charset  The character set of this part.
      */
     function setCharset($charset)
     {
@@ -520,31 +495,22 @@ class MIME_Part {
     }
 
     /**
-     * Get the character set of this part. This function only will return
-     * data if this is a text/* part.
-     *
-     * @access public
+     * Get the character set to use for of this part.  Returns a charset for
+     * all types (not just 'text/*') since we use this charset to determine
+     * how to encode text in MIME headers.
      *
      * @return string  The character set of this part.  Returns null if there
      *                 is no character set.
      */
     function getCharset()
     {
-        /* If this is not a text/* part, character set isn't pertinent. */
-        if ($this->getPrimaryType() != 'text') {
-            return null;
-        }
-
         $charset = $this->getContentTypeParameter('charset');
-
         return (empty($charset)) ? null : $charset;
     }
 
     /**
      * Set the description of this part.
      *
-     * @access public
-     *
      * @param string $description  The description of this part.
      */
     function setDescription($description)
@@ -555,12 +521,10 @@ class MIME_Part {
     /**
      * Get the description of this part.
      *
-     * @access public
-     *
-     * @param optional boolean $decode   MIME decode description?
-     * @param optional boolean $default  If the name parameter doesn't exist,
-     *                                   should we use the default name from
-     *                                   the description parameter?
+     * @param boolean $decode   MIME decode description?
+     * @param boolean $default  If the name parameter doesn't exist, should we
+     *                          use the default name from the description
+     *                          parameter?
      *
      * @return string  The description of this part.
      */
@@ -585,8 +549,6 @@ class MIME_Part {
     /**
      * Set the transfer encoding to use for this part.
      *
-     * @access public
-     *
      * @param string $encoding  The transfer encoding to use.
      */
     function setTransferEncoding($encoding)
@@ -605,11 +567,9 @@ class MIME_Part {
     /**
      * Add a MIME subpart.
      *
-     * @access public
-     *
-     * @param object MIME_Part $mime_part  Add a MIME_Part subpart to the
-     *                                     current MIME_Part.
-     * @param optional string $index       The index of the added MIME_Part.
+     * @param MIME_Part $mime_part  Add a MIME_Part subpart to the current
+     *                              MIME_Part.
+     * @param string $index         The index of the added MIME_Part.
      */
     function addPart($mime_part, $index = null)
     {
@@ -642,8 +602,6 @@ class MIME_Part {
     /**
      * Get a list of all MIME subparts.
      *
-     * @access public
-     *
      * @return array  An array of the MIME_Part subparts.
      */
     function getParts()
@@ -654,12 +612,10 @@ class MIME_Part {
     /**
      * Retrieve a specific MIME part.
      *
-     * @access public
-     *
      * @param string $id  The MIME_Part ID string.
      *
-     * @return object MIME_Part  The MIME_Part requested.  Returns false if
-     *                           the part doesn't exist.
+     * @return MIME_Part  The MIME_Part requested, or false if the part
+     *                    doesn't exist.
      */
     function getPart($id)
     {
@@ -682,8 +638,6 @@ class MIME_Part {
     /**
      * Remove a MIME_Part subpart.
      *
-     * @access public
-     *
      * @param string $id  The MIME Part to delete.
      */
     function removePart($id)
@@ -697,10 +651,8 @@ class MIME_Part {
     /**
      * Alter a current MIME subpart.
      *
-     * @access public
-     *
-     * @param string $id                   The MIME Part ID to alter.
-     * @param object MIME_Part $mime_part  The MIME Part to store.
+     * @param string $id            The MIME Part ID to alter.
+     * @param MIME_Part $mime_part  The MIME Part to store.
      */
     function alterPart($id, $mime_part)
     {
@@ -715,15 +667,21 @@ class MIME_Part {
      *
      * @access private
      *
-     * @param string $id                  The MIME_Part ID string.
-     * @param array &$parts               A list of MIME_Part objects.
-     * @param optional boolean $retarray  Return a pointer to the array that
-     *                                    stores (would store) the part
-     *                                    rather than the part itself?
+     * @param string $id         The MIME_Part ID string.
+     * @param array &$parts      A list of MIME_Part objects.
+     * @param boolean $retarray  Return a pointer to the array that stores
+     *                           (would store) the part rather than the part
+     *                           itself?
      */
     function &_partFind($id, &$parts, $retarray = false)
     {
-        if (empty($this->_idmap)) {
+        /* Pointers don't persist through sessions; therefore, we must make
+         * sure that the IdMap is destroyed at the end of each request.
+         * How can we do this? We check to see if $_idmap contains an array
+         * of MIME_Parts or an array of arrays. */
+        $check = reset($this->_idmap);
+        if (empty($check) || !is_a($check, 'MIME_Part')) {
+            $this->_idmap = array();
             $this->_generateIdMap($this->_parts);
         }
 
@@ -738,7 +696,8 @@ class MIME_Part {
         if (isset($this->_idmap[$id])) {
             return $this->_idmap[$id];
         } else {
-            return false;
+            $part = false;
+            return $part;
         }
     }
 
@@ -763,8 +722,6 @@ class MIME_Part {
     /**
      * Add information about the MIME_Part.
      *
-     * @access public
-     *
      * @param string $label  The information label.
      * @param mixed $data    The information to store.
      */
@@ -776,8 +733,6 @@ class MIME_Part {
     /**
      * Retrieve information about the MIME_Part.
      *
-     * @access public
-     *
      * @param string $label  The information label.
      *
      * @return mixed  The information requested.
@@ -791,8 +746,6 @@ class MIME_Part {
     /**
      * Add a disposition parameter to this part.
      *
-     * @access public
-     *
      * @param string $label  The disposition parameter label.
      * @param string $data   The disposition parameter data.
      */
@@ -804,8 +757,6 @@ class MIME_Part {
     /**
      * Get a disposition parameter from this part.
      *
-     * @access public
-     *
      * @param string $label  The disposition parameter label.
      *
      * @return string  The data requested.
@@ -819,8 +770,6 @@ class MIME_Part {
     /**
      * Get all parameters from the Content-Disposition header.
      *
-     * @access public
-     *
      * @return array  An array of all the parameters
      *                Returns the empty array if no parameters set.
      */
@@ -832,8 +781,6 @@ class MIME_Part {
     /**
      * Add a content type parameter to this part.
      *
-     * @access public
-     *
      * @param string $label  The disposition parameter label.
      * @param string $data   The disposition parameter data.
      */
@@ -845,8 +792,6 @@ class MIME_Part {
     /**
      * Clears a content type parameter from this part.
      *
-     * @access public
-     *
      * @param string $label  The disposition parameter label.
      * @param string $data   The disposition parameter data.
      */
@@ -858,8 +803,6 @@ class MIME_Part {
     /**
      * Get a content type parameter from this part.
      *
-     * @access public
-     *
      * @param string $label  The content type parameter label.
      *
      * @return string  The data requested.
@@ -873,8 +816,6 @@ class MIME_Part {
     /**
      * Get all parameters from the Content-Type header.
      *
-     * @access public
-     *
      * @return array  An array of all the parameters
      *                Returns the empty array if no parameters set.
      */
@@ -886,8 +827,6 @@ class MIME_Part {
     /**
      * Sets a new string to use for EOLs.
      *
-     * @access public
-     *
      * @param string $eol  The string to use for EOLs.
      */
     function setEOL($eol)
@@ -898,8 +837,6 @@ class MIME_Part {
     /**
      * Get the string to use for EOLs.
      *
-     * @access public
-     *
      * @return string  The string to use for EOLs.
      */
     function getEOL()
@@ -910,10 +847,7 @@ class MIME_Part {
     /**
      * Add the appropriate MIME headers for this part to an existing array.
      *
-     * @access public
-     *
-     * @param optional array $headers  An array of any other headers for the
-     *                                 part.
+     * @param array $headers  An array of any other headers for the part.
      *
      * @return array  The headers, with the MIME headers added.
      */
@@ -985,9 +919,7 @@ class MIME_Part {
     /**
      * Return the entire part in MIME format. Includes headers on request.
      *
-     * @access public
-     *
-     * @param optional boolean $headers  Include the MIME headers?
+     * @param boolean $headers  Include the MIME headers?
      *
      * @return string  The MIME string.
      */
@@ -1044,9 +976,7 @@ class MIME_Part {
      * Returns the encoded part in strict RFC 822 & 2045 output - namely, all
      * newlines end with the canonical <CR><LF> sequence.
      *
-     * @access public
-     *
-     * @param optional boolean $headers  Include the MIME headers?
+     * @param boolean $headers  Include the MIME headers?
      *
      * @return string  The entire MIME part.
      */
@@ -1060,8 +990,6 @@ class MIME_Part {
      * Should we make sure the message is encoded via 7-bit (e.g. to adhere
      * to mail delivery standards such as RFC 2821)?
      *
-     * @access public
-     *
      * @param boolean $use7bit  Use 7-bit encoding?
      */
     function strict7bit($use7bit)
@@ -1073,8 +1001,6 @@ class MIME_Part {
      * Get the transfer encoding for the part based on the user requested
      * transfer encoding and the current contents of the part.
      *
-     * @access public
-     *
      * @return string  The transfer-encoding of this part.
      */
     function getTransferEncoding()
@@ -1132,8 +1058,6 @@ class MIME_Part {
     /**
      * Retrieves the current encoding of the contents in the object.
      *
-     * @access public
-     *
      * @return string  The current encoding.
      */
     function getCurrentEncoding()
@@ -1144,8 +1068,6 @@ class MIME_Part {
     /**
      * Encodes the contents with the part's transfer encoding.
      *
-     * @access public
-     *
      * @return string  The encoded text.
      */
     function transferEncode()
@@ -1189,8 +1111,6 @@ class MIME_Part {
     /**
      * Decodes the contents of the part to either a 7bit or 8bit encoding.
      *
-     * @access public
-     *
      * @return string  The decoded text.
      *                 Returns the empty string if there is no text to decode.
      */
@@ -1250,8 +1170,6 @@ class MIME_Part {
      * The boundary content-type parameter must be set for this function to
      * work correctly.
      *
-     * @access public
-     *
      * @return boolean  True if the contents were successfully split.
      *                  False if any error occurred.
      */
@@ -1296,11 +1214,9 @@ class MIME_Part {
      * Replace newlines in this part's contents with those specified by either
      * the given newline sequence or the part's current EOL setting.
      *
-     * @access public
-     *
-     * @param string $text          The text to replace.
-     * @param optional string $eol  The EOL sequence to use. If not present,
-     *                              uses the part's current EOL setting.
+     * @param string $text  The text to replace.
+     * @param string $eol   The EOL sequence to use. If not present, uses the
+     *                      part's current EOL setting.
      *
      * @return string  The text with the newlines replaced by the desired
      *                 newline sequence.
@@ -1319,8 +1235,6 @@ class MIME_Part {
      * since a user defined 'boundary' Content-Type parameter will override
      * this value.
      *
-     * @access public
-     *
      * @return string  The unique boundary string.
      */
     function getUniqueID()
@@ -1331,8 +1245,6 @@ class MIME_Part {
     /**
      * Determine the size of a MIME_Part and its child members.
      *
-     * @access public
-     *
      * @return integer  Size of the MIME_Part, in bytes.
      */
     function getBytes()
@@ -1368,8 +1280,6 @@ class MIME_Part {
      * object from IMAP header information without loading the data of the
      * part).
      *
-     * @access public
-     *
      * @param integer $bytes  The size of this part in bytes.
      */
     function setBytes($bytes)
@@ -1380,8 +1290,6 @@ class MIME_Part {
     /**
      * Output the size of this MIME_Part in KB.
      *
-     * @access public
-     *
      * @return string  Size of the MIME_Part, in string format.
      */
     function getSize()
@@ -1398,8 +1306,6 @@ class MIME_Part {
     /**
      * Add to the list of CIDs for this part.
      *
-     * @access public
-     *
      * @param array $cids  A list of MIME IDs of the part.
      *                     Key - MIME ID
      *                     Value - CID for the part
@@ -1412,8 +1318,6 @@ class MIME_Part {
     /**
      * Returns the list of CIDs for this part.
      *
-     * @access public
-     *
      * @return array  The list of CIDs for this part.
      */
     function getCIDList()
@@ -1425,10 +1329,8 @@ class MIME_Part {
     /**
      * Sets the Content-ID header for this part.
      *
-     * @access public
-     *
-     * @param optional string $cid  Use this CID (if not already set).  Else,
-     *                              generate a random CID.
+     * @param string $cid  Use this CID (if not already set).  Else, generate a
+     *                     random CID.
      */
     function setContentID($cid = null)
     {
@@ -1441,8 +1343,6 @@ class MIME_Part {
     /**
      * Alter the MIME ID of this part.
      *
-     * @access public
-     *
      * @param string $mimeid  The MIME ID.
      */
     function setMIMEId($mimeid)
@@ -1453,8 +1353,6 @@ class MIME_Part {
     /**
      * Returns the MIME ID of this part.
      *
-     * @access public
-     *
      * @return string  The MIME ID.
      */
     function getMIMEId()
@@ -1467,8 +1365,6 @@ class MIME_Part {
      * e.g., if the base part has MIME ID of 2, and you want the first
      * subpart of the base part, the relative MIME ID is 2.1.
      *
-     * @access public
-     *
      * @param string $id  The relative part ID.
      *
      * @return string  The relative MIME ID.
@@ -1482,8 +1378,6 @@ class MIME_Part {
     /**
      * Returns a mapping of all MIME IDs to their content-types.
      *
-     * @access public
-     *
      * @return array  KEY: MIME ID, VALUE: Content type
      */
     function contentTypeMap()
