@@ -3,7 +3,7 @@
  * The MIME_Viewer_html class renders out HTML text with an effort to
  * remove potentially malicious code.
  *
- * $Horde: framework/MIME/MIME/Viewer/html.php,v 1.14.4.6 2005/03/20 18:39:44 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer/html.php,v 1.14.4.11 2005/07/03 05:04:10 selsky Exp $
  *
  * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
@@ -15,7 +15,6 @@
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @author  Jon Parise <jon@horde.org>
  * @author  Michael Slusarz <slusarz@horde.org>
- * @version $Revision: 1.14.4.6 $
  * @since   Horde 3.0
  * @package Horde_MIME_Viewer
  */
@@ -24,9 +23,7 @@ class MIME_Viewer_html extends MIME_Viewer {
     /**
      * Render out the currently set contents.
      *
-     * @access public
-     *
-     * @param optional array $params  Any parameters the viewer may need.
+     * @param array $params  Any parameters the viewer may need.
      *
      * @return string  The rendered text.
      */
@@ -72,11 +69,8 @@ class MIME_Viewer_html extends MIME_Viewer {
             $base = $this->_cleanHTML($base);
         }
 
-        /* Removes HTML comments (including some scripts & styles) if
-         * displaying inline. */
-        if (!$attachment) {
-            $data = preg_replace('/<!--.*?-->/s', '', $data);
-        }
+        /* Removes HTML comments (including some scripts & styles). */
+        $data = preg_replace('/<!--.*?-->/s', '', $data);
 
         /* Change space entities to space characters. */
         $data = preg_replace('/&#(x0*20|0*32);?/i', ' ', $data);
@@ -160,6 +154,11 @@ class MIME_Viewer_html extends MIME_Viewer {
             '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
             '(d|&#0*68;?|&#0*44;?|&#0*100;?|&#0*64;?)\s*/i',
 
+            '/<([^>a-z]*)' .
+            '(x|&#0*88;?|&#0*58;?|&#0*120;?|&#0*78;?)\s*' .
+            '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
+            '(l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)\s*/i',
+
             '/<([^>a-z]*)' .
             '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
             '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
@@ -197,23 +196,16 @@ class MIME_Viewer_html extends MIME_Viewer {
 
         $data = preg_replace($malicious, '<HordeCleaned_tag', $data);
 
-        /* Comment out style/link tags, only if we are viewing inline.
-         * NEVER show style tags to Netscape 4.x users since 1) the
-         * output will really, really suck and 2) there might be
-         * security issues. */
-        if (!$attachment ||
-            ($GLOBALS['browser']->isBrowser('mozilla') &&
-             ($GLOBALS['browser']->getMajor() == 4))) {
-            $pattern = array('/\s+style\s*=/i',
-                             '|<style[^>]*>(?:\s*<\!--)*|i',
-                             '|(?:-->\s*)*</style>|i',
-                             '|(<link[^>]*>)|i');
-            $replace = array(' HordeCleaned=',
-                             '<!--',
-                             '-->',
-                             '<!-- $1 -->');
-            $data = preg_replace($pattern, $replace, $data);
-        }
+        /* Comment out style/link tags. */
+        $pattern = array('/\s+style\s*=/i',
+                         '|<style[^>]*>(?:\s*<\!--)*|i',
+                         '|(?:-->\s*)*</style>|i',
+                         '|(<link[^>]*>)|i');
+        $replace = array(' HordeCleaned=',
+                         '<!--',
+                         '-->',
+                         '<!-- $1 -->');
+        $data = preg_replace($pattern, $replace, $data);
 
         /* A few other matches. */
         $pattern = array('|<([^>]*)&{.*}([^>]*)>|',
@@ -248,8 +240,6 @@ class MIME_Viewer_html extends MIME_Viewer {
     /**
      * Return the content-type of the rendered text.
      *
-     * @access public
-     *
      * @return string  The MIME Content-Type.
      */
     function getType()
