@@ -1,12 +1,12 @@
 <?php
 
-require_once 'Horde/Notification/Listener.php';
+require_once 'Horde/Notification/Listener/status.php';
 
 /**
  * The Notification_Listener_mobile:: class provides functionality for
  * displaying messages from the message stack on mobile devices.
  *
- * $Horde: framework/Notification/Notification/Listener/mobile.php,v 1.8.10.1 2005/01/03 12:19:10 jan Exp $
+ * $Horde: framework/Notification/Notification/Listener/mobile.php,v 1.8.10.6 2005/07/03 05:11:48 selsky Exp $
  *
  * Copyright 2003-2005 Chuck Hagenbuch <chuck@horde.org>
  *
@@ -14,23 +14,20 @@ require_once 'Horde/Notification/Listener.php';
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.8.10.1 $
  * @since   Horde 3.0
  * @package Horde_Notification
  */
-class Notification_Listener_mobile extends Notification_Listener {
+class Notification_Listener_mobile extends Notification_Listener_status {
 
     /**
      * The Horde_Mobile:: object that status lines should be added to.
      *
-     * @var object Horde_Mobile $_mobile
+     * @var Horde_Mobile
      */
     var $_mobile = null;
 
     /**
      * Constructor
-     *
-     * @access public
      */
     function Notification_Listener_mobile()
     {
@@ -43,8 +40,7 @@ class Notification_Listener_mobile extends Notification_Listener {
     /**
      * Associate a Horde_Mobile:: object with the listener.
      *
-     * @param object Horde_Mobile  The Horde_Mobile:: object to send
-     *                             status lines to.
+     * @param Horde_Mobile  The Horde_Mobile:: object to send status lines to.
      */
     function setMobileObject(&$mobile)
     {
@@ -54,8 +50,6 @@ class Notification_Listener_mobile extends Notification_Listener {
     /**
      * Return a unique identifier for this listener.
      *
-     * @access public
-     *
      * @return string  Unique id.
      */
     function getName()
@@ -67,14 +61,16 @@ class Notification_Listener_mobile extends Notification_Listener {
      * Outputs the status line if there are any messages on the 'mobile'
      * message stack.
      *
-     * @access public
-     *
-     * @param array &$messageStack     The stack of messages.
-     * @param optional array $options  An array of options.
-     *                                 Options: 'nospace'
+     * @param array &$messageStack  The stack of messages.
+     * @param array $options        An array of options. Options: 'nospace'
      */
     function notify(&$messageStack, $options = array())
     {
+        if (!$this->_mobile) {
+            parent::Notification_Listener_status();
+            return parent::notify($messageStack, $options);
+        }
+
         if (count($messageStack)) {
             while ($message = array_shift($messageStack)) {
                 $this->getMessage($message);
@@ -87,14 +83,17 @@ class Notification_Listener_mobile extends Notification_Listener {
     /**
      * Outputs one message.
      *
-     * @access public
-     *
      * @param array $message  One message hash from the stack.
      */
     function getMessage($message)
     {
+        if (!$this->_mobile) {
+            parent::Notification_Listener_status();
+            return parent::getMessage($message);
+        }
+
         $event = $this->getEvent($message);
-        $this->_mobile->add(new Horde_Mobile_text(sprintf(_("%s: %s"), $this->_handles[$message['type']], $event->getMessage())));
+        $this->_mobile->add(new Horde_Mobile_text(sprintf(_("%s: %s"), $this->_handles[$message['type']], strip_tags($event->getMessage()))));
     }
 
 }
