@@ -6,7 +6,7 @@ require_once 'Horde/DataTree.php';
  * The Perms_datatree:: class provides a DataTree driver for the Horde
  * permissions system.
  *
- * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.2 2005/03/05 00:19:22 chuck Exp $
+ * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.7 2005/07/18 10:08:51 jan Exp $
  *
  * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2004-2005 Jan Schneider <jan@horde.org>
@@ -25,7 +25,7 @@ class Perms_datatree extends Perms {
      * Pointer to a DataTree instance to manage the different
      * permissions.
      *
-     * @var object DataTree $_datatree
+     * @var DataTree
      */
     var $_datatree;
 
@@ -93,7 +93,8 @@ class Perms_datatree extends Perms {
      */
     function &getPermissionById($cid)
     {
-        return $this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
+        $object = $this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
+        return $object;
     }
 
     /**
@@ -135,7 +136,7 @@ class Perms_datatree extends Perms {
      *                           DataTreeObject_Permission object.
      * @param string $user       The user to check for. Defaults to the current
      *                           user.
-     * @param string $creator    The user who created the event.
+     * @param string $creator    The user who created the object.
      *
      * @return mixed  A bitmask of permissions the user has, false if there
      *                are none.
@@ -145,7 +146,7 @@ class Perms_datatree extends Perms {
         if (!is_a($permission, 'DataTreeObject_Permission')) {
             $permission = &$this->getPermission($permission);
             if (is_a($permission, 'PEAR_Error')) {
-                Horde::logMessage($permission, __FILE__, __LINE__, PEAR_LOG_INFO);
+                Horde::logMessage($permission, __FILE__, __LINE__, PEAR_LOG_DEBUG);
                 return false;
             }
         }
@@ -224,7 +225,7 @@ class Perms_datatree extends Perms {
      *
      * @param string $permission  The permission to check.
      * @param string $user        The user to check for.
-     * @param int $perm           The permission level that needs to be checked
+     * @param integer $perm       The permission level that needs to be checked
      *                            for.
      * @param string $creator     The creator of the event
      *
@@ -266,16 +267,17 @@ class Perms_datatree extends Perms {
      */
     function &getTree()
     {
-        return $this->_datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
+        $tree = $this->_datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
+        return $tree;
     }
 
 }
 
 /**
- * Extension of the DataTreeObject class for storing Permission
- * information in the DataTree driver. If you want to store
- * specialized Permission information, you should extend this class
- * instead of extending DataTreeObject directly.
+ * Extension of the DataTreeObject class for storing Permission information in
+ * the DataTree driver. If you want to store specialized Permission
+ * information, you should extend this class instead of extending
+ * DataTreeObject directly.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
@@ -285,9 +287,8 @@ class Perms_datatree extends Perms {
 class DataTreeObject_Permission extends DataTreeObject {
 
     /**
-     * The DataTreeObject_Permission constructor. Just makes sure to
-     * call the parent constructor so that the perm's name is set
-     * properly.
+     * The DataTreeObject_Permission constructor. Just makes sure to call the
+     * parent constructor so that the perm's name is set properly.
      *
      * @param string $name The name of the perm.
      */
@@ -297,7 +298,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Update the permissions based on data passed in the array.
+     * Updates the permissions based on data passed in the array.
      *
      * @param array $perms  An array containing the permissions which are to be
      *                      updated.
@@ -414,12 +415,13 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Give a user additional permissions to this object.
+     * Grants a user additional permissions to this object.
      *
-     * @param string   $user        The user to grant additional permissions to.
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param string $user          The user to grant additional permissions
+     *                              to.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addUserPermission($user, $permission, $update = true)
     {
@@ -438,11 +440,11 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Grant guests additional permissions to this object.
+     * Grants guests additional permissions to this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update     (optional) Whether to automatically update the
-     *                             backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addGuestPermission($permission, $update = true)
     {
@@ -458,11 +460,11 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Grant creators additional permissions to this object.
+     * Grants creators additional permissions to this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addCreatorPermission($permission, $update = true)
     {
@@ -478,11 +480,11 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Grant additional default permissions to this object.
+     * Grants additional default permissions to this object.
      *
-     * @param integer $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean $update      (optional) Whether to automatically update the
-     *                             backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addDefaultPermission($permission, $update = true)
     {
@@ -498,12 +500,13 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Give a group additional permissions to this object.
+     * Grants a group additional permissions to this object.
      *
-     * @param integer  $groupId    The id of the group to grant additional permissions to.
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update     (optional) Whether to automatically update the
-     *                             backend. Defaults to true.
+     * @param integer $groupId      The id of the group to grant additional
+     *                              permissions to.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addGroupPermission($groupId, $permission, $update = true)
     {
@@ -523,13 +526,13 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a permission that a user currently has on this object.
+     * Removes a permission that a user currently has on this object.
      *
-     * @param string   $user         The user to remove the permission from.
-     * @param constant $permission   The permission (PERMS_DELE, etc.) to
-     *                               remove.
-     * @param optional bool $update  Whether to automatically update the
-     *                               backend. Defaults to true.
+     * @param string $user          The user to remove the permission from.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeUserPermission($user, $permission, $update = true)
     {
@@ -548,11 +551,12 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a permission that guests currently have on this object.
+     * Removes a permission that guests currently have on this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeGuestPermission($permission, $update = true)
     {
@@ -565,11 +569,12 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a permission that creators currently have on this object.
+     * Removes a permission that creators currently have on this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeCreatorPermission($permission, $update = true)
     {
@@ -582,11 +587,12 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a default permission on this object.
+     * Removes a default permission on this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeDefaultPermission($permission, $update = true)
     {
@@ -599,12 +605,14 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a permission that a group currently has on this object.
+     * Removes a permission that a group currently has on this object.
      *
-     * @param integer  $groupId     The id of the group to remove the permission from.
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param integer $groupId      The id of the group to remove the
+     *                              permission from.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeGroupPermission($groupId, $permission, $update = true)
     {
@@ -624,10 +632,10 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get an array of all user permissions on this object.
+     * Returns an array of all user permissions on this object.
      *
-     * @param optional integer $perm  List only users with this permission
-     *                                level. Defaults to all users.
+     * @param integer $perm  List only users with this permission level.
+     *                       Defaults to all users.
      *
      * @return array  All user permissions for this object, indexed by user.
      */
@@ -649,7 +657,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get the guest permissions on this object.
+     * Returns the guest permissions on this object.
      *
      * @return integer  The guest permissions on this object.
      */
@@ -661,7 +669,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get the creator permissions on this object.
+     * Returns the creator permissions on this object.
      *
      * @return integer  The creator permissions on this object.
      */
@@ -673,7 +681,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get the default permissions on this object.
+     * Returns the default permissions on this object.
      *
      * @return integer  The default permissions on this object.
      */
@@ -685,10 +693,10 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get an array of all group permissions on this object.
+     * Returns an array of all group permissions on this object.
      *
-     * @param optional integer $perm  List only users with this permission
-     *                                level. Defaults to all users.
+     * @param integer $perm  List only users with this permission level.
+     *                       Defaults to all users.
      *
      * @return array  All group permissions for this object, indexed by group.
      */
