@@ -1,16 +1,25 @@
 <?php
 
 // Preference constants (bitmasks)
-/** @constant _PREF_LOCKED Preference is administratively locked. */
+
+/**
+ * Preference is administratively locked.
+ */
 define('_PREF_LOCKED', 1);
 
-/** @constant _PREF_SHARED Preference is shared amongst applications. */
+/**
+ * Preference is shared amongst applications.
+ */
 define('_PREF_SHARED', 2);
 
-/** @constant _PREF_DIRTY Preference value has been changed. */
+/**
+ * Preference value has been changed.
+ */
 define('_PREF_DIRTY', 4);
 
-/** @constant _PREF_DEFAULT Preference value is the application default. */
+/**
+ * Preference value is the application default.
+ */
 define('_PREF_DEFAULT', 8);
 
 require_once 'Horde/String.php';
@@ -43,7 +52,7 @@ require_once 'Horde/String.php';
  *     'help'   => *Name of the entry in the XML help file*
  * );
  *
- * $Horde: framework/Prefs/Prefs.php,v 1.137.10.1 2005/01/03 12:19:10 jan Exp $
+ * $Horde: framework/Prefs/Prefs.php,v 1.137.10.11 2005/07/23 22:38:17 slusarz Exp $
  *
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
  *
@@ -51,7 +60,6 @@ require_once 'Horde/String.php';
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.137.10.1 $
  * @since   Horde 1.3
  * @package Horde_Prefs
  */
@@ -69,7 +77,7 @@ class Prefs {
      *
      * @access private
      *
-     * @var array $_prefs
+     * @var array
      */
     var $_prefs = array();
 
@@ -78,44 +86,42 @@ class Prefs {
      * maintain the application scope between sets of preferences. By
      * default, all preferences belong to the "global" (Horde) scope.
      *
-     * @var string $scope
+     * @var string
      */
     var $_scope = 'horde';
 
     /**
-     * String containing the current username. This indicates the
-     * owner of the preferences.
+     * String containing the current username. This indicates the owner of the
+     * preferences.
      *
-     * @var string $user
+     * @var string
      */
     var $_user = '';
 
     /**
      * Boolean indicating whether preference caching should be used.
      *
-     * @var boolean $caching
+     * @var boolean
      */
     var $_caching = false;
 
     /**
      * Hash holding preferences with hook functions defined.
      *
-     * @var array $_hooks
+     * @var array
      */
     var $_hooks = array();
 
     /**
      * Have we run hook functions yet?
      *
-     * @var boolean $_hooksCalled
+     * @var boolean
      */
     var $_hooksCalled = null;
 
     /**
-     * Default constructor (must be called from each extending class
-     * in their constructors via parent::Prefs()).
-     *
-     * @access public
+     * Default constructor (must be called from each extending class in their
+     * constructors via parent::Prefs()).
      */
     function Prefs()
     {
@@ -125,8 +131,6 @@ class Prefs {
     /**
      * Returns the charset used by the concrete preference backend.
      *
-     * @access public
-     *
      * @return string  The preference backend's charset.
      */
     function getCharset()
@@ -135,10 +139,8 @@ class Prefs {
     }
 
     /**
-     * Updates the session-based preferences cache (if available) with
-     * the current set of preferences.
-     *
-     * @access public
+     * Updates the session-based preferences cache (if available) with the
+     * current set of preferences.
      */
     function cacheUpdate()
     {
@@ -157,10 +159,8 @@ class Prefs {
     }
 
     /**
-     * Tries to find the requested preferences in the cache. If they
-     * exist, update the $prefs hash with the cached values.
-     *
-     * @access public
+     * Tries to find the requested preferences in the cache. If they exist,
+     * update the $prefs hash with the cached values.
      *
      * @return boolean  True on success, false on failure.
      */
@@ -194,11 +194,9 @@ class Prefs {
     /**
      * Adds a new preference entry to the $prefs hash.
      *
-     * @access public
-     *
-     * @param string $pref            The name of the preference to add.
-     * @param optional string $val    The initial value of the preference.
-     * @param optional integer $mask  The initial bitmask of the preference.
+     * @param string $pref   The name of the preference to add.
+     * @param string $val    The initial value of the preference.
+     * @param integer $mask  The initial bitmask of the preference.
      */
     function add($pref, $val = '', $mask = 0)
     {
@@ -210,8 +208,6 @@ class Prefs {
     /**
      * Removes a preference entry from the $prefs hash.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to remove.
      */
     function remove($pref)
@@ -225,8 +221,6 @@ class Prefs {
      * Sets the given preferences ($pref) to the specified value
      * ($val), if the preference is modifiable.
      *
-     * @access public
-     *
      * @param string $pref      The name of the preference to modify.
      * @param string $val       The new value for this preference.
      * @param boolean $convert  If true the preference value gets converted
@@ -311,8 +305,6 @@ class Prefs {
     /**
      * Returns the value of the requested preference.
      *
-     * @access public
-     *
      * @param string $pref      The name of the preference to retrieve.
      * @param boolean $convert  If true the preference value gets converted
      *                          from the backend's charset to the current
@@ -322,9 +314,14 @@ class Prefs {
      */
     function getValue($pref, $convert = true)
     {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
         return (isset($this->_prefs[$pref]['v'])) ?
             ($convert ?
-             $this->convertFromDriver($this->_prefs[$pref]['v'], NLS::getCharset()) :
+             $this->convertFromDriver($this->_prefs[$pref]['v'], $charset) :
              $this->_prefs[$pref]['v']) :
             null;
     }
@@ -332,8 +329,6 @@ class Prefs {
     /**
      * Modifies the "locked" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "locked" bit.
      */
@@ -345,8 +340,6 @@ class Prefs {
     /**
      * Returns the state of the "locked" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  The boolean state of $pref's "locked" bit.
@@ -359,8 +352,6 @@ class Prefs {
     /**
      * Modifies the "shared" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "shared" bit.
      */
@@ -372,8 +363,6 @@ class Prefs {
     /**
      * Returns the state of the "shared" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  The boolean state of $pref's "shared" bit.
@@ -397,8 +386,6 @@ class Prefs {
     /**
      * Returns the state of the "dirty" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  The boolean state of $pref's "dirty" bit.
@@ -411,8 +398,6 @@ class Prefs {
     /**
      * Modifies the "default" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "default" bit.
      *
@@ -426,8 +411,6 @@ class Prefs {
     /**
      * Returns the default value of the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to get the default for.
      *
      * @return string       The preference's default value.
@@ -443,8 +426,6 @@ class Prefs {
      * Determines if the current preference value is the default
      * value from prefs.php or a user defined value
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  True if the preference is the application default
@@ -497,8 +478,6 @@ class Prefs {
     /**
      * Determines whether the current preference is empty.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  True if the preference is empty.
@@ -511,8 +490,6 @@ class Prefs {
     /**
      * Returns the scope of the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to examine.
      *
      * @return string  The scope of the $pref.
@@ -582,7 +559,7 @@ class Prefs {
      * This function provides common cleanup functions for all of the driver
      * implementations.
      *
-     * @param optional boolean $all  Clean up all Horde preferences.
+     * @param boolean $all  Clean up all Horde preferences.
      */
     function cleanup($all = false)
     {
@@ -644,12 +621,10 @@ class Prefs {
     }
 
     /**
-     * Populates the $prefs hash with new entries and externally
-     * defined default values.
+     * Populates the $prefs hash with new entries and externally defined
+     * default values.
      *
      * @param string $app  The application to load defaults for.
-     *
-     * @access public
      */
     function _setDefaults($app)
     {
@@ -719,7 +694,7 @@ class Prefs {
      */
     function _callHooks()
     {
-        if (!is_null($this->_hooksCalled) && !Auth::getAuth()) {
+        if (!is_null($this->_hooksCalled)) {
             return;
         }
 
@@ -733,7 +708,8 @@ class Prefs {
         include $registry->get('fileroot', 'horde') . '/config/hooks.php';
         foreach ($this->_hooks as $pref) {
             if ($this->isLocked($pref) ||
-                !$this->getValue($pref)) {
+                !$this->getValue($pref) ||
+                $this->isDefault($pref)) {
                 $func = '_prefs_hook_' . $pref;
                 if (function_exists($func)) {
                     $this->_setValue($pref, $func($this->_user));
@@ -745,24 +721,20 @@ class Prefs {
     /**
      * Attempts to return a concrete Prefs instance based on $driver.
      *
-     * @param mixed $driver              The type of concrete Prefs subclass
-     *                                   to return. This is based on the
-     *                                   storage driver ($driver). The code is
-     *                                   dynamically included. If $driver is
-     *                                   an array, then we will look in
-     *                                   $driver[0]/lib/Prefs/ for the subclass
-     *                                   implementation named $driver[1].php.
-     * @param optional string $scope     The scope for this set of preferences.
-     * @param optional string $user      The name of the user who owns this
-     *                                   set of preferences.
-     * @param optional string $password  The password associated with $user.
-     * @param optional array $params     A hash containing any additional
-     *                                   configuration or connection
-     *                                   parameters a subclass might need.
-     * @param optional boolean $caching  Should caching be used?
+     * @param mixed $driver     The type of concrete Prefs subclass to return.
+     *                          If $driver is an array, then we will look in
+     *                          $driver[0]/lib/Prefs/ for the subclass
+     *                          implementation named $driver[1].php.
+     * @param string $scope     The scope for this set of preferences.
+     * @param string $user      The name of the user who owns this set of
+     *                          preferences.
+     * @param string $password  The password associated with $user.
+     * @param array $params     A hash containing any additional configuration
+     *                          or connection parameters a subclass might need.
+     * @param boolean $caching  Should caching be used?
      *
-     * @return object Prefs  The newly created concrete Prefs instance, or
-     *                       false on error.
+     * @return Prefs  The newly created concrete Prefs instance, or false on
+     *                error.
      */
     function &factory($driver, $scope = 'horde', $user = '', $password = '',
                       $params = null, $caching = true)
@@ -825,24 +797,19 @@ class Prefs {
      *
      * This method must be invoked as: $var = &Prefs::singleton()
      *
-     * @param mixed $driver              The type of concrete Prefs subclass
-     *                                   to return. This is based on the
-     *                                   storage driver ($driver). The code is
-     *                                   dynamically included. If $driver is
-     *                                   an array, then we will look in
-     *                                   $driver[0]/lib/Prefs/ for the subclass
-     *                                   implementation named $driver[1].php.
-     * @param optional string $scope     The scope for this set of preferences.
-     * @param optional string $user      The name of the user who owns this
-     *                                   set of preferences.
-     * @param optional string $password  The password associated with $user.
-     * @param optional array $params     A hash containing any additional
-     *                                   configuration or connection
-     *                                   parameters a subclass might need.
-     * @param optional boolean $caching  Should caching be used?
+     * @param mixed $driver     The type of concrete Prefs subclass to return.
+     *                          If $driver is an array, then we will look in
+     *                          $driver[0]/lib/Prefs/ for the subclass
+     *                          implementation named $driver[1].php.
+     * @param string $scope     The scope for this set of preferences.
+     * @param string $user      The name of the user who owns this set of
+     *                          preferences.
+     * @param string $password  The password associated with $user.
+     * @param array $params     A hash containing any additional configuration
+     *                          or connection parameters a subclass might need.
+     * @param boolean $caching  Should caching be used?
      *
-     * @return object Prefs  The concrete Prefs reference, or false on an
-     *                       error.
+     * @return Prefs  The concrete Prefs reference, or false on an error.
      */
     function &singleton($driver, $scope = 'horde', $user = '', $password = '',
                         $params = null, $caching = true)
