@@ -2,44 +2,46 @@
 /**
  * Preferences storage implementation for PHP's LDAP extention.
  *
- * Required parameters:
- * ====================
- *   'basedn'    --  The base DN for the LDAP server.
- *   'hostspec'  --  The hostname of the LDAP server.
- *   'uid'       --  The username search key.
+ * Required parameters:<pre>
+ *   'basedn'    The base DN for the LDAP server.
+ *   'hostspec'  The hostname of the LDAP server.
+ *   'uid'       The username search key.</pre>
  *
- * Optional parameters:
- * ====================
- *   'password'  --  'rootdn's password for bind authentication.
- *   'port'      --  The port of the LDAP server.
- *                   DEFAULT: 389
- *   'rootdn'    --  The DN of the root (administrative) account to bind for
- *                   write operations.
- *   'username'  --  TODO
- *   'version'   --  The version of the LDAP protocol to use.
- *                   DEFAULT: NONE (system default will be used)
+ * Optional parameters:<pre>
+ *   'password'  'rootdn's password for bind authentication.
+ *   'port'      The port of the LDAP server.
+ *               DEFAULT: 389
+ *   'rootdn'    The DN of the root (administrative) account to bind for
+ *               write operations.
+ *   'username'  TODO
+ *   'version'   The version of the LDAP protocol to use.
+ *               DEFAULT: NONE (system default will be used)</pre>
  *
- * NOTE: parameter 'username' for $params has been deprecated. Use 'rootdn'.
+ * NOTE: parameter 'username' has been deprecated. Use 'rootdn'.
  *
  *
  * If setting up as the Horde preference handler in conf.php, the following
  * is an example configuration.
  * The schemas needed for ldap are in horde/scripts/ldap.
  *
+ * <pre>
  *   $conf['prefs']['driver'] = 'ldap';
  *   $conf['prefs']['params']['hostspec'] = 'localhost';
  *   $conf['prefs']['params']['port'] = '389';
  *   $conf['prefs']['params']['basedn'] = 'dc=example,dc=org';
  *   $conf['prefs']['params']['uid'] = 'mail';
+ * </pre>
  *
  * The following is valid but would only be necessary if users
  * do NOT have permission to modify their own LDAP accounts.
  *
+ * <pre>
  *   $conf['prefs']['params']['rootdn'] = 'cn=Manager,dc=example,dc=org';
  *   $conf['prefs']['params']['password'] = 'password';
+ * </pre>
  *
  *
- * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.4 2005/03/03 23:04:35 chuck Exp $
+ * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.11 2005/09/28 10:24:06 jan Exp $
  *
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
  *
@@ -55,48 +57,45 @@ class Prefs_ldap extends Prefs {
     /**
      * Hash containing connection parameters.
      *
-     * @var array $params
+     * @var array
      */
     var $_params = array();
 
     /**
      * Handle for the current LDAP connection.
      *
-     * @var integer $connection
+     * @var resource
      */
     var $_connection;
 
     /**
      * Boolean indicating whether or not we're connected to the LDAP server.
      *
-     * @var boolean $_connected
+     * @var boolean
      */
     var $_connected = false;
 
     /**
      * String holding the user's DN.
      *
-     * @var string $dn
+     * @var string
      */
     var $_dn = '';
 
     /**
      * Constructs a new LDAP preferences object.
      *
-     * @access public
-     *
-     * @param string $user               The user who owns these preferences.
-     * @param string $password           The password associated with $user.
-     * @param string $scope              The current application scope.
-     * @param array $params              A hash containing connection
-     *                                   parameters.
-     * @param optional boolean $caching  Should caching be used?
+     * @param string $user      The user who owns these preferences.
+     * @param string $password  The password associated with $user.
+     * @param string $scope     The current application scope.
+     * @param array $params     A hash containing connection parameters.
+     * @param boolean $caching  Should caching be used?
      */
     function Prefs_ldap($user, $password, $scope = '',
                         $params = array(), $caching = false)
     {
         if (!Util::extensionExists('ldap')) {
-            Horde::fatal(PEAR::raiseError(_("Prefs_ldap: Required LDAP extension not found."), __FILE__, __LINE__));
+            Horde::fatal(PEAR::raiseError(_("Prefs_ldap: Required LDAP extension not found.")), __FILE__, __LINE__);
         }
 
         $this->_user = $user;
@@ -301,10 +300,7 @@ class Prefs_ldap extends Prefs {
     }
 
     /**
-     * Retrieves the requested set of preferences from the user's LDAP
-     * entry.
-     *
-     * @access public
+     * Retrieves the requested set of preferences from the user's LDAP entry.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -324,7 +320,7 @@ class Prefs_ldap extends Prefs {
         /* Only fetch the fields for the attributes we need. */
         $attrs = array('hordePrefs');
         if (strcmp($this->_scope, 'horde') != 0) {
-            array_push($attrs, $this->_scope . 'Prefs');
+            $attrs[] = $this->_scope . 'Prefs';
         }
 
         /* Search for the multi-valued field containing the array of
@@ -335,7 +331,7 @@ class Prefs_ldap extends Prefs {
             if ($search) {
                 $result = ldap_get_entries($this->_connection, $search);
                 if ($result && $result['count'] > 0) {
-                    continue;
+                    break;
                 }
             }
         }
@@ -414,8 +410,6 @@ class Prefs_ldap extends Prefs {
     /**
      * Stores preferences to the LDAP server.
      *
-     * @access public
-     *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
     function store()
@@ -504,8 +498,6 @@ class Prefs_ldap extends Prefs {
 
     /**
      * Clears all preferences from the LDAP backend.
-     *
-     * @access public
      */
     function clear()
     {
@@ -552,9 +544,7 @@ class Prefs_ldap extends Prefs {
     /**
      * Perform cleanup operations.
      *
-     * @access public
-     *
-     * @param optional boolean $all  Cleanup all Horde preferences.
+     * @param boolean $all  Cleanup all Horde preferences.
      */
     function cleanup($all = false)
     {
