@@ -2,24 +2,24 @@
 /**
  * SessionHandler:: implementation for MySQL (native).
  *
- * Required values for $params:<pre>
- *     'hostspec'  --  The hostname of the database server.
- *     'protocol'  --  The communication protocol ('tcp', 'unix', etc.).
- *     'username'  --  The username with which to connect to the database.
- *     'password'  --  The password associated with 'username'.
- *     'database'  --  The name of the database.
- *     'table'     --  The name of the sessiondata table in 'database'.</pre>
+ * Required parameters:<pre>
+ *   'hostspec'  The hostname of the database server.
+ *   'protocol'  The communication protocol ('tcp', 'unix', etc.).
+ *   'username'  The username with which to connect to the database.
+ *   'password'  The password associated with 'username'.
+ *   'database'  The name of the database.
+ *   'table'     The name of the sessiondata table in 'database'.</pre>
  *
- * Required for some configurations:
- *     'port'  --  The port on which to connect to the database.
+ * Required for some configurations:<pre>
+ *   'port'  The port on which to connect to the database.</pre>
  *
- * Optional parameters:
- *     'persistent'  --  Use persistent DB connections? (boolean)
+ * Optional parameters:<pre>
+ *   'persistent'  Use persistent DB connections? (boolean)</pre>
  *
- * The table structure can be created by the scripts/db/sessionhandler.sql
- * script.
+ * The table structure for the SessionHandler can be found in
+ * horde/scripts/sql/horde_sessionhandler.sql.
  *
- * $Horde: framework/SessionHandler/SessionHandler/mysql.php,v 1.16.12.2 2005/02/01 19:07:15 chuck Exp $
+ * $Horde: framework/SessionHandler/SessionHandler/mysql.php,v 1.16.12.7 2005/06/26 04:28:31 chuck Exp $
  *
  * Copyright 2002-2005 Mike Cochrane <mike@graftonhall.co.nz>
  * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
@@ -33,69 +33,59 @@
  */
 class SessionHandler_mysql extends SessionHandler {
 
-    /**
-     * Hash containing connection parameters.
-     *
-     * @var array $_params
-     */
-    var $_params = array();
-
     /**
      * Handle for the current database connection.
      *
-     * @var resource $_db
+     * @var resource
      */
     var $_db;
 
     /**
      * Are we connected to the SQL server.
      *
-     * @var boolean $_connected
+     * @var boolean
      */
     var $_connected = false;
 
     /**
-     * Constructs a new MySQL SessionHandler object.
+     * Close the SessionHandler backend.
      *
-     * @access public
-     *
-     * @param optional array $params  A hash containing connection parameters.
-     *                                See details above.
-     */
-    function SessionHandler_mysql($params = array())
-    {
-        $this->_params = $params;
-    }
-
-    /**
-     * TODO
-     */
-    function open($save_path, $session_name)
-    {
-        /* Connect to database. */
-        $this->_connect();
-    }
-
-    /**
-     * TODO
+     * @return boolean  True on success, false otherwise.
      */
     function close()
     {
         /* Disconnect from database. */
-        $this->_disconnect();
+        if ($this->_connected) {
+            $this->_connected = false;
+            return @mysql_close($this->_db);
+        }
+
+        return true;
     }
 
     /**
-     * TODO
+     * Read the data for a particular session identifier from the
+     * SessionHandler backend.
+     *
+     * @param string $id  The session identifier.
+     *
+     * @return string  The session data.
      */
     function read($id)
     {
         /* Make sure we have a valid database connection. */
         $this->_connect();
 
-        /* Session timeout, don't rely on garbage collection */
+        /* Session timeout, don't rely on garbage collection. */
         $timeout = time() - ini_get('session.gc_maxlifetime');
 
+        /* Start a transaction. */
+        $result = @mysql_query('SET AUTOCOMMIT=0', $this->_db);
+        $result = @mysql_query('BEGIN', $this->_db);
+        if (!$result) {
+            return '';
+        }
+
         $query = sprintf('SELECT session_data FROM %s WHERE session_id = %s' .
                          ' AND session_lastmodified > %s',
                          $this->_params['table'],
@@ -107,7 +97,7 @@ class SessionHandler_mysql extends SessionHandler {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         $result = @mysql_query($query, $this->_db);
-        if (!$result){
+        if (!$result) {
             Horde::logMessage('Error retrieving session data (id = ' . $id . ')',
                               __FILE__, __LINE__, PEAR_LOG_ERR);
             return '';
@@ -117,7 +107,12 @@ class SessionHandler_mysql extends SessionHandler {
     }
 
     /**
-     * TODO
+     * Write session data to the SessionHandler backend.
+     *
+     * @param string $id            The session identifier.
+     * @param string $session_data  The session data.
+     *
+     * @return boolean  True on success, false otherwise.
      */
     function write($id, $session_data)
     {
@@ -133,16 +128,26 @@ class SessionHandler_mysql extends SessionHandler {
                          time());
 
         $result = @mysql_query($query, $this->_db);
-        if (!$result){
+        if (!$result) {
+            $result = @mysql_query('ROLLBACK', $this->_db);
+            $result = @mysql_query('SET AUTOCOMMIT=1', $this->_db);
             Horde::logMessage('Error writing session data', __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
 
+        $result = @mysql_query('COMMIT', $this->_db);
+        $result = @mysql_query('SET AUTOCOMMIT=1', $this->_db);
+
         return true;
     }
 
     /**
-     * TODO
+     * Destroy the data for a particular session identifier in the
+     * SessionHandler backend.
+     *
+     * @param string $id  The session identifier.
+     *
+     * @return boolean  True on success, false otherwise.
      */
     function destroy($id)
     {
@@ -168,7 +173,11 @@ class SessionHandler_mysql extends SessionHandler {
     }
 
     /**
-     * TODO
+     * Garbage collect stale sessions from the SessionHandler backend.
+     *
+     * @param integer $maxlifetime  The maximum age of a session.
+     *
+     * @return boolean  True on success, false otherwise.
      */
     function gc($maxlifetime = 300)
     {
@@ -193,6 +202,43 @@ class SessionHandler_mysql extends SessionHandler {
         return @mysql_affected_rows($this->_db);
     }
 
+    /**
+     * Get a list of the valid session identifiers.
+     *
+     * @return array  A list of valid session identifiers.
+     */
+    function getSessionIDs()
+    {
+        /* Make sure we have a valid database connection. */
+        $this->_connect();
+
+        /* Session timeout, don't rely on garbage collection */
+        $timeout = time() - ini_get('session.gc_maxlifetime');
+
+        $query = sprintf('SELECT session_id FROM %s' .
+                         ' WHERE session_lastmodified > %s',
+                         $this->_params['table'],
+                         $timeout);
+
+        /* Log the query at a DEBUG log level. */
+        Horde::logMessage(sprintf('SQL Query by SessionHandler_mysql::getSessionIDs(): query = "%s"', $query),
+                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        $result = @mysql_query($query, $this->_db);
+        if (!$result) {
+            Horde::logMessage('Error getting session IDs',
+                              __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
+
+        $sessions = array();
+
+        while ($row = mysql_fetch_row($result))
+            $sessions[] = $row[0];
+
+        return $sessions;
+    }
+
     /**
      * Escape a mysql string.
      *
@@ -259,19 +305,4 @@ class SessionHandler_mysql extends SessionHandler {
         $this->_connected = true;
     }
 
-    /**
-     * Disconnect from the SQL server and clean up the connection.
-     *
-     * @access private
-     */
-    function _disconnect()
-    {
-        if ($this->_connected) {
-            $this->_connected = false;
-            return @mysql_close($this->_db);
-        }
-
-        return true;
-    }
-
 }
