@@ -2,7 +2,7 @@
 /**
  * The Text:: class provides common methods for manipulating text.
  *
- * $Horde: framework/Horde/Horde/Text.php,v 1.2.10.1 2005/01/03 12:19:02 jan Exp $
+ * $Horde: framework/Horde/Horde/Text.php,v 1.2.10.3 2005/05/12 14:51:58 jan Exp $
  *
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
  *
@@ -10,7 +10,6 @@
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.2.10.1 $
  * @since   Horde 1.3
  * @package Horde_Text
  */
@@ -25,7 +24,12 @@ class Text {
      */
     function htmlSpaces($text = '')
     {
-        $text = @htmlspecialchars($text, ENT_COMPAT, NLS::getCharset());
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
+        $text = @htmlspecialchars($text, ENT_COMPAT, $charset);
         $text = str_replace("\t", '&nbsp; &nbsp; &nbsp; &nbsp; ', $text);
         $text = str_replace('  ', '&nbsp; ', $text);
         $text = str_replace('  ', ' &nbsp;', $text);
