@@ -2,7 +2,7 @@
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.7 2005/03/09 11:26:25 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.14 2005/09/01 12:05:58 jan Exp $
  *
  * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
  *
@@ -18,7 +18,7 @@ class Horde_iCalendar {
     /**
      * The parent (containing) iCalendar object.
      *
-     * @var object Horde_iCalendar $_container
+     * @var Horde_iCalendar
      */
     var $_container = false;
 
@@ -27,22 +27,21 @@ class Horde_iCalendar {
      * DTSTART, etc.). Which are present depends on the object and on
      * what kind of component it is.
      *
-     * @var array $_attributes
+     * @var array
      */
     var $_attributes = array();
 
     /**
      * Any children (contained) iCalendar components of this object.
      *
-     * @var array $_components
+     * @var array
      */
     var $_components = array();
 
     /**
-     * According to RFC 2425, we should always use CRLF-terminated
-     * lines.
+     * According to RFC 2425, we should always use CRLF-terminated lines.
      *
-     * @var string $_newline
+     * @var string
      */
     var $_newline = "\r\n";
 
@@ -66,25 +65,25 @@ class Horde_iCalendar {
             if ($container !== false) {
                 $component->_container = &$container;
             }
-            return $component;
         } else {
             // Should return an dummy x-unknown type class here.
-            return false;
+            $component = false;
         }
+        return $component;
     }
 
     /**
      * Set the value of an attribute.
      *
-     * @param string  $name   The name of the attribute.
-     * @param string  $value  The value of the attribute.
-     * @param array   $params (optional) Array containing any addition
-     *                        parameters for this attribute.
-     * @param boolean $append (optional) True to append the attribute, False
-     *                        to replace the first matching attribute found.
-     * @param array $values   (optional) array representation of $value.
-     *                        For comma/semicolon seperated lists of values.
-     *                        If not set use $value as single array element.
+     * @param string $name     The name of the attribute.
+     * @param string $value    The value of the attribute.
+     * @param array $params    Array containing any addition parameters for
+     *                         this attribute.
+     * @param boolean $append  True to append the attribute, False to replace
+     *                         the first matching attribute found.
+     * @param array $values    Array representation of $value.  For
+     *                         comma/semicolon seperated lists of values.  If
+     *                         not set use $value as single array element.
      */
     function setAttribute($name, $value, $params = array(), $append = true, $values = false)
     {
@@ -117,9 +116,9 @@ class Horde_iCalendar {
      * Sets parameter(s) for an (already existing) attribute.  The
      * parameter set is merged into the existing set.
      *
-     * @param string $name    The name of the attribute.
-     * @param array  $params  Array containing any additional
-     *                        parameters for this attribute.
+     * @param string $name   The name of the attribute.
+     * @param array $params  Array containing any additional parameters for
+     *                       this attribute.
      * @return boolean  True on success, false if no attribute $name exists.
      */
     function setParameter($name, $params)
@@ -128,7 +127,7 @@ class Horde_iCalendar {
         foreach ($keys as $key) {
             if ($this->_attributes[$key]['name'] == $name) {
                 $this->_attributes[$key]['params'] =
-                    array_merge($this->_attributes[$key]['params'] , $params);
+                    array_merge($this->_attributes[$key]['params'], $params);
                 return true;
             }
         }
@@ -139,9 +138,9 @@ class Horde_iCalendar {
     /**
      * Get the value of an attribute.
      *
-     * @param string  $name    The name of the attribute.
-     * @param boolean $params  Return the parameters for this attribute
-     *                         instead of its value.
+     * @param string $name     The name of the attribute.
+     * @param boolean $params  Return the parameters for this attribute instead
+     *                         of its value.
      *
      * @return mixed (object)  PEAR_Error if the attribute does not exist.
      *               (string)  The value of the attribute.
@@ -202,9 +201,9 @@ class Horde_iCalendar {
      * Returns the value of an attribute, or a specified default value
      * if the attribute does not exist.
      *
-     * @param string $name     The name of the attribute.
-     * @param mixed  $default  (optional) What to return if the attribute
-     *                         specified by $name does not exist.
+     * @param string $name    The name of the attribute.
+     * @param mixed $default  What to return if the attribute specified by
+     *                        $name does not exist.
      *
      * @return mixed (string) The value of $name.
      *               (mixed)  $default if $name does not exist.
@@ -218,7 +217,7 @@ class Horde_iCalendar {
     /**
      * Remove all occurences of an attribute.
      *
-     * @param string  $name   The name of the attribute.
+     * @param string $name  The name of the attribute.
      */
     function removeAttribute($name)
     {
@@ -233,8 +232,9 @@ class Horde_iCalendar {
     /**
      * Get attributes for all tags or for a given tag.
      *
-     * @param string  $tag   (optional) return attributes for this tag.
-     *                       or all attributes if not given
+     * @param string $tag  Return attributes for this tag, or all attributes if
+     *                     not given.
+     *
      * @return array  An array containing all the attributes and their types.
      */
     function getAllAttributes($tag = false)
@@ -254,7 +254,7 @@ class Horde_iCalendar {
     /**
      * Add a vCalendar component (eg vEvent, vTimezone, etc.).
      *
-     * @param object Horde_iCalendar $component  Component (subclass) to add.
+     * @param Horde_iCalendar $component  Component (subclass) to add.
      */
     function addComponent($component)
     {
@@ -347,8 +347,8 @@ class Horde_iCalendar {
     }
 
     /**
-     * Clears the iCalendar object (resets the components and
-     * attributes arrays).
+     * Clears the iCalendar object (resets the components and attributes
+     * arrays).
      */
     function clear()
     {
@@ -363,7 +363,7 @@ class Horde_iCalendar {
     {
         // Default values.
         $requiredAttributes['VERSION'] = '2.0';
-        $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library' . (defined('HORDE_VERSION') ? ', Horde ' . constant('HORDE_VERSION') : '') . ' //EN';
+        $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library' . (defined('HORDE_VERSION') ? ', Horde ' . constant('HORDE_VERSION') : '') . '//EN';
         $requiredAttributes['METHOD'] = 'PUBLISH';
 
         foreach ($requiredAttributes as $name => $default_value) {
@@ -378,7 +378,7 @@ class Horde_iCalendar {
     /**
      * Export this entry as a hash array with tag names as keys.
      *
-     * @param boolean (optional) $paramsInKeys
+     * @param boolean $paramsInKeys
      *                If false, the operation can be quite lossy as the
      *                parameters are ignored when building the array keys.
      *                So if you export a vcard with
@@ -405,16 +405,18 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parse a string containing vCalendar data.
+     * Parses a string containing vCalendar data.
      *
-     * @param string  $text  The data to parse.
-     * @param string  $base  The type of the base object.
-     * @param string  $charset (optional) The encoding charset for $text. Defaults to utf-8
-     * @param boolean $clear (optional) True to clear() the iCal object before parsing.
+     * @param string $text     The data to parse.
+     * @param string $base     The type of the base object.
+     * @param string $charset  The encoding charset for $text. Defaults to
+     *                         utf-8.
+     * @param boolean $clear   If true clears the iCal object before parsing.
      *
      * @return boolean  True on successful import, false otherwise.
      */
-    function parsevCalendar($text, $base = 'VCALENDAR', $charset = 'utf-8', $clear = true)
+    function parsevCalendar($text, $base = 'VCALENDAR', $charset = 'utf-8',
+                            $clear = true)
     {
         if ($clear) {
             $this->clear();
@@ -504,10 +506,13 @@ class Horde_iCalendar {
                 case 'COMPLETED':
                 case 'CREATED':
                 case 'LAST-MODIFIED':
-                case 'BDAY':
                     $this->setAttribute($tag, $this->_parseDateTime($value), $params);
                     break;
 
+                case 'BDAY':
+                    $this->setAttribute($tag, $this->_parseDate($value), $params);
+                    break;
+
                 case 'DTEND':
                 case 'DTSTART':
                 case 'DUE':
@@ -630,7 +635,7 @@ class Horde_iCalendar {
     /**
      * Export this component in vCal format.
      *
-     * @param string $base  (optional) The type of the base object.
+     * @param string $base  The type of the base object.
      *
      * @return string  vCal format data.
      */
@@ -972,13 +977,13 @@ class Horde_iCalendar {
             $text = $parts[0];
         }
 
-        if (strlen($text) != 8) {
+        if (!preg_match('/^(\d{4})-?(\d{2})-?(\d{2})$/', $text, $match)) {
             return false;
         }
 
-        return array('year' => intval(substr($text, 0, 4)),
-                     'month' => intval(substr($text, 4, 2)),
-                     'mday' => intval(substr($text, 6, 2)));
+        return array('year' => $match[1],
+                     'month' => $match[2],
+                     'mday' => $match[3]);
     }
 
     /**
@@ -1107,9 +1112,9 @@ class Horde_iCalendar {
      *
      * Uses imap_8bit if available.
      *
-     * @param  string $input  The string to be encoded.
+     * @param string $input  The string to be encoded.
      *
-     * @return string         The quoted-printable encoded string.
+     * @return string  The quoted-printable encoded string.
      */
     function _quotedPrintableEncode($input = '')
     {
