@@ -25,7 +25,7 @@ if (!function_exists('_')) {
  * The Horde_Test:: class provides functions used in the test scripts
  * used in the various applications (test.php).
  *
- * $Horde: horde/lib/Test.php,v 1.31.4.2 2005/01/03 12:25:41 jan Exp $
+ * $Horde: horde/lib/Test.php,v 1.31.4.11.2.1 2005/10/19 23:01:17 slusarz Exp $
  *
  * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
@@ -48,44 +48,32 @@ class Horde_Test {
      * Array that holds the list of Horde applications.
      * (Loaded from config/registry.php)
      *
-     * @var array $applications
+     * @var array
      */
     var $applications = array();
 
     /**
      * Cached results of getApplications().
      *
-     * @var array $_appoutput
+     * @var array
      */
     var $_appoutput = array();
 
     /**
      * The PHP version of the system.
      *
-     * @var array $_phpver
+     * @var array
      */
     var $_phpver;
 
-    /**
-     * Location of the base Horde test script.
-     *
-     * @var string $_testscript
-     */
-    var $_testscript;
-
     /**
      * Constructor.
-     *
-     * @access public
      */
     function Horde_Test()
     {
         include_once HORDE_BASE . '/config/registry.php';
         ksort($this->applications);
 
-        /* Set the location of the base test script. */
-        $this->_testscript = $_SERVER['PHP_SELF'];
-
         /* Store the PHP version information. */
         $this->_phpver = $this->splitPHPVersion(phpversion());
 
@@ -100,8 +88,6 @@ class Horde_Test {
     /**
      * Parse PHP version.
      *
-     * @access public
-     *
      * @param string $version  A PHP-style version string (X.X.X).
      *
      * @param array  The parsed string.
@@ -159,8 +145,6 @@ class Horde_Test {
     /**
      * Check the list of PHP modules.
      *
-     * @access public
-     *
      * @param array $modlist  The module list.
      * <pre>
      * KEY:   module name
@@ -198,7 +182,7 @@ class Horde_Test {
 
             if (is_null($status_out)) {
                 $mod_test = extension_loaded($key);
-                $status_out = $this->_status($mod_test);
+                $status_out = $this->_status($mod_test, $fatal);
             }
 
             $entry[] = $descrip;
@@ -206,6 +190,9 @@ class Horde_Test {
 
             if (!is_null($error_msg) && !$mod_test) {
                 $entry[] = $error_msg;
+                if (!$fatal) {
+                    $entry[] = 1;
+                }
             }
 
             $output .= $this->_outputLine($entry);
@@ -222,8 +209,6 @@ class Horde_Test {
     /**
      * Checks the list of PHP settings.
      *
-     * @access public
-     *
      * @param array $modlist  The settings list.
      * <code>
      * KEY:   setting name
@@ -258,8 +243,6 @@ class Horde_Test {
     /**
      * Check the list of PEAR modules.
      *
-     * @access public
-     *
      * @param array $pear_list  The PEAR module list.
      * <pre>
      * KEY:   PEAR class name
@@ -329,8 +312,8 @@ class Horde_Test {
                 }
             }
             $entry[] = $key;
-            $entry[] = $this->_status($result);
-          
+            $entry[] = $this->_status($result, !empty($val['required']));
+
             if ($result) {
                 $succeeded[$key] = true;
             } else {
@@ -338,6 +321,9 @@ class Horde_Test {
                     $error_msg .= ' THIS IS A REQUIRED MODULE!';
                 }
                 $entry[] = $error_msg;
+                if (empty($val['required'])) {
+                    $entry[] = 1;
+                }
             }
 
             $output .= $this->_outputLine($entry);
@@ -352,8 +338,6 @@ class Horde_Test {
     /**
      * Check the list of required files
      *
-     * @access public
-     *
      * @param array $file_list  The file list.
      * <pre>
      * KEY:   file path
@@ -388,20 +372,22 @@ class Horde_Test {
     }
 
     /**
-     * Displays an error screen with a list of all configuration files that
-     * are missing, together with a description what they do and how they
-     * are created.
+     * Displays an error screen with a list of all configuration files
+     * that are missing, together with a description what they do and
+     * how they are created. If a file can be automatically created
+     * from the defaults, then we do that instead and don't display an
+     * error.
      *
      * @param string $app        The application name
      * @param string $appBase    The path to the application
-     * @param array $files       An array with the "standard" configuration files
+     * @param array  $files      An array with the "standard" configuration files
      *                           that should be checked. Currently supported:
      *                           - conf.php
      *                           - prefs.php
      *                           - mime_drivers.php
-     * @param array $additional  (optional) An associative array containing
-     *                           more files (as keys) and error message (as
-     *                           values) if they don't exist.
+     * @param array $additional  An associative array containing more files (as
+     *                           keys) and error message (as values) if they
+     *                           don't exist.
      */
     function configFilesMissing($app, $appBase, $files, $additional = array())
     {
@@ -410,13 +396,31 @@ class Horde_Test {
         }
         $files = array_merge($files, array_keys($additional));
 
+        /* Try to auto-create missing .dist files. */
+        $indices = array_keys($files);
+        foreach ($indices as $index) {
+            if (is_readable($appBase . '/config/' . $files[$index])) {
+                unset($files[$index]);
+            } else {
+                if (@file_exists($appBase . '/config/' . $files[$index] . '.dist') &&
+                    @copy($appBase . '/config/' . $files[$index] . '.dist', $appBase . '/config/' . $files[$index])) {
+                    unset($files[$index]);
+                }
+            }
+        }
+
+        /* Return if we have no missing files left. */
+        if (!count($files)) {
+            return;
+        }
+
         $descriptions = array_merge(array(
             'conf.php' => sprintf('This is the main %s configuration file. ' .
                                   'It contains paths and options for the %s ' .
                                   'scripts. You need to login as an ' .
                                   'administrator and create the file with ' .
                                   'the web frontend under "Administration => ' .
-                                  'Configuration".',
+                                  'Setup".',
                                   $app, $app, $appBase . '/config'),
             'prefs.php' => sprintf('This file controls the default preferences ' .
                                    'for %s, and also controls which preferences ' .
@@ -427,7 +431,7 @@ class Horde_Test {
                                           'downloadable.', $app),
             'backends.php' => sprintf('This file controls what backends are ' .
                                       'available from %s.', $app),
-            'sources.php' => sprintf('This file defines the list of  available ' .
+            'sources.php' => sprintf('This file defines the list of available ' .
                                      'sources for %s.', $app)
         ), $additional);
         $title = sprintf('%s is not properly configured', $app);
@@ -442,12 +446,10 @@ class Horde_Test {
 HEADER;
 
         foreach ($files as $file) {
-            if (!@is_readable($appBase . '/config/' . $file)) {
-                $description = $descriptions[$file];
-                echo <<< FILE
+            $description = $descriptions[$file];
+            echo <<< FILE
     <h3>$file</h3><p>$description</p>
 FILE;
-            }
         }
 
         echo <<< FOOTER
@@ -462,8 +464,6 @@ FOOTER;
     /**
      * Check the list of required Horde applications.
      *
-     * @access public
-     *
      * @param array $app_list  The application list.
      * <pre>
      * KEY:   application name
@@ -507,8 +507,6 @@ FOOTER;
     /**
      * Is this a 'recent' version of PEAR?
      *
-     * @access public
-     *
      * @param boolean  True if a recent version of PEAR.
      */
     function isRecentPEAR()
@@ -523,8 +521,6 @@ FOOTER;
     /**
      * Obtain information on the PHP version.
      *
-     * @access public
-     *
      * @return object stdClass  TODO
      */
     function getPhpVersionInformation()
@@ -533,8 +529,9 @@ FOOTER;
         $url = urlencode($_SERVER['PHP_SELF']);
         $vers_check = true;
 
-        $output->phpinfo = $this->_testscript . '?mode=phpinfo&url=' . $url;
-        $output->extensions = $this->_testscript . '?mode=extensions&url=' . $url;
+        $testscript = $this->applications['horde']['webroot'] . '/test.php';
+        $output->phpinfo = $testscript . '?mode=phpinfo&url=' . $url;
+        $output->extensions = $testscript . '?mode=extensions&url=' . $url;
         $output->version = phpversion();
         $output->major = $this->_phpver['major'];
         if (isset($this->_phpver['minor'])) {
@@ -549,12 +546,11 @@ FOOTER;
         if ($output->major < '4.3') {
             $output->status = 'This version of PHP is not supported. You need to upgrade to a more recent version.';
             $vers_check = false;
-        } elseif (($output->major == '4.3') || ($output->major == '4.4')) {
+        } elseif (($output->major == '4.3') ||
+                  ($output->major == '4.4') ||
+                  ($output->major == '5.0')) {
             $output->status = 'You are running a supported version of PHP.';
             $output->status_color = 'green';
-        } elseif ($output->major == '5.0') {
-            $output->status = 'PHP 5 is supported experimentally. You may not see any error messages, but the changed behaviour of objects in PHP 5 has unpredictable side effects.';
-            $ouput->status_color = 'orange';
         } else {
             $output->status = 'Wow, a mystical version of PHP from the future. Let <a href="mailto:dev@lists.horde.org">dev@lists.horde.org</a> know what version you have so we can fix this script.';
             $output->status_color = 'orange';
@@ -570,8 +566,6 @@ FOOTER;
     /**
      * Get the application list.
      *
-     * @access public
-     *
      * @return array  List of stdClass objects.
      *                KEY: application name
      *                ELEMENT 'version': Version of application
@@ -609,16 +603,19 @@ FOOTER;
      *
      * @access private
      *
-     * @param boolean $bool  The result of the status check.
+     * @param boolean $bool      The result of the status check.
+     * @param boolean $required  Whether the checked item is required.
      *
      * @return string  The HTML of the result of the status check.
      */
-    function _status($bool)
+    function _status($bool, $required = true)
     {
         if ($bool) {
             return '<font color="green"><b>Yes</b></font>';
-        } else {
+        } elseif ($required) {
             return '<font color="red"><b>No</b></font>';
+        } else {
+            return '<font color="orange"><b>No</b></font>';
         }
     }
 
@@ -632,6 +629,7 @@ FOOTER;
      * 1st value: Header
      * 2nd value: Test Result
      * 3rd value: Error message (if present)
+     * 4th value: Error level (if present): 0 = error, 1 = warning
      * </pre>
      *
      * @return string  HTML output.
@@ -640,7 +638,8 @@ FOOTER;
     {
         $output = '<li>' . array_shift($entry) . ': ' . array_shift($entry);
         if (!empty($entry)) {
-            $output .= '<br /><font color="red"><b>' . array_shift($entry) . '</b></font>' . "\n";
+            $msg = array_shift($entry);
+            $output .= '<br /><font color="' . (empty($entry) || !array_shift($entry) ? 'red' : 'orange') . '"><b>' . $msg . '</b></font>' . "\n";
         }
         $output .= '</li>' . "\n";
 
