@@ -8,7 +8,7 @@
  * The PHP diff code used in this package was originally written by Geoffrey
  * T. Dairiki and is used with his permission.
  *
- * $Horde: framework/Text_Diff/Diff.php,v 1.11 2004/12/07 19:26:19 chuck Exp $
+ * $Horde: framework/Text_Diff/Diff.php,v 1.11.2.3 2005/07/03 05:14:18 selsky Exp $
  *
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
@@ -18,7 +18,7 @@ class Text_Diff {
     /**
      * Array of changes.
      *
-     * @var array $_edits
+     * @var array
      */
     var $_edits;
 
@@ -97,9 +97,9 @@ class Text_Diff {
     /**
      * Computes the length of the Longest Common Subsequence (LCS).
      *
-     * This is mostly for diagnostic purposed.
+     * This is mostly for diagnostic purposes.
      *
-     * @return int  The length of the LCS.
+     * @return integer  The length of the LCS.
      */
     function lcs()
     {
@@ -157,7 +157,7 @@ class Text_Diff {
      */
     function _trimNewlines(&$line, $key)
     {
-        $line = str_replace(array("\n", "\r", "\r\n"), '', $line);
+        $line = str_replace(array("\n", "\r"), '', $line);
     }
 
     /**
@@ -196,7 +196,7 @@ class Text_Diff {
 }
 
 /**
- * $Horde: framework/Text_Diff/Diff.php,v 1.11 2004/12/07 19:26:19 chuck Exp $
+ * $Horde: framework/Text_Diff/Diff.php,v 1.11.2.3 2005/07/03 05:14:18 selsky Exp $
  *
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
@@ -252,7 +252,8 @@ class Text_MappedDiff extends Text_Diff {
  *
  * @author  Jon Parise <jon@horde.org>
  * @package Text_Diff
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Engine_xdiff {
 
@@ -307,7 +308,7 @@ class Text_Diff_Engine_xdiff {
  * More ideas are taken from:
  * http://www.ics.uci.edu/~eppstein/161/960229.html
  *
- * Some ideas (and a bit of code) are from from analyze.c, from GNU
+ * Some ideas (and a bit of code) are taken from analyze.c, of GNU
  * diffutils-2.7, which can be found at:
  * ftp://gnudist.gnu.org/pub/gnu/diffutils/diffutils-2.7.tar.gz
  *
@@ -317,7 +318,8 @@ class Text_Diff_Engine_xdiff {
  *
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
  * @package Text_Diff
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Engine_native {
 
@@ -728,7 +730,8 @@ class Text_Diff_Engine_native {
 /**
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Op {
 
@@ -755,7 +758,8 @@ class Text_Diff_Op {
 /**
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Op_copy extends Text_Diff_Op {
 
@@ -778,7 +782,8 @@ class Text_Diff_Op_copy extends Text_Diff_Op {
 /**
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Op_delete extends Text_Diff_Op {
 
@@ -798,7 +803,8 @@ class Text_Diff_Op_delete extends Text_Diff_Op {
 /**
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Op_add extends Text_Diff_Op {
 
@@ -818,7 +824,8 @@ class Text_Diff_Op_add extends Text_Diff_Op {
 /**
  * @package Text_Diff
  * @author  Geoffrey T. Dairiki <dairiki@dairiki.org>
- * @access  private
+ *
+ * @access private
  */
 class Text_Diff_Op_change extends Text_Diff_Op {
 
