@@ -2,43 +2,45 @@
 
 require_once dirname(__FILE__) . '/sql.php';
 
-/** @constant integer VFS_FLAG_READ  Permission for read access. */
+/**
+ * Permission for read access.
+ */
 define('VFS_FLAG_READ', 1);
 
-/** @constant integer VFS_FLAG_WRITE  Permission for read access. */
+/**
+ * Permission for read access.
+ */
 define('VFS_FLAG_WRITE', 2);
 
 /**
  * Multi User VFS implementation for PHP's PEAR database
  * abstraction layer.
  *
- * <pre>
- * Required values for $params:
- *      'phptype'       The database type (ie. 'pgsql', 'mysql, etc.).
+ * Required values for $params:<pre>
+ *      'phptype'       The database type (ie. 'pgsql', 'mysql', etc.).</pre>
+ *
+ * Optional values:<pre>
+ *      'table'         The name of the vfs table in 'database'. Defaults to
+ *                      'horde_muvfs'.</pre>
+ *
+ * Required by some database implementations:<pre>
  *      'hostspec'      The hostname of the database server.
  *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
+ *      'database'      The name of the database.
  *      'username'      The username with which to connect to the database.
  *      'password'      The password associated with 'username'.
- *      'database'      The name of the database.
- *
- * Optional values:
- *      'table'         The name of the vfs table in 'database'. Defaults to
- *                      'horde_muvfs'.
- *
- * Required by some database implementations:
  *      'options'       Additional options to pass to the database.
  *      'tty'           The TTY on which to connect to the database.
- *      'port'          The port on which to connect to the database.
- * </pre>
+ *      'port'          The port on which to connect to the database.</pre>
  *
  * Known Issues:
- * Delete is not recusive, so file and folders that used to be in a folder that
- * gets deleted life forever in the database, or re-appear when the folder is
- * recreated.
- * Rename has the same issue, if files are lost if a folder is renamed.
+ * Delete is not recusive, so files and folders that used to be in a folder
+ * that gets deleted live forever in the database, or re-appear when the folder
+ * is recreated.
+ * Rename has the same issue, so files are lost if a folder is renamed.
  *
  * The table structure for the VFS can be found in
- * horde/scripts/db/muvfs.sql.
+ * data/muvfs.sql.
  *
  * Database specific notes:
  *
@@ -53,7 +55,7 @@ define('VFS_FLAG_WRITE', 2);
  *    mssql.textsize = 0 ; zero to pass through
  * </code>
  *
- * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.1 2005/01/03 12:19:20 jan Exp $
+ * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.10 2005/07/03 04:52:43 selsky Exp $
  *
  * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
  *
@@ -62,7 +64,6 @@ define('VFS_FLAG_WRITE', 2);
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.30.4.1 $
  * @since   Horde 2.2
  * @package VFS
  */
@@ -71,7 +72,7 @@ class VFS_musql extends VFS_sql {
     /**
      * List of permissions and if they can be changed in this VFS
      *
-     * @var array $_permissions
+     * @var array
      */
     var $_permissions = array(
         'owner' => array('read' => false, 'write' => false, 'execute' => false),
@@ -82,12 +83,10 @@ class VFS_musql extends VFS_sql {
     /**
      * Store a file in the VFS from raw data.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param string $data                  The file data.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $data         The file data.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -118,10 +117,9 @@ class VFS_musql extends VFS_sql {
 
             foreach ($paths as $thispath) {
                 $results = $this->_db->getAll(sprintf('SELECT vfs_owner, vfs_perms FROM %s
-                                                       WHERE vfs_path = %s AND vfs_name= %s',
-                                                      $this->_params['table'],
-                                                      $this->_db->quote($previous),
-                                                      $this->_db->quote($thispath)));
+                                                       WHERE vfs_path = ? AND vfs_name= ?',
+                                                      $this->_params['table']),
+                                              array($previous, $thispath));
                 if (!is_array($results) || count($results) < 1) {
                     return PEAR::raiseError(_("Unable to create VFS file."));
                 }
@@ -148,8 +146,6 @@ class VFS_musql extends VFS_sql {
     /**
      * Delete a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      *
@@ -163,11 +159,9 @@ class VFS_musql extends VFS_sql {
         }
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_id, vfs_owner, vfs_perms FROM %s
-                                                WHERE vfs_path = %s AND vfs_name= %s AND vfs_type = %s',
-                                               $this->_params['table'],
-                                               $this->_db->quote($path),
-                                               $this->_db->quote($name),
-                                               $this->_db->quote(VFS_FILE)));
+                                                WHERE vfs_path = ? AND vfs_name= ? AND vfs_type = ?',
+                                               $this->_params['table']),
+                                       array($path, $name, VFS_FILE));
 
         if (!is_array($fileList) || count($fileList) < 1) {
             return PEAR::raiseError(_("Unable to delete VFS file."));
@@ -178,9 +172,9 @@ class VFS_musql extends VFS_sql {
            them. So we have to delete the one they have access to. */
         foreach ($fileList as $file) {
             if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
-                $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = %s',
-                                                    $this->_params['table'],
-                                                    $this->_db->quote($file[0])));
+                $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = ?',
+                                                    $this->_params['table']),
+                                            array($file[0]));
 
                 if ($this->_db->affectedRows() == 0) {
                     return PEAR::raiseError(_("Unable to delete VFS file."));
@@ -196,8 +190,6 @@ class VFS_musql extends VFS_sql {
     /**
      * Rename a file or folder in the VFS.
      *
-     * @access public
-     *
      * @param string $oldpath  The old path to the file.
      * @param string $oldname  The old filename.
      * @param string $newpath  The new path of the file.
@@ -213,27 +205,23 @@ class VFS_musql extends VFS_sql {
         }
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_id, vfs_owner, vfs_perms FROM %s
-                                                       WHERE vfs_path = %s AND vfs_name= %s',
-                                                       $this->_params['table'],
-                                                       $this->_db->quote($oldpath),
-                                                       $this->_db->quote($oldname)));
+                                                       WHERE vfs_path = ? AND vfs_name= ?',
+                                                       $this->_params['table']),
+                                       array($oldpath, $oldname));
 
         if (!is_array($fileList) || count($fileList) < 1) {
             return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
         /* There may be one or more files with the same name but the user may
-	 * not have read access to them, so doesn't see them. So we have to
-	 * rename the one they have access to. */
+         * not have read access to them, so doesn't see them. So we have to
+         * rename the one they have access to. */
         foreach ($fileList as $file) {
             if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
-                $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = %s, vfs_name = %s, vfs_modified = %s
-                                                    WHERE vfs_id = %s',
-                                                    $this->_params['table'],
-                                                    $this->_db->quote($newpath),
-                                                    $this->_db->quote($newname),
-                                                    $this->_db->quote(time()),
-                                                    $this->_db->quote($file[0])));
+                $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = ?, vfs_name = ?, vfs_modified = ?
+                                                    WHERE vfs_id = ?',
+                                                    $this->_params['table']),
+                                            array($newpath, $newname, time(), $file[0]));
                 return $result;
             }
         }
@@ -244,8 +232,6 @@ class VFS_musql extends VFS_sql {
     /**
      * Creates a folder on the VFS.
      *
-     * @access public
-     *
      * @param string $path  Holds the path of directory to create folder.
      * @param string $name  Holds the name of the new folder.
      *
@@ -265,10 +251,9 @@ class VFS_musql extends VFS_sql {
 
             foreach ($paths as $thispath) {
                 $results = $this->_db->getAll(sprintf('SELECT vfs_owner, vfs_perms FROM %s
-                                                               WHERE vfs_path = %s AND vfs_name= %s',
-                                                               $this->_params['table'],
-                                                               $this->_db->quote($previous),
-                                                               $this->_db->quote($thispath)));
+                                                               WHERE vfs_path = ? AND vfs_name= ?',
+                                                               $this->_params['table']),
+                                              array($previous, $thispath));
                 if (!is_array($results) || count($results) < 1) {
                     return PEAR::raiseError(_("Unable to create VFS directory."));
                 }
@@ -291,26 +276,17 @@ class VFS_musql extends VFS_sql {
 
         $id = $this->_db->nextId($this->_params['table']);
         return $this->_db->query(sprintf('INSERT INTO %s (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified, vfs_owner, vfs_perms)
-                                         VALUES (%s, %s, %s, %s, %s, %s, %s)',
-                                         $this->_params['table'],
-                                         $this->_db->quote($id),
-                                         $this->_db->quote(VFS_FOLDER),
-                                         $this->_db->quote($path),
-                                         $this->_db->quote($name),
-                                         $this->_db->quote(time()),
-                                         $this->_db->quote($this->_params['user']),
-                                         $this->_db->quote(0)
-                                         ));
+                                         VALUES (?, ?, ?, ?, ?, ?, ?)',
+                                         $this->_params['table']),
+                                 array($id, VFS_FOLDER, $path, $name, time(), $this->_params['user'], 0));
     }
 
     /**
      * Delete a file from the VFS.
      *
-     * @access public
-     *
-     * @param string $path                 The path to delete the folder from.
-     * @param string $name                 The foldername to use.
-     * @param optional boolean $recursive  Force a recursive delete?
+     * @param string $path        The path to delete the folder from.
+     * @param string $name        The foldername to use.
+     * @param boolean $recursive  Force a recursive delete?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -338,24 +314,22 @@ class VFS_musql extends VFS_sql {
         }
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_id, vfs_owner, vfs_perms FROM %s
-                                                       WHERE vfs_path = %s AND vfs_name= %s AND vfs_type = %s',
-                                                       $this->_params['table'],
-                                                       $this->_db->quote($path),
-                                                       $this->_db->quote($name),
-                                                       $this->_db->quote(VFS_FOLDER)));
+                                                       WHERE vfs_path = ? AND vfs_name= ? AND vfs_type = ?',
+                                                       $this->_params['table']),
+                                       array($path, $name, VFS_FOLDER));
 
         if (!is_array($fileList) || count($fileList) < 1) {
             return PEAR::raiseError(_("Unable to delete VFS directory."));
         }
 
         /* There may be one or more folders with the same name but as the user
-	 * may not have read access to them, they don't see them. So we have to
-	 * delete the one they have access to */
+         * may not have read access to them, they don't see them. So we have to
+         * delete the one they have access to */
         foreach ($fileList as $file) {
             if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
-                $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = %s',
-                                                    $this->_params['table'],
-                                                    $this->_db->quote($file[0])));
+                $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = ?',
+                                                    $this->_params['table']),
+                                            array($file[0]));
 
                 if ($this->_db->affectedRows() == 0) {
                     return PEAR::raiseError(_("Unable to delete VFS directory."));
@@ -372,11 +346,10 @@ class VFS_musql extends VFS_sql {
     /**
      * Return a list of the contents of a folder.
      *
-     * @param string $path                The path of the directory.
-     * @param optional mixed $filter      String/hash to filter file/dirname
-     *                                    on.
-     * @param optional boolean $dotfiles  Show dotfiles?
-     * @param optional boolean $dironly   Show only directories?
+     * @param string $path       The path of the directory.
+     * @param mixed $filter      String/hash to filter file/dirname on.
+     * @param boolean $dotfiles  Show dotfiles?
+     * @param boolean $dironly   Show only directories?
      *
      * @return mixed  File list on success or false on failure.
      */
@@ -392,12 +365,9 @@ class VFS_musql extends VFS_sql {
         $fileList = array();
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_name, vfs_type, vfs_modified, vfs_owner, vfs_perms, vfs_data FROM %s
-                                               WHERE vfs_path = %s AND (vfs_owner = %s or vfs_perms && %s)',
-                                               $this->_params['table'],
-                                               $this->_db->quote($path),
-                                               $this->_db->quote($this->_params['user']),
-                                               $this->_db->quote(VFS_FLAG_READ)
-                                               ));
+                                               WHERE vfs_path = ? AND (vfs_owner = ? or vfs_perms && ?)',
+                                               $this->_params['table']),
+                                       array($path, $this->_params['user'], VFS_FLAG_READ));
         if (is_a($fileList, 'PEAR_Error')) {
             return $fileList;
         }
@@ -456,8 +426,6 @@ class VFS_musql extends VFS_sql {
     /**
      * Changes permissions for an Item on the VFS.
      *
-     * @access public
-     *
      * @param string $path  Holds the path of directory of the Item.
      * @param string $name  Holds the name of the Item.
      *
@@ -476,25 +444,23 @@ class VFS_musql extends VFS_sql {
         $perm |= ($val & 2) ? VFS_FLAG_WRITE : 0;
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_id, vfs_owner, vfs_perms FROM %s
-                                                       WHERE vfs_path = %s AND vfs_name= %s',
-                                                       $this->_params['table'],
-                                                       $this->_db->quote($path),
-                                                       $this->_db->quote($name)));
+                                                       WHERE vfs_path = ? AND vfs_name= ?',
+                                                       $this->_params['table']),
+                                       array($path, $name));
 
         if (!is_array($fileList) || count($fileList) < 1) {
             return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
         /* There may be one or more files with the same name but the user may
-	 * not have read access to them, so doesn't see them. So we have to
-	 * chmod the one they have access to. */
+         * not have read access to them, so doesn't see them. So we have to
+         * chmod the one they have access to. */
         foreach ($fileList as $file) {
             if ($file[1] == $this->_params['user'] || $file[2] & VFS_FLAG_WRITE) {
-                $result = $this->_db->query(sprintf('UPDATE %s SET vfs_perms = %s
-                                                    WHERE vfs_id = %s',
-                                                    $this->_params['table'],
-                                                    $this->_db->quote($perm),
-                                                    $this->_db->quote($file[0])));
+                $result = $this->_db->query(sprintf('UPDATE %s SET vfs_perms = ?
+                                                    WHERE vfs_id = ?',
+                                                    $this->_params['table']),
+                                            array($perm, $file[0]));
                 return $result;
             }
         }
