@@ -8,7 +8,7 @@
  * See the enclosed file COPYING for license information (GPL). If you did not
  * receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * $Horde: horde/templates/javascript/tree.js,v 1.62.2.3 2005/01/05 10:33:01 jan Exp $
+ * $Horde: horde/templates/javascript/tree.js,v 1.62.2.8.2.1 2005/10/18 16:01:51 jan Exp $
  *
  * @author  Marko Djukic <marko@oblo.com>
  * @package Horde_Tree
@@ -63,7 +63,6 @@ function Horde_Tree(instanceName)
     this.node_pos       = new Array();
     this.dropline       = new Array();
     this.output         = '';
-    this.altCount       = 0;
 }
 
 Horde_Tree.prototype.setImgDir = function(imgDir)
@@ -92,8 +91,11 @@ Horde_Tree.prototype._renderTree = function()
         this.buildTree(this.rootNodes[i]);
     }
     document.getElementById(this.target).innerHTML = this.output;
-    this.altCount = 0;
     this._correctWidthForScrollbar();
+    // If using alternating row shading, work out correct shade.
+    if (this.options['alternate']) {
+        this.stripe();
+    }
 }
 
 /**
@@ -109,14 +111,7 @@ Horde_Tree.prototype._buildHeader = function()
         return '';
     }
 
-    var html = '<div';
-    /* If using alternating row shading, work out correct
-     * shade. */
-    if (this.options['alternate']) {
-        html += ' class="item' + this.altCount + '"';
-        this.altCount = 1 - this.altCount;
-    }
-    html += '>';
+    var html = '<div>';
     for (var i = 0; i < this.header.length; i++) {
         html += '<div';
         if (this.header[i]['class']) {
@@ -176,22 +171,15 @@ Horde_Tree.prototype.buildTree = function(nodeId)
 Horde_Tree.prototype.buildLine = function(nodeId)
 {
     var style = '';
-    var rowClass = '';
+    var rowClass = 'treeRow';
     if (this.nodes[nodeId]['class']) {
-        rowClass = this.nodes[nodeId]['class'];
-    }
-    if (this.options['alternate']) {
-        rowClass += ' item' + this.altCount;
-        this.altCount = 1 - this.altCount;
-    }
-    if (rowClass) {
-        rowClass = ' class="' + rowClass + '"';
+        rowClass += ' ' + this.nodes[nodeId]['class'];
     }
 
-    var line = '<div style="height:20px;clear:both"' + rowClass + '>';
+    var line = '<div class="' + rowClass + '">';
 
-    /* If we have headers, track which logical "column" we're in
-     * for any given cell of content. */
+    // If we have headers, track which logical "column" we're in for
+    // any given cell of content.
     var column = 0;
 
     if (typeof(this.nodes[nodeId]['extra']) != 'undefined' &&
@@ -236,12 +224,12 @@ Horde_Tree.prototype.buildLine = function(nodeId)
 
     for (var i = this.renderStatic ? 1 : 0; i < this.nodes[nodeId]['indent']; i++) {
         if (this.dropline[i] && this.options['lines']) {
-            line += '<img src="' + this.imgDir + '/' + this.imgLine + '" alt="|&nbsp;&nbsp;&nbsp;" height="20" width="20" style="vertical-align:middle" />';
+            line += '<img src="' + this.imgDir + '/' + this.imgLine + '" alt="|&nbsp;&nbsp;&nbsp;" height="20" width="20" />';
         } else {
-            line += '<img src="' + this.imgDir + '/' + this.imgBlank + '" alt="&nbsp;&nbsp;&nbsp;" height="20" width="20" style="vertical-align:middle" />';
+            line += '<img src="' + this.imgDir + '/' + this.imgBlank + '" alt="&nbsp;&nbsp;&nbsp;" height="20" width="20" />';
         }
     }
-    line += this._setNodeToggle(nodeId) + this._setNodeIcon(nodeId);
+    line += this._setNodeToggle(nodeId);
     if (this.options['multiline']) {
         line += '</td><td>';
     }
@@ -312,9 +300,9 @@ Horde_Tree.prototype._setLabel = function(nodeId)
             onclick = ' onclick="' + this.nodes[nodeId]['onclick'] + '"';
         }
 
-        return '<a' + urlClass + ' href="' + this.nodes[nodeId]['url'] + '"' + target + onclick + '>' + label + '</a>';
+        return '<a' + urlClass + ' href="' + this.nodes[nodeId]['url'] + '"' + target + onclick + '>' + this._setNodeIcon(nodeId) + label + '</a>';
     } else {
-        return '<span style="cursor:pointer;vertical-align:middle" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')">' + label + '</span>';
+        return '<span class="toggle" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')">' + this._setNodeIcon(nodeId) + label + '</span>';
     }
 }
 
@@ -323,36 +311,36 @@ Horde_Tree.prototype._setNodeToggle = function(nodeId)
     var attrib = '';
     if (this.nodes[nodeId]['indent'] == '0' &&
         typeof(this.nodes[nodeId]['children']) != 'undefined') {
-        /* Top level with children. */
+        // Top level with children.
         this.dropline[0] = false;
         if (this.renderStatic) {
             return '';
         } else {
-            attrib = ' style="cursor:pointer;vertical-align:middle" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')"';
+            attrib = ' style="cursor:pointer" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')"';
         }
     } else if (this.nodes[nodeId]['indent'] != '0' &&
-        typeof(this.nodes[nodeId]['children']) == 'undefined') {
-        /* Node no children. */
+               typeof(this.nodes[nodeId]['children']) == 'undefined') {
+        // Node no children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
-            /* Not last node. */
+            // Not last node.
             this.dropline[this.nodes[nodeId]['indent']] = true;
         } else {
             this.dropline[this.nodes[nodeId]['indent']] = false;
         }
     } else if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
-        /* Node with children. */
+        // Node with children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
-            /* Not last node. */
+            // Not last node.
             this.dropline[this.nodes[nodeId]['indent']] = true;
         } else {
-            /* Last node. */
+            // Last node.
             this.dropline[this.nodes[nodeId]['indent']] = false;
         }
         if (!this.renderStatic) {
-            attrib = ' style="cursor:pointer;vertical-align:middle" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')"';
+            attrib = ' style="cursor:pointer" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')"';
         }
     } else {
-        /* Top level node with no children. */
+        // Top level node with no children.
         if (this.renderStatic) {
             return '';
         }
@@ -364,7 +352,7 @@ Horde_Tree.prototype._setNodeToggle = function(nodeId)
     if (nodeToggle[1]) {
         img += 'alt="' + nodeToggle[1] + '" ';
     }
-    img += attrib + ' height="20" width="20" style="vertical-align:middle" />';
+    img += attrib + ' height="20" width="20" />';
     return img;
 }
 
@@ -373,7 +361,7 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
     var nodeToggle = new Array('', '');
     if (this.nodes[nodeId]['indent'] == '0' &&
         typeof(this.nodes[nodeId]['children']) != 'undefined') {
-        /* Top level with children. */
+        // Top level with children.
         if (this.renderStatic) {
             return nodeToggle;
         } else if (!this.options['lines']) {
@@ -381,16 +369,16 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
             nodeToggle[1] = '&nbsp;&nbsp;&nbsp;'
         } else if (this.nodes[nodeId]['expanded']) {
             nodeToggle[0] = this.imgMinusOnly;
-            nodeToggle[1] = '&#8211;';
+            nodeToggle[1] = '-';
         } else {
             nodeToggle[0] = this.imgPlusOnly;
             nodeToggle[1] = '+';
         }
     } else if (this.nodes[nodeId]['indent'] != '0' &&
         typeof(this.nodes[nodeId]['children']) == 'undefined') {
-        /* Node no children. */
+        // Node no children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
-            /* Not last node. */
+            // Not last node.
             if (this.options['lines']) {
                 nodeToggle[0] = this.imgJoin;
                 nodeToggle[1] = '|-';
@@ -399,7 +387,7 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
                 nodeToggle[1] = '&nbsp;&nbsp;&nbsp;';
             }
         } else {
-            /* Last node. */
+            // Last node.
             if (this.options['lines']) {
                 nodeToggle[0] = this.imgJoinBottom;
                 nodeToggle[1] = '`-';
@@ -409,9 +397,9 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
             }
         }
     } else if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
-        /* Node with children. */
+        // Node with children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
-            /* Not last node. */
+            // Not last node.
             if (!this.options['lines']) {
                 nodeToggle[0] = this.imgBlank;
                 nodeToggle[1] = '&nbsp;&nbsp;&nbsp;';
@@ -420,29 +408,29 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
                 nodeToggle[1] = '|-';
             } else if (this.nodes[nodeId]['expanded']) {
                 nodeToggle[0] = this.imgMinus;
-                nodeToggle[1] = '&#8211;';
+                nodeToggle[1] = '-';
             } else {
                 nodeToggle[0] = this.imgPlus;
                 nodeToggle[1] = '+';
             }
         } else {
-            /* Last node. */
+            // Last node.
             if (!this.options['lines']) {
                 nodeToggle[0] = this.imgBlank;
-                nodeToggle[1] = '&nbsp';
+                nodeToggle[1] = '&nbsp;';
             } else if (this.renderStatic) {
                 nodeToggle[0] = this.imgJoinBottom;
                 nodeToggle[1] = '`-';
             } else if (this.nodes[nodeId]['expanded']) {
                 nodeToggle[0] = this.imgMinusBottom;
-                nodeToggle[1] = '&#8211;';
+                nodeToggle[1] = '-';
             } else {
                 nodeToggle[0] = this.imgPlusBottom;
                 nodeToggle[1] = '+';
             }
         }
     } else {
-        /* Top level node with no children. */
+        // Top level node with no children.
         if (this.renderStatic) {
             return nodeToggle;
         }
@@ -468,7 +456,7 @@ Horde_Tree.prototype._setNodeIcon = function(nodeId)
     }
 
     if (typeof(this.nodes[nodeId]['icon']) != 'undefined') {
-        /* Node has a user defined icon. */
+        // Node has a user defined icon.
         if (!this.nodes[nodeId]['icon']) {
             return '';
         }
@@ -478,53 +466,96 @@ Horde_Tree.prototype._setNodeIcon = function(nodeId)
             img = this.nodes[nodeId]['icon'];
         }
     } else {
-        /* Use standard icon set. */
+        // Use standard icon set.
         if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
-            /* Node with children. */
+            // Node with children.
             img = (this.nodes[nodeId]['expanded']) ? this.imgFolderOpen
                                                    : this.imgFolder;
         } else {
-            /* Node no children. */
+            // Node no children.
             img = this.imgLeaf;
         }
     }
 
-    var imgtxt = '<img src="' + imgDir + img + '" style="vertical-align:middle" ';
+    var imgtxt = '<img src="' + imgDir + img + '"';
 
     if (typeof(this.nodes[nodeId]['iconalt']) != 'undefined') {
-        imgtxt += 'alt="' + this.nodes[nodeId]['iconalt'] + '" ';
+        imgtxt += ' alt="' + this.nodes[nodeId]['iconalt'] + '"';
     }
 
-    return imgtxt + '/> ';
+    return imgtxt + ' /> ';
 }
 
 Horde_Tree.prototype.toggle = function(nodeId)
 {
     this.nodes[nodeId]['expanded'] = !this.nodes[nodeId]['expanded'];
     if (this.nodes[nodeId]['expanded']) {
-        document.getElementById('nodeChildren_' + nodeId).style.display = 'block';
+        if (node = document.getElementById('nodeChildren_' + nodeId)) {
+            node.style.display = 'block';
+        }
     } else {
-        document.getElementById('nodeChildren_' + nodeId).style.display = 'none';
+        if (node = document.getElementById('nodeChildren_' + nodeId)) {
+            node.style.display = 'none';
+        }
+    }
+
+    // If using alternating row shading, work out correct shade.
+    if (this.options['alternate']) {
+        this.stripe();
     }
 
     nodeToggle = this._getNodeToggle(nodeId);
-    toggle = document.getElementById('nodeToggle_' + nodeId);
-    toggle.src = this.imgDir + '/' + nodeToggle[0];
-    toggle.alt = nodeToggle[1];
+    if (toggle = document.getElementById('nodeToggle_' + nodeId)) {
+        toggle.src = this.imgDir + '/' + nodeToggle[0];
+        toggle.alt = nodeToggle[1];
+    }
 
     this.saveState(nodeId, this.nodes[nodeId]['expanded'])
 }
 
+Horde_Tree.prototype.stripe = function()
+{
+    // The element to start striping.
+    var id = arguments[0] ? arguments[0] : this.target;
+
+    // The flag we'll use to keep track of whether the current row is
+    // odd or even.
+    var even = arguments[1] ? arguments[1] : false;
+
+    // Obtain a reference to the tree parent element.
+    var tree = document.getElementById(id);
+    if (!tree) {
+        return even;
+    }
+
+    // Iterate over each child div.
+    for (var i = 0; i < tree.childNodes.length; i++) {
+        if (tree.childNodes[i].id.indexOf('nodeChildren') != -1) {
+            if (this.nodes[tree.childNodes[i].id.replace('nodeChildren_', '')]['expanded']) {
+                even = this.stripe(tree.childNodes[i].id, even);
+            }
+        } else {
+            tree.childNodes[i].className = tree.childNodes[i].className.replace(' rowEven', '').replace(' rowOdd', '');
+            tree.childNodes[i].className += even ? ' rowEven' : ' rowOdd';
+
+            // Flip from odd to even, or vice-versa.
+            even = !even;
+        }
+    }
+
+    return even;
+}
+
 Horde_Tree.prototype.saveState = function(nodeId, expanded)
 {
     var newCookie = '';
     var oldCookie = this._getCookie(this._instanceName + '_expanded');
     if (expanded) {
-        /* Expand requested so add to cookie. */
+        // Expand requested so add to cookie.
         newCookie = (oldCookie) ? oldCookie + ',' : '';
         newCookie = newCookie + nodeId;
     } else {
-        /* Collapse requested so remove from cookie. */
+        // Collapse requested so remove from cookie.
         var nodes = oldCookie.split(',');
         var newNodes = new Array();
         for (var i = 0; i < nodes.length; i++) {
