@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: horde/test.php,v 1.122.6.3 2005/01/28 12:44:00 jan Exp $
+ * $Horde: horde/test.php,v 1.122.6.10.2.2 2005/10/19 17:23:28 jan Exp $
  *
  * Copyright 2002-2005 Brent J. Nordquist <bjn@horde.org>
  * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
@@ -46,7 +46,7 @@ $module_version = HORDE_VERSION;
 /* PHP module capabilities. */
 $module_list = array(
     'ctype' => 'Ctype Support',
-    'domxml' => 'DOM XML Support',
+    (version_compare(phpversion(), '5') < 0 ? 'domxml' : 'dom') => 'DOM XML Support',
     'ftp' => 'FTP Support',
     'gd' => 'GD Support',
     'gettext' => array(
@@ -58,15 +58,21 @@ $module_list = array(
     'imap' => 'IMAP Support',
     'ldap' => 'LDAP Support',
     'mbstring' => 'Mbstring Support',
-    'mcal' => 'MCAL Support',
     'mcrypt' => 'Mcrypt Support',
     'fileinfo' => array(
-        'descrip' => 'MIME Magic Support',
-        'error' => 'The fileinfo PECL module will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL extensions.'
+        'descrip' => 'MIME Magic Support (fileinfo)',
+        'error' => 'The fileinfo PECL module or the mime_magic PHP extension (see below) will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'
+    ),
+    'mime_magic' => array(
+        'descrip' => 'MIME Magic Support (mime_magic)',
+        'error' => 'The fileinfo PECL module (see above) or the mime_magic PHP extension will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'
     ),
     'mysql' => 'MySQL Support',
     'openssl' => 'OpenSSL Support',
     'pgsql' => 'PostgreSQL Support',
+    'session' => array(
+        'descrip' => 'Session Support',
+        'fatal' => true),
     'xml' => 'XML Support',
     'zlib' => array(
         'descrip' => 'Zlib Support',
@@ -86,10 +92,6 @@ $setting_list = array(
         'setting' => false,
         'error' => 'If PHP\'s internal memory limit is turned on and if not set high enough Horde will not be able to handle large data items (e.g. large mail attachments in IMP). If possible, you should disable the PHP memory limit by recompiling PHP <i>without</i> the "--enable-memory-limit" flag. If this is not possible, then you should set the value of memory_limit in php.ini to a sufficiently high value (Current value of memory_limit: ' . ini_get('memory_limit') . ').'
     ),
-    'file_uploads' => array(
-        'setting' => true,
-        'error' => 'file_uploads must be enabled for some features like sending emails with IMP.'
-    ),
     'safe_mode' => array(
         'setting' => false,
         'error' => 'If safe_mode is enabled, Horde cannot set enviroment variables, which means Horde will be unable to translate the user interface into different languages.'
@@ -107,9 +109,9 @@ $setting_list = array(
 
 /* PEAR */
 $pear_list = array(
-    'Mail_RFC822' => array(
+    'Mail' => array(
         'path' => 'Mail/RFC822.php',
-        'error' => 'Make sure you are using a recent version of PEAR which includes the Mail_RFC822 class.'
+        'error' => 'You do not have the Mail package installed on your system. See the INSTALL file for instructions on how to install the package.'
     ),
     'Mail_Mime' => array(
         'path' => 'Mail/mimeDecode.php',
@@ -141,7 +143,7 @@ $pear_list = array(
     ),
     'HTTP_Request' => array(
         'path' => 'HTTP/Request.php',
-        'error' => 'Parts of Horde (Jonah, the XML-RPC client/server) use the HTTP_Request library to retrieve URLs and do other HTTP requests.'
+        'error' => 'Parts of Horde (HTML composition in IMP, Jonah, the XML-RPC client/server) use the HTTP_Request library to retrieve URLs and do other HTTP requests.'
     ),
     'File' => array(
         'path' => 'File/CSV.php',
@@ -154,6 +156,10 @@ $pear_list = array(
     'Services_Weather' => array(
         'path' => 'Services/Weather.php',
         'error' => 'Services_Weather is used by the weather applet/block on the portal page.'
+    ),
+    'XML_Serializer' => array(
+        'path' => 'XML/Serializer.php',
+        'error' => 'XML_Serializer is used by Services_Weather module on the weather applet/block on the portal page.'
     )
 );
 
@@ -303,6 +309,16 @@ require TEST_TEMPLATES . 'php_version.inc';
     <?php echo $setting_output ?>
 </ul>
 
+<h1>File Uploads</h1>
+<ul>
+    <?php echo $horde_test->phpSettingCheck(array('file_uploads' => array('setting' => true, 'error' => 'file_uploads must be enabled for some features like sending emails with IMP.' ))) ?>
+<?php if ($dir = ini_get('upload_tmp_dir')): ?>
+    <li>upload_tmp_dir: <font color="<?php echo is_writable($dir) ? 'green' : 'red' ?>"><b><?php echo $dir ?></b></font></li>
+<?php endif; ?>
+    <li>upload_max_filesize: <?php echo ini_get('upload_max_filesize') ?></li>
+    <li>post_max_size: <?php echo ini_get('post_max_size') ?></li>
+</ul>
+
 <h1>Required Horde Configuration Files</h1>
 <ul>
     <?php echo $config_output ?>
